@@ -1,7 +1,7 @@
 /*
  * NSS boilerplate stuff, for libreswan.
  *
- * Copyright (C) 2016, Andrew Cagney <cagney@gnu.org>
+ * Copyright (C) 2016,2020 Andrew Cagney <cagney@gnu.org>
  * Copyright (C) 2018 Sahana Prasad <sahana.prasad07@gmail.com>
  *
  * This program is free software; you can redistribute it and/or modify it
@@ -19,27 +19,45 @@
 #define CKAID_H
 
 #include <stdbool.h>		/* for bool */
+#include <stdint.h>		/* for uint8_t */
 #include <secitem.h>		/* for SECItem */
+#include <stddef.h>		/* for size_t */
 
 #include "err.h"
 #include "chunk.h"
 
+struct jambuf;
+
 /*
  * For rationale behind *_t? Blame chunk_t.
  *
- * XXX: Can ckaid be changed to bytes[]+length?  The length is
- * determined by the hash algorithm(s) used to generate the ckaid?
+ * Field names are so that it is chunk_t like.
+ *
+ * Assume SHA1 is being used for the CKAID
  */
+#define CKAID_SIZE BYTES_FOR_BITS(160)
+
 typedef struct {
-	SECItem *nss;
+	size_t len;
+	uint8_t ptr[CKAID_SIZE];
 } ckaid_t;
 
-bool ckaid_starts_with(ckaid_t ckaid, const char *start);
-char *ckaid_as_string(ckaid_t ckaid);
+bool ckaid_starts_with(const ckaid_t *ckaid, const char *start);
+
+err_t string_to_ckaid(const char *string, ckaid_t *ckaid);
+
+ /* raw bytes in lower-case hex */
+typedef struct {
+	char buf[CKAID_SIZE * 2 + 1/*nul*/ + 1/*canary*/];
+} ckaid_buf;
+const char *str_ckaid(const ckaid_t *ckaid, ckaid_buf *buf);
+size_t jam_ckaid(struct jambuf *buf, const ckaid_t *ckaid);
+
 err_t form_ckaid_rsa(chunk_t modulus, ckaid_t *ckaid);
 err_t form_ckaid_ecdsa(chunk_t pub_value, ckaid_t *ckaid);
-err_t form_ckaid_nss(const SECItem *const nss_ckaid, ckaid_t *ckaid);
-void freeanyckaid(ckaid_t *ckaid);
-void DBG_log_ckaid(const char *prefix, ckaid_t ckaid);
+bool ckaid_eq_nss(const ckaid_t *l, const SECItem *r);
+
+ckaid_t ckaid_from_secitem(const SECItem *const nss_ckaid);
+SECItem same_ckaid_as_secitem(const ckaid_t *ciaid);
 
 #endif
