@@ -24,7 +24,7 @@
 #include <stdint.h>		/* for intmax_t */
 #include <stdbool.h>		/* for bool */
 
-struct lswlog;
+struct jambuf;
 
 /*
  * XXX: This value isn't typed so what is it really the max of?
@@ -48,15 +48,19 @@ struct lswlog;
  * Sigh.
  */
 
-typedef struct { intmax_t ms; } deltatime_t;
+typedef struct { struct timeval dt; } deltatime_t;
 
-#define DELTATIME_INIT(S) { (intmax_t)((S) * 1000) }
+#define DELTATIME_INIT(S) { .dt = { .tv_sec = (S), } }
 
 deltatime_t deltatime(time_t secs);
 deltatime_t deltatime_ms(intmax_t ms);
 
-/* sign(a - b) */
-int deltatime_cmp(deltatime_t a, deltatime_t b);
+/* for monotime(a-b) and realtime(a-b) */
+deltatime_t deltatime_timevals_diff(struct timeval l, struct timeval r);
+
+/* sign(a - b); see timercmp() for hacks origin */
+int deltatime_cmp_sign(deltatime_t a, deltatime_t b);
+#define deltatime_cmp(A, OP, B) (deltatime_cmp_sign(A, B) OP 0)
 
 /* max(a, b) */
 deltatime_t deltatime_max(deltatime_t a, deltatime_t b);
@@ -64,6 +68,9 @@ deltatime_t deltatime_max(deltatime_t a, deltatime_t b);
 /* a+b */
 deltatime_t deltatime_add(deltatime_t a, deltatime_t b);
 
+/* a-b */
+deltatime_t deltatime_sub(deltatime_t a, deltatime_t b);
+
 /* a*s */
 deltatime_t deltatime_mulu(deltatime_t a, unsigned scalar);
 
@@ -73,24 +80,19 @@ deltatime_t deltatime_divu(deltatime_t a, unsigned scalar);
 intmax_t deltamillisecs(deltatime_t d);
 intmax_t deltasecs(deltatime_t d);
 deltatime_t deltatimescale(int num, int denom, deltatime_t d);
-bool deltaless(deltatime_t a, deltatime_t b);
-bool deltaless_tv_dt(const struct timeval a, const deltatime_t b);
 
-/* Convert to struct timeval. */
-struct timeval deltatimeval(deltatime_t);
+/* Convert to/from struct timeval - time used by libevent. */
+struct timeval timeval_from_deltatime(deltatime_t);
+deltatime_t deltatime_from_timeval(const struct timeval a);
 
 /* output as "smart" seconds */
 
 typedef struct {
-	char buf[100]; /* true length ???? */
+	/* slightly over size */
+	char buf[sizeof("-18446744073709551615.1000000")+1/*canary*/]; /* true length ???? */
 } deltatime_buf;
+
 const char *str_deltatime(deltatime_t d, deltatime_buf *buf);
-
-/* fmt_deltatime() */
-size_t lswlog_deltatime(struct lswlog *buf, deltatime_t d);
-
-/* But what about -ve? */
-#define PRI_DELTATIME "%jd.%03jd"
-#define pri_deltatime(D) (deltamillisecs(D) / 1000), (deltamillisecs(D) % 1000)
+size_t jam_deltatime(struct jambuf *buf, deltatime_t d);
 
 #endif
