@@ -2,20 +2,23 @@
 # define _DNSSEC_H
 
 #include <stdbool.h>
-#include <event2/event.h>
-#include <unbound.h>
 
 #include "ip_address.h"
+#include "diag.h"
+
+struct logger;
+struct event_base;
 
 extern void unbound_ctx_free(void);
 extern void unbound_sync_init(bool do_dnssec, const char *rootfile,
-			 const char *trusted);
+			      const char *trusted, struct logger *logger);
 
-extern bool unbound_event_init(struct event_base *eb, bool do_dnssec,
-		const char *rootfile, const char *trusted);
+extern diag_t unbound_event_init(struct event_base *eb, bool do_dnssec,
+				 const char *rootfile, const char *trusted,
+				 struct logger *logger);
 
 extern bool unbound_resolve(char *src, size_t srclen, int af,
-				ip_address *ipaddr);
+			    ip_address *ipaddr, struct logger *logger);
 
 extern struct ub_ctx *get_unbound_ctx(void);
 
