@@ -13,7 +13,7 @@ struct ip_info {
 	 * ip_address
 	 */
 	unsigned ip_version; /* 4 or 6 */
-	const char *ip_name;
+	const char *ip_name; /* "IPv4" or "IPv6" */
 	size_t ip_size; /* 4 or 16 */
 	/* 0.0.0.0 or :: */
 	const ip_address any_address;
@@ -29,7 +29,7 @@ struct ip_info {
 	/*
 	 * ip_subnet.
 	 */
-	int mask_cnt; /* 32 or 128 */
+	unsigned mask_cnt; /* 32 or 128 */
 	/* unspecified address - ::/128 or 0.0.0.0/32 - matches no addresses */
 	const ip_subnet no_addresses;
 	/* default route - ::/0 or 0.0.0.0/0 - matches all addresses */
@@ -52,8 +52,11 @@ struct ip_info {
 	/*
 	 * ID stuff.
 	 */
-	uint8_t id_addr, id_subnet, id_range;
+	enum ike_id_type id_ip_addr;
+	enum ike_id_type id_ip_addr_subnet;
+	enum ike_id_type id_ip_addr_range;
 
+	size_t (*jam_address)(struct jambuf *buf, const struct ip_info *info, const struct ip_bytes *bytes);
 };
 
 extern const struct ip_info ipv4_info;
