@@ -21,9 +21,10 @@
 #include "err.h"
 #include "ip_endpoint.h"
 #include "libreswan.h"		/* for ipsec_spi_t */
-#include "jambuf.h"
 #include "ip_protocol.h"
 
+struct jambuf;
+
 /*
  * to identify an SA, we need
  */
@@ -47,7 +48,7 @@ typedef struct {
 	 * This is in network order (but is manipulated like an int.
 	 *
 	 * XXX: Does this mean it is the SPI that the remote end
-	 * expects to see on its incomming packets?
+	 * expects to see on its incoming packets?
 	 */
 #               define  SPI_PASS        256     /* magic values... */
 #               define  SPI_DROP        257     /* ...for use... */
@@ -63,12 +64,6 @@ typedef struct {
 	 * Don't confuse this with the IP version of the above
 	 * address.
 	 */
-#define SA_ICMP &ip_protocol_unspec
-#define SA_IPIP &ip_protocol_ipip
-#define SA_ESP &ip_protocol_esp
-#define SA_AH &ip_protocol_ah
-#define SA_COMP &ip_protocol_comp
-#define SA_INT &ip_protocol_int
 	const struct ip_protocol *proto;
 
 } ip_said;
@@ -88,8 +83,8 @@ typedef struct {
 	char buf[5 + ULTOT_BUF + 1 + sizeof(address_buf)];
 } said_buf;
 
-void jam_said(jambuf_t *buf, const ip_said *said, int format);
-const char *str_said(const ip_said *said, int format, said_buf *buf);
+void jam_said(struct jambuf *buf, const ip_said *said);
+const char *str_said(const ip_said *said, said_buf *buf);
 
 /*
  * Details.
@@ -104,5 +99,6 @@ ip_address said_address(const ip_said *said);
 
 extern err_t ttosa(const char *src, size_t srclen, ip_said *dst);
 #define SATOT_BUF       sizeof(said_buf)
+#define SAMIGTOT_BUF    (16 + SATOT_BUF + ADDRTOT_BUF)
 
 #endif
