@@ -69,15 +69,24 @@ typedef ip_subnet ip_selector;
 void pexpect_selector(const ip_selector *s, const char *t, where_t where);
 #define pselector(S) pexpect_selector(S, #S, HERE)
 
-ip_selector selector_from_address(const ip_address *address,
-				  const ip_protoport *protoport);
-ip_selector selector_from_subnet(const ip_subnet *subnet,
-				 const ip_protoport *protoport);
+ip_selector selector_from_address(const ip_address *address);
+ip_selector selector_from_address_protocol(const ip_address *address,
+					   const ip_protocol *protocol);
+ip_selector selector_from_address_protocol_port(const ip_address *address,
+						const ip_protocol *protocol,
+						ip_port port);
+
 ip_selector selector_from_endpoint(const ip_endpoint *address);
-#if 0
-ip_selector selector_from_range(const ip_range *range,
-				const ip_protoport *protoport);
-#endif
+
+ip_selector selector_from_subnet_protocol_port(const ip_subnet *subnet,
+					       const ip_protocol *protocol,
+					       ip_port port);
+
+ip_selector selector_from_address_protoport(const ip_address *address,
+					    const ip_protoport *protoport);
+ip_selector selector_from_subnet_protoport(const ip_subnet *subnet,
+					   const ip_protoport *protoport);
+
 err_t range_to_selector(const ip_range *range,
 			const ip_protoport *protoport,
 			ip_selector *selector);
@@ -110,8 +119,10 @@ ip_protoport selector_protoport(const ip_selector *selector);
 
 ip_port selector_port(const ip_selector *selector);
 
+/* hacks */
 int selector_hport(const ip_selector *selector);
-void update_selector_hport(ip_selector *selector, unsigned hport);
+#define update_selector_hport(SELECTOR, HPORT) { (SELECTOR)->addr.hport = (HPORT); }
+#define update_selector_ipproto(SELECTOR, IPPROTO) { (SELECTOR)->addr.ipproto = (IPPROTO); }
 
 
 /* assuming a subnet like XFRM does */
@@ -133,7 +144,21 @@ bool selector_eq(const ip_selector *l, const ip_selector *r);
 typedef struct {
 	char buf[sizeof(address_buf) + 4/*"/NNN"*/ + 6/*:65535*/];
 } selector_buf;
+
 const char *str_selector(const ip_selector *selector, selector_buf *out);
-void jam_selector(struct jambuf *buf, const ip_selector *selector);
+size_t jam_selector(struct jambuf *buf, const ip_selector *selector);
+
+const char *str_selector_sensitive(const ip_selector *selector, selector_buf *out);
+size_t jam_selector_sensitive(struct jambuf *buf, const ip_selector *selector);
+
+typedef struct {
+	char buf[sizeof(selector_buf) + sizeof("=UNKNOWN=UNKNOWN=>") + sizeof(selector_buf)];
+} selectors_buf;
+
+const char *str_selectors(const ip_selector *src, const ip_selector *dst, selectors_buf *out);
+size_t jam_selectors(struct jambuf *buf, const ip_selector *src, const ip_selector *dst);
+
+const char *str_selectors_sensitive(const ip_selector *src, const ip_selector *dst, selectors_buf *out);
+size_t jam_selectors_sensitive(struct jambuf *buf, const ip_selector *src, const ip_selector *dst);
 
 #endif
