@@ -18,13 +18,10 @@
  *
  */
 
-#ifndef _LSW_CONF_H
-#define _LSW_CONF_H
+#ifndef LSW_CONF_H
+#define LSW_CONF_H
 
-#include "constants.h"
-
-# include <nss.h>
-# include <pk11pub.h>	/* from nss3 devel */
+struct logger;
 
 struct lsw_conf_options {
 	char *rootdir;			/* default is "" --- used for testing */
@@ -46,10 +43,11 @@ const struct lsw_conf_options *lsw_init_options(void);
 void lsw_conf_free_oco(void);
 void lsw_conf_rootdir(const char *root_dir);
 void lsw_conf_secretsfile(const char *secretsfile);
-void lsw_conf_confddir(const char *confddir);
-void lsw_conf_nssdir(const char *nssdir);
+void lsw_conf_confddir(const char *confddir, struct logger *logger);
+void lsw_conf_nssdir(const char *nssdir, struct logger *logger);
 void lsw_conf_nsspassword(const char *nsspassword);
 
-extern int libreswan_selinux(void);
+extern int libreswan_selinux(struct logger *logger);
+
+#endif
 
-#endif /* _LSW_ALLOC_H_ */
