@@ -4,32 +4,28 @@
 #include <sys/types.h>
 #include <libreswan.h>
 
+extern enum_names ip_protocol_id_names;	/* aka ipproto_*; in ip_protocol.c */
+
 extern enum_names kern_interface_names;
 extern enum_names timer_event_names;
 
 extern enum_names dpd_action_names;
-#ifdef NETKEY_SUPPORT
+#ifdef XFRM_SUPPORT
 extern enum_names netkey_sa_dir_names;
 #endif
 extern enum_names sd_action_names;
 extern enum_names stf_status_names;
-extern enum_names ikev2_asym_auth_name;
+extern enum_names keyword_authby_names;
 
 extern const enum_names debug_names;
 extern const enum_names debug_help;
 extern const struct lmod_info debug_lmod_info;
 
-extern const enum_names impair_names;
-extern const enum_names impair_help;
-extern const struct lmod_info impair_lmod_info;
-
 extern enum_names connection_kind_names;
 extern enum_names routing_story;
 extern enum_names certpolicy_type_names;
-extern const char *const sa_policy_bit_names[];
-extern const char *const sighash_policy_bit_names[];
 extern enum_names oakley_attr_names;
-extern const char *const oakley_attr_bit_names[];
+extern enum_names oakley_attr_bit_names;
 extern enum_names *const oakley_attr_val_descs[];
 extern const unsigned int oakley_attr_val_descs_roof;
 extern enum_names ipsec_attr_names;
@@ -53,7 +49,6 @@ extern enum_enum_names payload_type_names;
 extern enum_names ikev2_last_proposal_desc;
 extern enum_names ikev2_last_transform_desc;
 extern enum_names payload_names_ikev1orv2;
-extern const char *const payload_name_ikev1[];	/* suitable for bitnamesof() */
 extern enum_names attr_msg_type_names;
 extern enum_names modecfg_attr_names;
 extern enum_names xauth_type_names;
@@ -63,13 +58,10 @@ extern enum_names ikev2_exchange_names;
 extern enum_names exchange_names_ikev1orv2;
 extern enum_enum_names exchange_type_names;
 extern enum_names ikev1_protocol_names;
-extern enum_names ikev2_protocol_names;
-extern enum_names ikev2_del_protocol_names;	/* subset of protocol names accepted by IKEv2 Delete */
 extern enum_names isakmp_transformid_names;
 extern enum_names ah_transformid_names;
 extern enum_names esp_transformid_names;
 extern enum_names ipcomp_transformid_names;
-extern enum_names notify_hash_algo_names;
 extern enum_names ike_idtype_names_extended0;
 extern enum_names ike_idtype_names_extended;
 extern enum_names ike_idtype_names;
@@ -82,7 +74,10 @@ extern enum_names ikev1_notify_names;
 
 /* IKEv2 */
 extern enum_names ikev2_auth_names;
-extern enum_names ikev2_sec_proto_id_names;
+extern enum_names ikev2_hash_algorithm_names;
+extern enum_names ikev2_proposal_protocol_id_names;	/* 1=IKE SA, 2=AH, 3=ESP */
+extern enum_names ikev2_delete_protocol_id_names;	/* 1=IKE SA, 2=AH, 3=ESP */
+extern enum_names ikev2_notify_protocol_id_names;	/* NONE=0, 1=IKE, 2=AH, 3=ESP */
 extern enum_names ikev2_trans_type_names;
 extern enum_names ikev2_trans_type_encr_names;
 extern enum_names ikev2_trans_type_prf_names;
@@ -113,7 +108,6 @@ extern enum_names dns_auth_level_names;
  */
 extern uint16_t secctx_attr_type;
 
-extern const char *const natt_bit_names[];
 extern enum_names natt_method_names;
 
 extern enum_names pkk_names;
