@@ -112,9 +112,9 @@ enum keyword_xauthby {
 };
 
 enum allow_global_redirect {
-	GLOBAL_REDIRECT_NO,
-	GLOBAL_REDIRECT_YES,
-	GLOBAL_REDIRECT_AUTO,
+	GLOBAL_REDIRECT_NO = 1,
+	GLOBAL_REDIRECT_YES = 2,
+	GLOBAL_REDIRECT_AUTO = 3,
 };
 
 enum keyword_xauthfail {
@@ -817,15 +817,21 @@ enum tcp_options {
        IKE_TCP_FALLBACK = 3,
 };
 
-/* Policies for establishing an SA
+/*
+ * Policies for establishing an SA
  *
- * These are used to specify attributes (eg. encryption) and techniques
- * (eg PFS) for an SA.
- * Note: certain CD_ definitions in whack.c parallel these -- keep them
- * in sync!
+ * These are used to specify attributes (eg. encryption) and
+ * techniques (eg PFS) for an SA.
+ *
+ * Note: certain CD_ definitions in whack.c parallel these -- keep
+ * them in sync!
  */
 
-extern const char *prettypolicy(lset_t policy);
+typedef struct {
+	char buf[512];/*arbitrary*/
+} policy_buf;
+const char *str_policy(lset_t policy, policy_buf *buf);
+size_t jam_policy(struct jambuf *buf, lset_t policy);
 
 /*
  * ISAKMP policy elements.
@@ -914,14 +920,6 @@ enum sa_policy_bits {
 	POLICY_AGGRESSIVE_IX,	/* do we do aggressive mode? */
 	POLICY_OVERLAPIP_IX,	/* can two conns that have subnet=vhost: declare the same IP? */
 
-	/*
-	 * this is mapped by parser's ikev2={four_state}. It is a bit richer
-	 * in that we can actually turn off everything, but it expands more
-	 * sensibly to an IKEv3 and other methods.
-	 */
-	POLICY_IKEV1_ALLOW_IX,	/* !accept IKEv1?  0x0100 0000 */
-	POLICY_IKEV2_ALLOW_IX,	/* accept IKEv2?   0x0200 0000 */
-
 	POLICY_IKEV2_ALLOW_NARROWING_IX,	/* Allow RFC-5669 section 2.9? 0x0800 0000 */
 	POLICY_IKEV2_PAM_AUTHORIZE_IX,
 	POLICY_SEND_REDIRECT_ALWAYS_IX,		/* next three policies are for RFC 5685 */
@@ -977,8 +975,6 @@ enum sa_policy_bits {
 #define POLICY_MODECFG_PULL	LELEM(POLICY_MODECFG_PULL_IX)	/* is modecfg pulled by client? */
 #define POLICY_AGGRESSIVE	LELEM(POLICY_AGGRESSIVE_IX)	/* do we do aggressive mode? */
 #define POLICY_OVERLAPIP	LELEM(POLICY_OVERLAPIP_IX)	/* can two conns that have subnet=vhost: declare the same IP? */
-#define POLICY_IKEV1_ALLOW	LELEM(POLICY_IKEV1_ALLOW_IX)	/* !accept IKEv1?  0x0100 0000 */
-#define POLICY_IKEV2_ALLOW	LELEM(POLICY_IKEV2_ALLOW_IX)	/* accept IKEv2?   0x0200 0000 */
 #define POLICY_IKEV2_ALLOW_NARROWING	LELEM(POLICY_IKEV2_ALLOW_NARROWING_IX)	/* Allow RFC-5669 section 2.9? 0x0800 0000 */
 #define POLICY_IKEV2_PAM_AUTHORIZE	LELEM(POLICY_IKEV2_PAM_AUTHORIZE_IX)    /* non-standard, custom PAM authorize call on ID */
 #define POLICY_SEND_REDIRECT_ALWAYS	LELEM(POLICY_SEND_REDIRECT_ALWAYS_IX)
@@ -996,6 +992,16 @@ enum sa_policy_bits {
 #define POLICY_IGNORE_PEER_DNS	LELEM(POLICY_IGNORE_PEER_DNS_IX)
 #define POLICY_RSASIG_v1_5	LELEM(POLICY_RSASIG_v1_5_IX)
 
+/*
+ * XXX: Danger!
+ *
+ * As well as IKEv2_HASH_ALGORITHM_* and the the lset_t constants
+ * NEGOTIATE_AUTH_HASH_*, pluto defines the enum POL_SIGHASH_*_IX and
+ * lset_t constants POL_SIGHHASH_* using different values.  Ulgh.
+ *
+ * sighash_policy_bit_names is for the _latter.
+ */
+
 #define NEGOTIATE_AUTH_HASH_SHA1		LELEM(IKEv2_HASH_ALGORITHM_SHA1)	/* rfc7427 does responder support SHA1? */
 #define NEGOTIATE_AUTH_HASH_SHA2_256		LELEM(IKEv2_HASH_ALGORITHM_SHA2_256)	/* rfc7427 does responder support SHA2-256?  */
 #define NEGOTIATE_AUTH_HASH_SHA2_384		LELEM(IKEv2_HASH_ALGORITHM_SHA2_384)	/* rfc7427 does responder support SHA2-384? */
@@ -1007,6 +1013,9 @@ enum sighash_policy_bits {
 	POL_SIGHASH_SHA2_384_IX,
 	POL_SIGHASH_SHA2_512_IX,
 };
+
+extern const struct enum_names sighash_policy_bit_names;
+
 #define POL_SIGHASH_SHA2_256 LELEM(POL_SIGHASH_SHA2_256_IX)
 #define POL_SIGHASH_SHA2_384 LELEM(POL_SIGHASH_SHA2_384_IX)
 #define POL_SIGHASH_SHA2_512 LELEM(POL_SIGHASH_SHA2_512_IX)
