@@ -66,7 +66,7 @@
  */
 
 #define WHACK_BASIC_MAGIC (((((('w' << 8) + 'h') << 8) + 'k') << 8) + 25)
-#define WHACK_MAGIC (((((('o' << 8) + 'h') << 8) + 'k') << 8) + 48)
+#define WHACK_MAGIC (((((('o' << 8) + 'h') << 8) + 'k') << 8) + 49)
 
 /* struct whack_end is a lot like connection.h's struct end
  * It differs because it is going to be shipped down a socket
@@ -171,6 +171,7 @@ struct whack_message {
 	bool whack_connection;
 	bool whack_async;
 
+	enum ike_version ike_version;
 	lset_t policy;
 	lset_t sighash_policy;
 	deltatime_t sa_ike_life_seconds;
@@ -278,8 +279,13 @@ struct whack_message {
 
 	/* for WHACK_OPINITIATE */
 	bool whack_oppo_initiate;
-	ip_address oppo_my_client, oppo_peer_client;
-	int oppo_proto, oppo_dport;
+	struct {
+		struct {
+			ip_address address;
+			ip_port port;
+		} local, remote;
+		unsigned ipproto;
+	} oppo;
 
 	/* for WHACK_TERMINATE: */
 	bool whack_terminate;
@@ -349,6 +355,8 @@ struct whack_message {
 	bool vti_shared; /* use remote %any and skip cleanup on down? */
 
 	/* for RFC 5685 - IKEv2 Redirect mechanism */
+	enum allow_global_redirect global_redirect;
+	char *global_redirect_to;
 	char *redirect_to;
 	char *accept_redirect_to;
 
