@@ -30,22 +30,26 @@
 #include "ip_endpoint.h"
 #include "ip_address.h"
 #include "socketwrapper.h"
-#include "libreswan/ipsec_tunnel.h"
-#include "libreswan/passert.h"
+//#include "libreswan/ipsec_tunnel.h"
+#include "passert.h"
 #include "ipsecconf/interfaces.h"
 #include "ipsecconf/exec.h"
 #include "ipsecconf/starterlog.h"
 #include "lswlog.h"	/* for pexpect() */
+#include "ip_info.h"
+#include "ip_protocol.h"
+#include "ip_sockaddr.h"
 
-bool starter_iface_find(const char *iface, int af, ip_address *dst, ip_address *nh)
+bool starter_iface_find(const char *iface, const struct ip_info *family,
+			ip_address *dst, ip_address *nh)
 {
-	/* XXX: danger REQ is recycled by ioctl() callss */
+	/* XXX: danger REQ is recycled by ioctl() calls */
 	struct ifreq req;
 
 	if (iface == NULL)
 		return false;	/* ??? can this ever happen? */
 
-	int sock = safe_socket(af, SOCK_DGRAM, 0);
+	int sock = safe_socket(family->af, SOCK_DGRAM, 0);
 	if (sock < 0)
 		return false;
 
@@ -66,12 +70,13 @@ bool starter_iface_find(const char *iface, int af, ip_address *dst, ip_address *
 	/* get NH */
 	if ((req.ifr_flags & IFF_POINTOPOINT) != 0x0 && nh != NULL &&
 	    (ioctl(sock, SIOCGIFDSTADDR, &req) == 0)) {
-		const ip_sockaddr *sa = (const ip_sockaddr *)&req.ifr_addr;
-		passert(&sa->sa == &req.ifr_addr);
-		if (sa->sa.sa_family == af) {
-			/* XXX: sizeof right? */
+		if (req.ifr_addr.sa_family == family->af) {
+			const ip_sockaddr sa = {
+				.len = family->sockaddr_size,
+				.sa.sa = req.ifr_addr,
+			};
 			ip_endpoint nhe;
-			happy(sockaddr_to_endpoint(sa, sizeof(*sa), &nhe));
+			happy(sockaddr_to_endpoint(&ip_protocol_unset, &sa, &nhe));
 			pexpect(endpoint_hport(&nhe) == 0);
 			*nh = endpoint_address(&nhe);
 		}
@@ -79,12 +84,13 @@ bool starter_iface_find(const char *iface, int af, ip_address *dst, ip_address *
 
 	/* get DST */
 	if (dst != NULL && ioctl(sock, SIOCGIFADDR, &req) == 0) {
-		const ip_sockaddr *sa = (const ip_sockaddr *)&req.ifr_addr;
-		passert(&sa->sa == &req.ifr_addr);
-		if (sa->sa.sa_family == af) {
-			/* XXX: sizeof right? */
+		if (req.ifr_addr.sa_family == family->af) {
+			const ip_sockaddr sa = {
+				.len = family->sockaddr_size,
+				.sa.sa = req.ifr_addr,
+			};
 			ip_endpoint dste;
-			happy(sockaddr_to_endpoint(sa, sizeof(*sa), &dste));
+			happy(sockaddr_to_endpoint(&ip_protocol_unset, &sa, &dste));
 			pexpect(endpoint_hport(&dste) == 0);
 			*dst = endpoint_address(&dste);
 		}
