@@ -23,23 +23,6 @@
 
 const ip_address unset_address; /* all zeros */
 
-ip_address strip_address(const ip_address *in, where_t where UNUSED)
-{
-	if (in->hport != 0 || in->ipproto != 0 || in->is_endpoint) {
-		address_buf b;
-		dbg("stripping address %s of is_endpoint=%d hport=%u ipproto=%u "PRI_WHERE,
-		    str_address(in, &b), in->is_endpoint,
-		    in->hport, in->ipproto, pri_where(where));
-	}
-	ip_address out = {
-		.is_address = true,
-		.version = in->version,
-		.bytes = in->bytes,
-	};
-	paddress(&out);
-	return out;
-}
-
 ip_address address_from_raw(const struct ip_info *afi, const struct ip_bytes *bytes)
 {
 	ip_address a = {
@@ -132,22 +115,15 @@ chunk_t address_as_chunk(ip_address *address)
 static size_t format_address_cooked(struct jambuf *buf, bool sensitive,
 				    const ip_address *address)
 {
-	/*
-	 * A NULL address can't be sensitive.
-	 */
-	if (address == NULL) {
-		return jam(buf, "<none>");
+	if (address_is_unset(address)) {
+		return jam_string(buf, "<unset-address>");
 	}
 
 	if (sensitive) {
-		return jam(buf, "<ip-address>");
+		return jam_string(buf, "<address>");
 	}
 
 	const struct ip_info *afi = address_type(address);
-	if (afi == NULL) {
-		return jam(buf, "<invalid>");
-	}
-
 	return afi->jam_address(buf, afi, &address->bytes);
 }
 
@@ -377,16 +353,15 @@ ip_address address_from_blit(const struct ip_info *afi,
 
 void pexpect_address(const ip_address *a, const char *s, where_t where)
 {
-	if (a != NULL && a->version != 0) {
-		/* i.e., is-set */
-		if (a->is_address == false ||
-		    a->is_endpoint == true ||
-		    a->hport != 0 ||
-		    a->ipproto != 0) {
-			address_buf b;
-			dbg("EXPECTATION FAILED: %s is not an address; "PRI_ADDRESS" "PRI_WHERE,
-			    s, pri_address(a, &b),
-			    pri_where(where));
-		}
+	if (address_is_unset(a)) {
+		return;
+	}
+
+	if (a->version == 0 ||
+	    a->is_address == false) {
+		address_buf b;
+		dbg("EXPECTATION FAILED: %s is not an address; "PRI_ADDRESS" "PRI_WHERE,
+		    s, pri_address(a, &b),
+		    pri_where(where));
 	}
 }
