@@ -23,28 +23,14 @@
 
 const ip_endpoint unset_endpoint; /* all zeros */
 
-ip_address strip_endpoint(const ip_endpoint *in, where_t where UNUSED)
-{
-	if (in->hport != 0 || in->ipproto != 0 || in->is_endpoint) {
-		endpoint_buf b;
-		dbg("stripping endpoint %s of is_endpoint=%d hport=%u ipproto=%u "PRI_WHERE,
-		    str_endpoint(in, &b), in->is_endpoint,
-		    in->hport, in->ipproto, pri_where(where));
-	}
-	ip_address out = {
-		.version = in->version,
-		.bytes = in->bytes,
-	};
-	return out;
-}
-
-static ip_endpoint raw_endpoint(const struct ip_protocol *protocol,
-				const ip_address *address, int hport)
+ip_endpoint endpoint_from_address_protocol_port(const ip_address *address,
+						const struct ip_protocol *protocol,
+						ip_port port)
 {
 	ip_endpoint endpoint = {
 		.version = address->version,
 		.bytes = address->bytes,
-		.hport = hport,
+		.hport = port.hport,
 		.ipproto = protocol->ipproto,
 		.is_endpoint = true,
 	};
@@ -57,15 +43,7 @@ static ip_endpoint raw_endpoint(const struct ip_protocol *protocol,
 ip_endpoint endpoint3(const struct ip_protocol *protocol,
 		      const ip_address *address, ip_port port)
 {
-#if 0
-	padddress(address);
-#endif
-	return raw_endpoint(protocol, address, hport(port));
-}
-
-ip_endpoint endpoint(const ip_address *address, int hport)
-{
-	return raw_endpoint(&ip_protocol_unset, address, hport);
+	return endpoint_from_address_protocol_port(address, protocol, port);
 }
 
 ip_address endpoint_address(const ip_endpoint *endpoint)
@@ -139,6 +117,9 @@ bool endpoint_is_unset(const ip_endpoint *endpoint)
 
 const struct ip_protocol *endpoint_protocol(const ip_endpoint *endpoint)
 {
+	if (endpoint_is_unset(endpoint)) {
+		return NULL;
+	}
 	return protocol_by_ipproto(endpoint->ipproto);
 }
 
@@ -171,29 +152,22 @@ bool endpoint_is_specified(const ip_endpoint *endpoint)
  * cannot be used, while for UDP, the source port is optional
  * and a value of zero means no port.
  */
+
 static size_t format_endpoint(struct jambuf *buf, bool sensitive,
-			    const ip_endpoint *endpoint)
+			      const ip_endpoint *endpoint)
 {
 	/*
-	 * A NULL endpoint can't be sensitive so always log it.
+	 * A NULL/unset endpoint can't be sensitive so always log it.
 	 */
-	if (endpoint == NULL) {
-		return jam(buf, "<none:>");
-	}
-
-	/*
-	 * An endpoint with no type (i.e., uninitialized) can't be
-	 * sensitive so always log it.
-	 */
-	const struct ip_info *afi = endpoint_type(endpoint);
-	if (afi == NULL) {
-		return jam(buf, "<unspecified:>");
+	if (endpoint_is_unset(endpoint)) {
+		return jam_string(buf, "<unset-endpoint>");
 	}
 
 	if (sensitive) {
-		return jam(buf, "<address:>");
+		return jam_string(buf, "<endpoint>");
 	}
 
+	const struct ip_info *afi = endpoint_type(endpoint);
 	ip_address address = endpoint_address(endpoint);
 	int hport = endpoint_hport(endpoint);
 	size_t s = 0;
@@ -201,19 +175,13 @@ static size_t format_endpoint(struct jambuf *buf, bool sensitive,
 	switch (afi->af) {
 	case AF_INET: /* N.N.N.N[:PORT] */
 		s += jam_address(buf, &address);
-		if (hport > 0) {
-			s += jam(buf, ":%d", hport);
-		}
+		s += jam(buf, ":%d", hport);
 		break;
 	case AF_INET6: /* [N:..:N]:PORT or N:..:N */
-		if (hport > 0) {
-			s += jam(buf, "[");
-			s += jam_address(buf, &address);
-			s += jam(buf, "]");
-			s += jam(buf, ":%d", hport);
-		} else {
-			s += jam_address(buf, &address);
-		}
+		s += jam(buf, "[");
+		s += jam_address(buf, &address);
+		s += jam(buf, "]");
+		s += jam(buf, ":%d", hport);
 		break;
 	default:
 		bad_case(afi->af);
@@ -223,7 +191,7 @@ static size_t format_endpoint(struct jambuf *buf, bool sensitive,
 
 size_t jam_endpoint(struct jambuf *buf, const ip_endpoint *endpoint)
 {
-	return format_endpoint(buf, false, endpoint);
+	return format_endpoint(buf, /*sensitive?*/false, endpoint);
 }
 
 const char *str_endpoint(const ip_endpoint *endpoint, endpoint_buf *dst)
@@ -235,7 +203,7 @@ const char *str_endpoint(const ip_endpoint *endpoint, endpoint_buf *dst)
 
 size_t jam_sensitive_endpoint(struct jambuf *buf, const ip_endpoint *endpoint)
 {
-	return format_endpoint(buf, !log_ip, endpoint);
+	return format_endpoint(buf, /*sensitive?*/!log_ip, endpoint);
 }
 
 const char *str_sensitive_endpoint(const ip_endpoint *endpoint, endpoint_buf *dst)
@@ -245,6 +213,26 @@ const char *str_sensitive_endpoint(const ip_endpoint *endpoint, endpoint_buf *ds
 	return dst->buf;
 }
 
+size_t jam_endpoints(struct jambuf *buf, const ip_endpoint *src, const ip_endpoint *dst)
+{
+	const ip_protocol *srcp = endpoint_protocol(src);
+	const ip_protocol *dstp = endpoint_protocol(dst);
+	size_t s = 0;
+	s += format_endpoint(buf, /*sensitive?*/false, src);
+	s += jam_char(buf, ' ');
+	s += jam_protocols(buf, srcp, '-', dstp);
+	s += jam_char(buf, ' ');
+	s += format_endpoint(buf, /*sensitive?*/false, dst);
+	return s;
+}
+
+const char *str_endpoints(const ip_endpoint *src, const ip_endpoint *dst, endpoints_buf *out)
+{
+	struct jambuf buf = ARRAY_AS_JAMBUF(out->buf);
+	jam_endpoints(&buf, src, dst);
+	return out->buf;
+}
+
 bool endpoint_eq(const ip_endpoint *l, const ip_endpoint *r)
 {
 	const struct ip_info *lt = endpoint_type(l);
@@ -295,16 +283,20 @@ bool endpoint_address_eq(const ip_endpoint *endpoint, const ip_address *address)
 
 void pexpect_endpoint(const ip_endpoint *e, const char *s, where_t where)
 {
-	if (e != NULL && e->version != 0) {
-		/* i.e., is set */
-		if (e->is_endpoint == false ||
-		    e->is_address == true ||
-		    e->hport == 0 ||
-		    e->ipproto == 0) {
-			endpoint_buf b;
-			dbg("EXPECTATION FAILED: %s is not an endpoint; "PRI_ENDPOINT" "PRI_WHERE,
-			    s, pri_endpoint(e, &b),
-			    pri_where(where));
-		}
+	if (endpoint_is_unset(e)) {
+		/* NULL or zero */
+		return;
+	}
+
+	const ip_protocol *protocol = endpoint_protocol(e);
+	if (e->version == 0 ||
+	    e->is_endpoint == false ||
+	    e->ipproto == 0 ||
+	    protocol == NULL ||
+	    (protocol->endpoint_requires_non_zero_port && e->hport == 0)) {
+		endpoint_buf b;
+		dbg("EXPECTATION FAILED: %s is not an endpoint; "PRI_ENDPOINT" "PRI_WHERE,
+		    s, pri_endpoint(e, &b),
+		    pri_where(where));
 	}
 }
