@@ -29,19 +29,20 @@ bool selector_is_unset(const ip_selector *selector)
 	return thingeq(*selector, unset_selector);
 }
 
-void jam_selector(struct jambuf *buf, const ip_selector *selector)
+size_t jam_selector(struct jambuf *buf, const ip_selector *selector)
 {
-	if (selector == NULL) {
-		jam(buf, "<none>/0");
-		return;
+	size_t s = 0;
+	if (selector_is_unset(selector)) {
+		return jam_string(buf, "<unset-selector>");
 	}
 	ip_address sa = selector_prefix(selector);
-	jam_address(buf, &sa); /* sensitive? */
-	jam(buf, "/%u", selector->maskbits);
+	s += jam_address(buf, &sa); /* sensitive? */
+	s += jam(buf, "/%u", selector->maskbits);
 	int port = selector_hport(selector);
 	if (port >= 0) {
-		jam(buf, ":%d", port);
+		s += jam(buf, ":%d", port);
 	}
+	return s;
 }
 
 const char *str_selector(const ip_selector *selector, selector_buf *out)
@@ -51,36 +52,95 @@ const char *str_selector(const ip_selector *selector, selector_buf *out)
 	return out->buf;
 }
 
-ip_selector selector_from_address(const ip_address *address,
-				  const ip_protoport *protoport)
+size_t jam_selector_sensitive(struct jambuf *buf, const ip_selector *selector)
 {
-	const struct ip_info *afi = address_type(address);
-	if (!pexpect(afi != NULL)) {
+	if (!log_ip) {
+		return jam_string(buf, "<selector>");
+	}
+	return jam_selector(buf, selector);
+}
+
+const char *str_selector_sensitive(const ip_selector *selector, selector_buf *out)
+{
+	struct jambuf buf = ARRAY_AS_JAMBUF(out->buf);
+	jam_selector_sensitive(&buf, selector);
+	return out->buf;
+}
+
+size_t jam_selectors(struct jambuf *buf, const ip_selector *src, const ip_selector *dst)
+{
+	const ip_protocol *srcp = selector_protocol(src);
+	const ip_protocol *dstp = selector_protocol(dst);
+	size_t s = 0;
+	s += jam_selector(buf, src);
+	s += jam_char(buf, ' ');
+	s += jam_protocols(buf, srcp, '-', dstp);
+	s += jam_char(buf, ' ');
+	s += jam_selector(buf, dst);
+	return s;
+}
+
+const char *str_selectors(const ip_selector *src, const ip_selector *dst, selectors_buf *out)
+{
+	struct jambuf buf = ARRAY_AS_JAMBUF(out->buf);
+	jam_selectors(&buf, src, dst);
+	return out->buf;
+}
+
+size_t jam_selectors_sensitive(struct jambuf *buf, const ip_selector *src, const ip_selector *dst)
+{
+	if(!log_ip) {
+		return jam_string(buf, "<selector> -> <selector>");
+	}
+	return jam_selectors(buf, src, dst);
+}
+
+const char *str_selectors_sensitive(const ip_selector *src, const ip_selector *dst, selectors_buf *out)
+{
+	struct jambuf buf = ARRAY_AS_JAMBUF(out->buf);
+	jam_selectors_sensitive(&buf, src, dst);
+	return out->buf;
+}
+
+ip_selector selector_from_address(const ip_address *address)
+{
+	return selector_from_address_protocol_port(address, &ip_protocol_unset, unset_port);
+}
+
+ip_selector selector_from_address_protocol(const ip_address *address,
+					   const ip_protocol *protocol)
+{
+	return selector_from_address_protocol_port(address, protocol, unset_port);
+}
+
+ip_selector selector_from_address_protocol_port(const ip_address *address,
+						const ip_protocol *protocol,
+						ip_port port)
+{
+	if (address_is_unset(address)) {
 		return unset_selector;
 	}
 	ip_subnet subnet = subnet_from_address(address);
-	return selector_from_subnet(&subnet, protoport);
+	return selector_from_subnet_protocol_port(&subnet, protocol, port);
 }
 
 ip_selector selector_from_endpoint(const ip_endpoint *endpoint)
 {
-	const struct ip_info *afi = endpoint_type(endpoint);
-	if (!pexpect(afi != NULL)) {
+	if (endpoint_is_unset(endpoint)) {
 		return unset_selector;
 	}
-	const ip_protocol *protocol = endpoint_protocol(endpoint);
-	ip_port port = endpoint_port(endpoint);
-	ip_protoport protoport = protoport2(protocol->ipproto, port);
 	ip_address address = endpoint_address(endpoint);
 	ip_subnet subnet = subnet_from_address(&address);
-	return selector_from_subnet(&subnet, &protoport);
+	const ip_protocol *protocol = endpoint_protocol(endpoint);
+	ip_port port = endpoint_port(endpoint);
+	return selector_from_subnet_protocol_port(&subnet, protocol, port);
 }
 
-ip_selector selector_from_subnet(const ip_subnet *subnet,
-				 const ip_protoport *protoport)
+ip_selector selector_from_subnet_protocol_port(const ip_subnet *subnet,
+					       const ip_protocol *protocol,
+					       ip_port port)
 {
-	const struct ip_info *afi = subnet_type(subnet);
-	if (!pexpect(afi != NULL)) {
+	if (subnet_is_unset(subnet)) {
 		return unset_selector;
 	}
 	ip_selector selector = {
@@ -89,14 +149,29 @@ ip_selector selector_from_subnet(const ip_subnet *subnet,
 		.addr = {
 			.version = subnet->addr.version,
 			.bytes = subnet->addr.bytes,
-			.ipproto = protoport->protocol,
-			.hport = protoport->port,
+			.ipproto = protocol->ipproto,
+			.hport = port.hport,
 		},
 	};
 	pselector(&selector);
 	return selector;
 }
 
+ip_selector selector_from_address_protoport(const ip_address *address,
+					    const ip_protoport *protoport)
+{
+	ip_subnet subnet = subnet_from_address(address);
+	return selector_from_subnet_protoport(&subnet, protoport);
+}
+
+ip_selector selector_from_subnet_protoport(const ip_subnet *subnet,
+					   const ip_protoport *protoport)
+{
+	const ip_protocol *protocol = protocol_by_ipproto(protoport->ipproto);
+	const ip_port port = ip_hport(protoport->hport);
+	return selector_from_subnet_protocol_port(subnet, protocol, port);
+}
+
 err_t range_to_selector(const ip_range *range,
 			const ip_protoport *protoport,
 			ip_selector *selector)
@@ -111,16 +186,10 @@ err_t range_to_selector(const ip_range *range,
 	if (err != NULL) {
 		return err;
 	}
-	*selector = selector_from_subnet(&subnet, protoport);
+	*selector = selector_from_subnet_protoport(&subnet, protoport);
 	return NULL;
 }
 
-#if 0
-ip_selector selector_from_range()
-{
-}
-#endif
-
 const struct ip_info *selector_type(const ip_selector *selector)
 {
 	if (selector == NULL) {
@@ -140,11 +209,6 @@ ip_port selector_port(const ip_selector *selector)
 	return ip_hport(selector->addr.hport);
 }
 
-void update_selector_hport(ip_selector *selector, unsigned hport)
-{
-	selector->addr.hport = hport;
-}
-
 unsigned selector_ipproto(const ip_selector *selector)
 {
 	return selector->addr.ipproto;
@@ -152,6 +216,9 @@ unsigned selector_ipproto(const ip_selector *selector)
 
 const ip_protocol *selector_protocol(const ip_selector *selector)
 {
+	if (selector_is_unset(selector)) {
+		return NULL;
+	}
 	return protocol_by_ipproto(selector->addr.ipproto);
 }
 
@@ -226,9 +293,10 @@ bool selector_in_selector(const ip_selector *l, const ip_selector *r)
 
 bool address_in_selector(const ip_address *address, const ip_selector *selector)
 {
-	ip_protoport protoport = selector_protoport(selector);
-	/* HACK: use same prot/port as selector so they always match */
-	ip_selector inner = selector_from_address(address, &protoport);
+	/* HACK: use same protocol/port as selector so they always match */
+	const ip_protocol *protocol = selector_protocol(selector);
+	const ip_port port = selector_port(selector);
+	ip_selector inner = selector_from_address_protocol_port(address, protocol, port);
 	return selector_in_selector(&inner, selector);
 }
 
