@@ -21,10 +21,13 @@
 
 void log_pexpect(where_t where, const char *message, ...)
 {
-	LSWLOG_PEXPECT_WHERE(where, buf) {
+	JAMBUF(buf) {
+		jam_string(buf, "EXPECTATION FAILED: ");
 		va_list args;
 		va_start(args, message);
-		lswlogvf(buf, message, args);
+		jam_va_list(buf, message, args);
 		va_end(args);
+		jam(buf, " "PRI_WHERE, pri_where(where));
+		jambuf_to_error_stream(buf); /* XXX: grrr */
 	}
 }
