@@ -16,51 +16,6 @@
 
 #include "ip_subnet.h"
 
-/*
- * subnettypeof - get the address type of an ip_subnet
- */
-int subnettypeof(src)
-const ip_subnet * src;
-{
-	return src->addr.u.v4.sin_family;
-}
-
-/*
- * networkof - get the network address of a subnet
- */
-void networkof(src, dst)
-const ip_subnet * src;
-ip_address *dst;
-{
-	*dst = src->addr;
-}
-
-/*
- * maskof - get the mask of a subnet, as an address
- */
-void maskof(src, dst)
-const ip_subnet * src;
-ip_address *dst;
-{
-	int b;
-	unsigned char buf[16];
-	size_t n = addrlenof(&src->addr);
-	unsigned char *p;
-
-	if ((size_t)src->maskbits > n * 8 || n > sizeof(buf))
-		return;	/* "can't happen" */
-
-	p = buf;
-	for (b = src->maskbits; b >= 8; b -= 8)
-		*p++ = 0xff;
-	if (b != 0)
-		*p++ = (0xff << (8 - b)) & 0xff;
-	while ((size_t)(p - buf) < n)
-		*p++ = 0;
-
-	(void) initaddr(buf, n, addrtypeof(&src->addr), dst);
-}
-
 /*
  * masktocount - convert a mask, expressed as an address, to a bit count
  */
@@ -69,12 +24,12 @@ masktocount(src)
 const ip_address * src;
 {
 	int b;
-	const unsigned char *bp;
-	size_t n;
 	const unsigned char *p;
 	const unsigned char *stop;
 
-	n = addrbytesptr_read(src, &bp);
+	shunk_t sc = address_as_shunk(src);
+	const uint8_t *bp = sc.ptr; /* cast const void * */
+	size_t n = sc.len;
 	if (n == 0)
 		return -1;
 
