@@ -18,12 +18,12 @@
 
 #include <string.h>
 #include <netdb.h>		/* for gethostbyname2() */
-#include <ctype.h>		/* for isxdigit() */
 
 #include "ip_address.h"
 #include "ip_info.h"
 #include "lswalloc.h"		/* for alloc_things(), pfree() */
 #include "lswlog.h"		/* for pexpect() */
+#include "hunk.h"		/* for char_is_xdigit() */
 
 /*
  * Legal ASCII characters in a domain name.  Underscore technically is not,
@@ -88,11 +88,12 @@ ttoaddr_base(const char *src,
 		if (oops == NULL)
 			return NULL;	/* it worked */
 
+	*allnumericfailed = 1;
+
 		if (*oops != '?')
 			return oops;	/* probably meant as d-d */
 	}
 
-	*allnumericfailed = 1;
 	return "not numeric";
 }
 
@@ -431,14 +432,14 @@ unsigned *retp;	/* return-value pointer */
 
 	p = *srcp;
 	d = 0;
-	while (p < stop && d < NDIG && isxdigit(*p)) {
+	while (p < stop && d < NDIG && char_isxdigit(*p)) {
 		p++;
 		d++;
 	}
 	if (d == 0)
 		return "non-hex field in IPv6 numeric address";
 
-	if (p < stop && d == NDIG && isxdigit(*p))
+	if (p < stop && d == NDIG && char_isxdigit(*p))
 		return "field in IPv6 numeric address longer than 4 hex digits";
 
 	oops = ttoul(*srcp, d, 16, &ret);
@@ -472,10 +473,18 @@ ttoaddr(const char *src,
 	err = ttoaddr_base(src, srclen, af, &numfailed, dst);
 
 	if (numfailed) {
-		if (err && af != AF_INET)
-			err = tryname(src, srclen, nultermd, AF_INET6, af, dst);
-		if (err && af != AF_INET6)
-			err = tryname(src, srclen, nultermd, AF_INET, af, dst);
+		/* err == non-numeric */
+		err_t v4err = NULL, v6err = NULL;
+		if (err && (af == AF_UNSPEC || af == AF_INET)) {
+			err = v4err = tryname(src, srclen, nultermd, AF_INET, af, dst);
+		}
+		if (err && (af == AF_UNSPEC || af == AF_INET6)) {
+			err = v6err = tryname(src, srclen, nultermd, AF_INET6, af, dst);
+		}
+		/* prefer the IPv4 error */
+		if (err != NULL && v4err != NULL) {
+			err = v4err;
+		}
 	}
 
 	return err;
@@ -483,7 +492,7 @@ ttoaddr(const char *src,
 
 err_t domain_to_address(shunk_t src, const struct ip_info *type, ip_address *dst)
 {
-	*dst = address_invalid;
+	*dst = unset_address;
 	if (src.len == 0) {
 		return "empty string";
 	}
@@ -510,7 +519,7 @@ ttoaddr_num(const char *src,
 
 err_t numeric_to_address(shunk_t src, const struct ip_info *type, ip_address *dst)
 {
-	*dst = address_invalid;
+	*dst = unset_address;
 	if (src.len == 0) {
 		return "empty string";
 	}
