@@ -29,11 +29,11 @@
 #include <sys/stat.h>
 #include <arpa/inet.h>
 #include <errno.h>
-#include <glob.h>
 #include <event2/event.h>
 #include <unbound.h>	/* from unbound devel */
 #include <unbound-event.h> /* from unbound devel */
 
+#include "lswglob.h"
 #include "dnssec.h"
 #include "constants.h"
 #include "lswlog.h"
@@ -48,28 +48,24 @@ void unbound_ctx_free(void)
 	}
 }
 
-static int globugh_ta(const char *epath, int eerrno)
-{
-	LOG_ERRNO(eerrno, "problem with trusted anchor file \"%s\"", epath);
-	return 1;	/* stop glob */
-}
-
-static void unbound_ctx_config(bool do_dnssec, const char *rootfile, const char *trusted)
+static void unbound_ctx_config(bool do_dnssec, const char *rootfile, const char *trusted,
+			       struct logger *logger)
 {
 	int ugh;
 
-	if (DBGP(DBG_DNS)) {
+	if (DBGP(DBG_BASE)) {
 		ub_ctx_debuglevel(dns_ctx, 5);
 		DBG_log("unbound context created - setting debug level to 5");
-	};
+	}
 
 	/* lookup from /etc/hosts before DNS lookups as people expect that */
 	ugh = ub_ctx_hosts(dns_ctx, "/etc/hosts");
 	if (ugh != 0) {
-		loglog(RC_LOG_SERIOUS, "error reading hosts: %s: %s",
-			ub_strerror(ugh), strerror(errno));
+		llog(RC_LOG_SERIOUS, logger,
+			    "error reading hosts: %s: %s",
+			    ub_strerror(ugh), strerror(errno));
 	} else {
-		DBGF(DBG_DNS, "/etc/hosts lookups activated");
+		dbg("/etc/hosts lookups activated");
 	}
 
 	/*
@@ -89,10 +85,11 @@ static void unbound_ctx_config(bool do_dnssec, const char *rootfile, const char
 	if (ugh != 0) {
 		int e = errno;	/* protect value from ub_strerror */
 
-		loglog(RC_LOG_SERIOUS, "error reading /etc/resolv.conf: %s: [errno: %s]",
-			ub_strerror(ugh), strerror(e));
+		llog(RC_LOG_SERIOUS, logger,
+			    "error reading /etc/resolv.conf: %s: [errno: %s]",
+			    ub_strerror(ugh), strerror(e));
 	} else {
-		DBGF(DBG_DNS, "/etc/resolv.conf usage activated");
+		dbg("/etc/resolv.conf usage activated");
 	}
 
 	/*
@@ -101,73 +98,81 @@ static void unbound_ctx_config(bool do_dnssec, const char *rootfile, const char
 	errno = 0;
 	ugh = ub_ctx_set_option(dns_ctx, "outgoing-port-avoid:", "0-65535");
 	if (ugh != 0) {
-		loglog(RC_LOG_SERIOUS, "error setting outgoing-port-avoid: %s: %s",
-			ub_strerror(ugh), strerror(errno));
+		llog(RC_LOG_SERIOUS, logger,
+			    "error setting outgoing-port-avoid: %s: %s",
+			    ub_strerror(ugh), strerror(errno));
 	} else {
-		DBGF(DBG_DNS, "outgoing-port-avoid set 0-65535");
+		dbg("outgoing-port-avoid set 0-65535");
 	}
 
 	errno = 0;
 	ugh = ub_ctx_set_option(dns_ctx, "outgoing-port-permit:", "32768-60999");
 		if (ugh != 0) {
-		loglog(RC_LOG_SERIOUS, "error setting outgoing-port-permit: %s: %s",
-			ub_strerror(ugh), strerror(errno));
+		llog(RC_LOG_SERIOUS, logger,
+			    "error setting outgoing-port-permit: %s: %s",
+			    ub_strerror(ugh), strerror(errno));
 	} else {
-		DBGF(DBG_DNS, "outgoing-port-permit set 32768-60999");
+		dbg("outgoing-port-permit set 32768-60999");
 	}
 
 	if (!do_dnssec) {
 		/* No DNSSEC - nothing more to configure */
-		DBGF(DBG_DNS, "dnssec validation disabled by configuration");
+		dbg("dnssec validation disabled by configuration");
 		return;
 	}
 
 	/* Only DNSSEC related configuration from here */
 	if (rootfile == NULL) {
 		if (trusted == NULL) {
-			loglog(RC_LOG_SERIOUS, "dnssec-enable=yes but no dnssec-rootkey-file or trust anchors specified.");
-			loglog(RC_LOG_SERIOUS, "WARNING: DNSSEC validation disabled");
+			llog(RC_LOG_SERIOUS, logger,
+				    "dnssec-enable=yes but no dnssec-rootkey-file or trust anchors specified.");
+			llog(RC_LOG_SERIOUS, logger,
+				    "WARNING: DNSSEC validation disabled");
 			return;
 		} else {
-			loglog(RC_LOG_SERIOUS, "dnssec-enable=yes but no dnssec-rootkey-file specified. Additional trust anchor file MUST include a root trust anchor or DNSSEC validation will be disabled");
+			llog(RC_LOG_SERIOUS, logger,
+				    "dnssec-enable=yes but no dnssec-rootkey-file specified. Additional trust anchor file MUST include a root trust anchor or DNSSEC validation will be disabled");
 		}
 	} else {
-		DBGF(DBG_DNS, "Loading dnssec root key from:%s", rootfile);
+		dbg("loading dnssec root key from:%s", rootfile);
 		errno = 0;
 		ugh = ub_ctx_add_ta_file(dns_ctx, rootfile);
 		if (ugh != 0) {
 			int e = errno;	/* protect value from ub_strerror */
 
-			loglog(RC_LOG_SERIOUS, "error adding dnssec root key: %s [errno: %s]",
-				ub_strerror(ugh), strerror(e));
-			loglog(RC_LOG_SERIOUS, "WARNING: DNSSEC validation disabled");
+			llog(RC_LOG_SERIOUS, logger,
+				    "error adding dnssec root key: %s [errno: %s]",
+				    ub_strerror(ugh), strerror(e));
+			llog(RC_LOG_SERIOUS, logger,
+				    "WARNING: DNSSEC validation disabled");
 		}
 	}
 
 	if (trusted == NULL) {
-		DBGF(DBG_DNS, "No additional dnssec trust anchors defined via dnssec-trusted= option");
+		dbg("no additional dnssec trust anchors defined via dnssec-trusted= option");
 	} else {
 		glob_t globbuf;
-		char **fnp;
-		int r = glob(trusted, GLOB_ERR, globugh_ta, &globbuf);
+		int r = lswglob(trusted, &globbuf, "trusted anchor", logger);
 
 		switch (r) {
 		case 0:	/* success */
-			for (fnp = globbuf.gl_pathv; fnp != NULL && *fnp != NULL; fnp++) {
+			for (char **fnp = globbuf.gl_pathv; fnp != NULL && *fnp != NULL; fnp++) {
 				ugh = ub_ctx_add_ta_file(dns_ctx, *fnp);
 				if (ugh != 0) {
-					loglog(RC_LOG_SERIOUS, "Ignored trusted key file %s: %s",
-						*fnp,  ub_strerror(ugh));
+					llog(RC_LOG_SERIOUS, logger,
+						    "Ignored trusted key file %s: %s",
+						    *fnp,  ub_strerror(ugh));
 				} else {
-					DBGF(DBG_DNS, "Added contents of trusted key file %s to unbound resolver context",
-						*fnp);
+					dbg("added contents of trusted key file %s to unbound resolver context",
+					    *fnp);
 				}
 			}
 			break;
 
 		case GLOB_NOSPACE:
-			loglog(RC_LOG_SERIOUS, "out of space processing dnssec-trusted= argument: %s",
-				trusted);
+			llog(RC_LOG_SERIOUS, logger,
+				    "out of space processing dnssec-trusted= argument: %s",
+				    trusted);
 			break;
 
 		case GLOB_ABORTED:
@@ -175,12 +180,14 @@ static void unbound_ctx_config(bool do_dnssec, const char *rootfile, const char
 			break;
 
 		case GLOB_NOMATCH:
-			loglog(RC_LOG_SERIOUS, "no trust anchor files matched '%s'", trusted);
+			llog(RC_LOG_SERIOUS, logger,
+				    "no trust anchor files matched '%s'", trusted);
 			break;
 
 		default:
-			loglog(RC_LOG_SERIOUS, "trusted key file '%s': unknown glob error %d",
-				trusted, r);
+			llog(RC_LOG_SERIOUS, logger,
+				    "trusted key file '%s': unknown glob error %d",
+				    trusted, r);
 			break;
 		}
 		globfree(&globbuf);
@@ -191,17 +198,17 @@ static void unbound_ctx_config(bool do_dnssec, const char *rootfile, const char
  * initialize a ub_ctx for asynchronous calls using libevent from pluto.
  *  only call once
  */
-bool unbound_event_init(struct event_base *eb, bool do_dnssec,
-			const char *rootfile, const char *trusted)
+diag_t unbound_event_init(struct event_base *eb, bool do_dnssec,
+			  const char *rootfile, const char *trusted,
+			  struct logger *logger)
 {
 	passert(dns_ctx == NULL); /* block re-entry to the function */
 	dns_ctx = ub_ctx_create_event(eb);
 	if (dns_ctx == NULL) {
-		loglog(RC_LOG_SERIOUS, "Failed to initialize unbound libevent ABI, please recompile libunbound with libevent support or recompile libreswan without USE_DNSSEC");
-		return FALSE;
+		return diag("failed to initialize unbound libevent ABI, please recompile libunbound with libevent support or recompile libreswan without USE_DNSSEC");
 	}
-	unbound_ctx_config(do_dnssec, rootfile, trusted);
-	return TRUE;
+	unbound_ctx_config(do_dnssec, rootfile, trusted, logger);
+	return NULL;
 }
 
 /*
@@ -210,20 +217,20 @@ bool unbound_event_init(struct event_base *eb, bool do_dnssec,
  * dns_ctx is static in this file. call unbound_ctx_free() to free it.
  */
 void unbound_sync_init(bool do_dnssec, const char *rootfile,
-			const char *trusted)
+		       const char *trusted, struct logger *logger)
 {
 	passert(dns_ctx == NULL); /* block re-entry to the function */
 	dns_ctx = ub_ctx_create();
 	passert(dns_ctx != NULL);
-	unbound_ctx_config(do_dnssec, rootfile, trusted);
+	unbound_ctx_config(do_dnssec, rootfile, trusted, logger);
 }
 
 /*
  * synchronous blocking resolving - simple replacement of ttoaddr()
  * src_len == 0 means "apply strlen"
- * af == AF_UNSPEC means "try both families"
+ * af == AF_UNSPEC means default to AF_INET(A/IPv4)
  */
-bool unbound_resolve(char *src, size_t srclen, int af, ip_address *ipaddr)
+bool unbound_resolve(char *src, size_t srclen, int af, ip_address *ipaddr, struct logger *logger)
 {
 	/* 28 = AAAA record, 1 = A record */
 	const int qtype = (af == AF_INET6) ? 28 : 1;
@@ -234,7 +241,7 @@ bool unbound_resolve(char *src, size_t srclen, int af, ip_address *ipaddr)
 	if (srclen == 0) {
 		srclen = strlen(src);
 		if (srclen == 0) {
-			libreswan_log("empty hostname in host lookup");
+			llog(RC_LOG, logger, "empty hostname in host lookup");
 			return FALSE;
 		}
 	}
@@ -243,37 +250,37 @@ bool unbound_resolve(char *src, size_t srclen, int af, ip_address *ipaddr)
 		int ugh = ub_resolve(dns_ctx, src, qtype, 1 /* CLASS IN */,
 				&result);
 		if (ugh != 0) {
-			libreswan_log("unbound error: %s", ub_strerror(ugh));
+			llog(RC_LOG, logger, "unbound error: %s", ub_strerror(ugh));
 			ub_resolve_free(result);
 			return FALSE;
 		}
 	}
 
 	if (result->bogus) {
-		libreswan_log("ERROR: %s failed DNSSEC validation!",
-			result->qname);
+		llog(RC_LOG, logger,
+			    "ERROR: %s failed DNSSEC validation!",
+			    result->qname);
 		ub_resolve_free(result);
 		return FALSE;
 	}
 	if (!result->havedata) {
 		if (result->secure) {
-			DBGF(DBG_DNS, "Validated reply proves '%s' does not exist",
+			dbg("validated reply proves '%s' does not exist",
 				src);
 		} else {
-			DBGF(DBG_DNS, "Failed to resolve '%s' (%s)", src,
+			dbg("failed to resolve '%s' (%s)", src,
 				result->bogus ? "BOGUS" : "insecure");
 		}
 		ub_resolve_free(result);
 		return FALSE;
 	} else if (!result->bogus) {
 		if (!result->secure) {
-			DBGF(DBG_DNS, "warning: %s lookup was not protected by DNSSEC!",
+			dbg("warning: %s lookup was not protected by DNSSEC!",
 				result->qname);
 		}
 	}
 
-#if 0
-	{
+	if (DBGP(DBG_TMI)) {
 		int i = 0;
 		DBG_log("The result has:");
 		DBG_log("qname: %s", result->qname);
@@ -289,7 +296,6 @@ bool unbound_resolve(char *src, size_t srclen, int af, ip_address *ipaddr)
 		}
 		DBG_log("result has %d data element(s)", i);
 	}
-#endif
 
 	/* XXX: for now pick the first one and return that */
 	passert(result->data[0] != NULL);
@@ -302,11 +308,11 @@ bool unbound_resolve(char *src, size_t srclen, int af, ip_address *ipaddr)
 			0, af, ipaddr);
 		ub_resolve_free(result);
 		if (err == NULL) {
-			DBGF(DBG_DNS, "success for %s lookup",
+			dbg("success for %s lookup",
 				(af == AF_INET) ? "IPv4" : "IPv6");
 			return TRUE;
 		} else {
-			libreswan_log("tnatoaddr failed in unbound_resolve()");
+			llog(RC_LOG, logger, "tnatoaddr failed in unbound_resolve()");
 			return FALSE;
 		}
 	}
