@@ -646,7 +646,7 @@ err_t lease_that_address(struct connection *c, const struct state *st)
 	ip_address ia = lease_address(pool, new_lease);
 	c->spd.that.has_lease = true;
 	c->spd.that.has_client = true;
-	c->spd.that.client = selector_from_address(&ia, &unset_protoport);
+	c->spd.that.client = selector_from_address(&ia);
 	new_lease->assigned_to = c->serialno;
 
 	if (DBGP(DBG_BASE)) {
