@@ -23,14 +23,16 @@
 struct ip_range;
 struct ip_pool;        /* forward declaration; definition is local to addresspool.c */
 
-extern struct ip_pool *install_addresspool(const ip_range *pool_range);
-extern err_t find_addresspool(const ip_range *pool_range, struct ip_pool **pool);
+extern struct ip_pool *install_addresspool(const ip_range *pool_range, struct logger *logger);
+extern bool find_addresspool(const ip_range *pool_range, struct ip_pool **pool, struct logger *logger);
 extern bool pool_size(ip_range *r, uint32_t *size);
 
 extern void unreference_addresspool(struct connection *c);
 extern void reference_addresspool(struct connection *c);
 
-extern err_t lease_an_address(const struct connection *c, const struct state *st, ip_address *ipa /*result*/);
-extern void rel_lease_addr(struct connection *c);
+extern err_t lease_that_address(struct connection *c, const struct state *st);
+extern void free_that_address_lease(struct connection *c);
+
+extern void show_addresspool_status(struct show *s);
 
 #endif /* _ADDRESSPOOL_H */
