@@ -441,13 +441,10 @@ void update_ends_from_this_host_addr(struct end *this, struct end *that);
 extern void restart_connections_by_peer(struct connection *c);
 extern void flush_revival(const struct connection *c);
 
-struct xfrm_user_sec_ctx_ike; /* forward declaration */
-
-extern void initiate_ondemand(const ip_address *our_client,
-			      const ip_address *peer_client,
-			      int transport_proto,
+extern void initiate_ondemand(const ip_endpoint *our_client,
+			      const ip_endpoint *peer_client,
 			      bool held, bool background,
-			      const chunk_t *sec_label,
+			      const chunk_t sec_label,
 			      const char *why,
 			      struct logger *logger);
 
@@ -497,11 +494,12 @@ extern struct connection
 			const uint8_t our_protocol,
 			const uint16_t out_port,
 			const uint8_t peer_protocol,
-			const uint16_t peer_port),
-	*find_connection_for_clients(struct spd_route **srp,
-				      const ip_address *our_client,
-				      const ip_address *peer_client,
-				      int transport_proto, chunk_t sec_label);
+			const uint16_t peer_port);
+struct connection *find_connection_for_clients(struct spd_route **srp,
+					       const ip_endpoint *our_client,
+					       const ip_endpoint *peer_client,
+					       chunk_t sec_label,
+					       struct logger *logger);
 
 /* instantiating routines */
 
@@ -512,25 +510,15 @@ extern struct connection *rw_instantiate(struct connection *c,
 					 const ip_subnet *peer_subnet,
 					 const struct id *peer_id);
 struct connection *oppo_instantiate(struct connection *c,
-				    const ip_address *peer_addr,
-				    const struct id *peer_id,
-				    const ip_address *our_client,
-				    const ip_address *peer_client);
+				    const struct id *remote_id,
+				    const ip_address *local_address,
+				    const ip_address *remote_address);
 extern struct connection *instantiate(struct connection *c,
 				      const ip_address *peer_addr,
 				      const struct id *peer_id);
 
-extern struct connection *build_outgoing_opportunistic_connection(
-		const ip_address *our_client,
-		const ip_address *peer_client,
-		const int transport_proto);
-
-/* worst case: "[" serial "] " myclient "=== ..." peer "===" peer_client '\0' <cookie> */
-#define CONN_INST_BUF \
-	(2 + 10 + 1 + SUBNETTOT_BUF + 7 + ADDRTOT_BUF + 3 + SUBNETTOT_BUF + 1 + 1)
-
-extern char *fmt_conn_instance(const struct connection *c,
-			       char buf[CONN_INST_BUF]);
+struct connection *build_outgoing_opportunistic_connection(const ip_endpoint *our_client,
+							   const ip_endpoint *peer_client);
 
 /* publicly useful? */
 size_t jam_connection_instance(struct jambuf *buf, const struct connection *c);
@@ -539,10 +527,20 @@ size_t jam_connection(struct jambuf *buf, const struct connection *c);
 /*
  * XXX: Instead of str_connection(), which would require a buffer big
  * enough to fit an any length name, there's PRI_CONNECTION et.al.
- */
+*/
 
 typedef struct {
-	char buf[CONN_INST_BUF];
+	char buf[/*why?*/ 1 +
+		 /*"["*/ 1 +
+		 /*<serialno>*/ 10 +
+		 /*"]"*/ 1 +
+		 /*<myclient*/SUBNETTOT_BUF +
+		 /*"=== ..."*/ 7 +
+		 /*<peer>*/ ADDRTOT_BUF +
+		 /*"==="*/ 3 +
+		 /*<peer_client>*/ SUBNETTOT_BUF +
+		 /*"\0"*/ 1 +
+		 /*<cookie>*/ 1];
 } connection_buf;
 
 const char *str_connection_instance(const struct connection *c,
