@@ -31,38 +31,42 @@
 #include <pk11pub.h>
 
 #include "chunk.h"
+#include "ike_spi.h"
 
 struct dh_desc;
 struct state;
 struct msg_digest;
+struct logger;
 
 /*
  * The DH secret (opaque, but we all know it is implemented using
  * NSS).
  */
-struct dh_secret;
+struct dh_local_secret;
 
-struct dh_secret *calc_dh_secret(const struct dh_desc *group,
-				 chunk_t *ke);
+struct dh_local_secret *calc_dh_local_secret(const struct dh_desc *group, struct logger *logger);
+chunk_t clone_dh_local_secret_ke(struct dh_local_secret *local_secret);
+const struct dh_desc *dh_local_secret_desc(struct dh_local_secret *local_secret);
 
-PK11SymKey *calc_dh_shared(struct dh_secret *secret,
-			   chunk_t remote_ke);
-
-void transfer_dh_secret_to_state(const char *helper, struct dh_secret **secret,
-				 struct state *st);
-
-void transfer_dh_secret_to_helper(struct state *st,
-				  const char *helper, struct dh_secret **secret);
-
-void free_dh_secret(struct dh_secret **secret);
+struct dh_local_secret *dh_local_secret_addref(struct dh_local_secret *local_secret, where_t where);
+void dh_local_secret_delref(struct dh_local_secret **local_secret, where_t where);
 
 /*
- * Compute dh storing result in .st_shared_nss.
+ * Compute dh using .st_dh_local_secret and REMOTE_KE, storing result
+ * in .st_dh_shared_secret.
  */
-typedef stf_status (dh_cb)(struct state *st,
-			   struct msg_digest **md);
 
-extern void submit_dh(struct state *st, chunk_t remote_ke,
-		      dh_cb *callback, const char *name);
+typedef stf_status (dh_shared_secret_cb)(struct state *st,
+					 struct msg_digest *md);
+
+extern void submit_dh_shared_secret(struct state *st, chunk_t remote_ke,
+				    dh_shared_secret_cb *callback, where_t where);
+
+/* internal */
+void calc_v1_skeyid_and_iv(struct state *st);
+void calc_v2_keymat(struct state *st,
+		    PK11SymKey *old_skey_d, /* SKEYSEED IKE Rekey */
+		    const struct prf_desc *old_prf, /* IKE Rekey */
+		    const ike_spis_t *new_ike_spis);
 
 #endif
