@@ -9,6 +9,7 @@
  * Copyright (C) 2013,2017 Paul Wouters <pwouters@redhat.com>
  * Copyright (C) 2015 Antony Antony <antony@phenome.org>
  * Copyright (C) 2017-2019 Andrew Cagney <cagney@gnu.org>
+ * Copyright (C) 2017 Mayank Totale <mtotale@gmail.com>
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the
@@ -45,7 +46,6 @@
 
 #include "sysdep.h"
 #include "constants.h"
-#include "lswlog.h"
 
 #include "defs.h"
 #include "id.h"
@@ -62,13 +62,15 @@
 #include "ikev2.h"
 #include "ipsec_doi.h"  /* needs demux.h and state.h */
 #include "timer.h"
-#include "udpfromto.h"
 #include "ip_sockaddr.h"
 #include "ip_address.h"
 #include "ip_endpoint.h"
+#include "ip_protocol.h"
 #include "ip_info.h"
 #include "pluto_stats.h"
 #include "ikev2_send.h"
+#include "iface.h"
+#include "impair_message.h"
 
 /*
  * read the message.
@@ -77,158 +79,53 @@
  * buffer and then copy it to a new, properly sized buffer.
  */
 
-static struct msg_digest *read_packet(const struct iface_port *ifp)
+static enum iface_status read_message(const struct iface_endpoint *ifp,
+				      struct msg_digest **mdp,
+				      struct logger *logger)
 {
-	int packet_len;
+	pexpect(*mdp == NULL);
+
 	/* ??? this buffer seems *way* too big */
 	uint8_t bigbuffer[MAX_INPUT_UDP_SIZE];
-
-	uint8_t *_buffer = bigbuffer;
-	ip_sockaddr from;
-	zero(&from);
-	socklen_t from_len = sizeof(from);
-#if defined(HAVE_UDPFROMTO)
-	ip_sockaddr to;
-	zero(&to);
-	socklen_t to_len   = sizeof(to);
-#endif
-
-#if defined(HAVE_UDPFROMTO)
-	packet_len = recvfromto(ifp->fd, bigbuffer,
-				sizeof(bigbuffer), /*flags*/ 0,
-				&from.sa, &from_len,
-				&to.sa, &to_len);
-#else
-	packet_len = recvfrom(ifp->fd, bigbuffer,
-			      sizeof(bigbuffer), /*flags*/ 0,
-			      &from.sa, &from_len);
-#endif
-	int packet_errno = errno; /* save!!! */
-
-	/* we do not do anything with *to* addresses yet... we will */
-
-	/*
-	 * Try to decode the from address.
-	 *
-	 * If that fails report some sense of error and then always
-	 * give up.
-	 */
-	ip_endpoint sender;
-	const char *from_ugh = sockaddr_to_endpoint(&from, from_len, &sender);
-	if (from_ugh != NULL) {
-		if (packet_len >= 0) {
-			/* technically it worked, but returned value was useless */
-			plog_global("recvfrom on %s returned malformed source sockaddr: %s",
-				    ifp->ip_dev->id_rname, from_ugh);
-		} else if (from_len == sizeof(from) &&
-			   all_zero((const void *)&from, sizeof(from)) &&
-			   packet_errno == ECONNREFUSED) {
-			/*
-			 * Tone down scary message for vague event: We
-			 * get "connection refused" in response to
-			 * some datagram we sent, but we cannot tell
-			 * which one.
-			 */
-			plog_global("recvfrom on %s failed; some IKE message we sent has been rejected with ECONNREFUSED (kernel supplied no details)",
-				    ifp->ip_dev->id_rname);
-		} else {
-			/* if from==0, this prints "unspecified", not "undisclosed", oops */
-			plog_global("recvfrom on %s failed; Pluto cannot decode source sockaddr in rejection: %s "PRI_ERRNO,
-				    ifp->ip_dev->id_rname, from_ugh,
-				    pri_errno(packet_errno));
-		}
-		return NULL;
-	}
-
-	/*
-	 * Managed to decode the from address; fudge up an MD so that
-	 * it be used as log context prefix.
-	 */
-
-	struct msg_digest stack_md = {
-		.iface = ifp,
-		.sender = sender,
+	struct iface_packet packet = {
+		.ptr = bigbuffer,
+		.len = sizeof(bigbuffer),
+		.logger = logger,
 	};
 
-	if (packet_len < 0) {
-		plog_md(&stack_md, "recvfrom on %s failed "PRI_ERRNO,
-			ifp->ip_dev->id_rname, pri_errno(packet_errno));
-		return NULL;
-	}
-
-	if (ifp->ike_float) {
-		uint32_t non_esp;
-
-		if (packet_len < (int)sizeof(uint32_t)) {
-			plog_md(&stack_md, "too small packet (%d)",
-				packet_len);
-			return NULL;
-		}
-		memcpy(&non_esp, _buffer, sizeof(uint32_t));
-		if (non_esp != 0) {
-			plog_md(&stack_md, "has no Non-ESP marker");
-			return NULL;
-		}
-		_buffer += sizeof(uint32_t);
-		packet_len -= sizeof(uint32_t);
-	}
-
-	/* We think that in 2013 Feb, Apple iOS Racoon
-	 * sometimes generates an extra useless buggy confusing
-	 * Non ESP Marker
-	 */
-	{
-		static const uint8_t non_ESP_marker[NON_ESP_MARKER_SIZE] =
-			{ 0x00, };
-		if (ifp->ike_float &&
-		    packet_len >= NON_ESP_MARKER_SIZE &&
-		    memeq(_buffer, non_ESP_marker,
-			   NON_ESP_MARKER_SIZE)) {
-			plog_md(&stack_md, "mangled with potential spurious non-esp marker");
-			return NULL;
-		}
-	}
-
-	if (packet_len == 1 && _buffer[0] == 0xff) {
-		/**
-		 * NAT-T Keep-alive packets should be discared by kernel ESPinUDP
-		 * layer. But bogus keep-alive packets (sent with a non-esp marker)
-		 * can reach this point. Complain and discard them.
-		 * Possibly too if the NAT mapping vanished on the initiator NAT gw ?
-		 */
-		endpoint_buf eb;
-		dbg("NAT-T keep-alive (bogus ?) should not reach this point. Ignored. Sender: %s",
-		    str_endpoint(&sender, &eb)); /* sensitive? */
-		return NULL;
+	enum iface_status status = ifp->io->read_packet(ifp, &packet, logger);
+	if (status != IFACE_OK) {
+		/* already logged */
+		return status;
 	}
 
 	/*
 	 * Create the real message digest; and set up md->packet_pbs
 	 * to describe it.
 	 */
-	struct msg_digest *md = alloc_md("msg_digest in read_packet");
-	md->sender = stack_md.sender;
-	md->iface = stack_md.iface;
+	struct msg_digest *md = alloc_md(ifp, &packet.sender, HERE);
 	init_pbs(&md->packet_pbs,
-		 clone_bytes(_buffer, packet_len,
+		 clone_bytes(packet.ptr, packet.len,
 			     "message buffer in read_packet()"),
-		 packet_len, "packet");
+		 packet.len, "packet");
 
-	endpoint_buf eb;
-	endpoint_buf b2;
-	dbg("*received %d bytes from %s on %s (%s)",
+	endpoint_buf sb;
+	endpoint_buf lb;
+	dbg("*received %d bytes from %s on %s %s using %s",
 	    (int) pbs_room(&md->packet_pbs),
-	    str_endpoint(&sender, &eb),
+	    str_endpoint(&md->sender, &sb),
 	    ifp->ip_dev->id_rname,
-	    str_endpoint(&ifp->local_endpoint, &b2));
+	    str_endpoint(&ifp->local_endpoint, &lb),
+	    ifp->protocol->name);
 
-	if (DBGP(DBG_RAW)) {
+	if (DBGP(DBG_BASE)) {
 		DBG_dump(NULL, md->packet_pbs.start, pbs_room(&md->packet_pbs));
 	}
 
 	pstats_ike_in_bytes += pbs_room(&md->packet_pbs);
 
-	return md;
+	*mdp = md;
+	return IFACE_OK;
 }
 
 /*
@@ -247,14 +144,16 @@ void process_packet(struct msg_digest **mdp)
 {
 	struct msg_digest *md = *mdp;
 
-	if (!in_struct(&md->hdr, &isakmp_hdr_desc, &md->packet_pbs,
-		       &md->message_pbs)) {
+	diag_t d = pbs_in_struct(&md->packet_pbs, &isakmp_hdr_desc,
+				 &md->hdr, sizeof(md->hdr), &md->message_pbs);
+	if (d != NULL) {
 		/*
 		 * The packet was very badly mangled. We can't be sure
 		 * of any content - not even to look for major version
 		 * number!  So we'll just drop it.
 		 */
-		plog_md(md, "received packet with mangled IKE header - dropped");
+		log_diag(RC_LOG, md->md_logger, &d,
+			 "dropping packet with mangled IKE header: ");
 		return;
 	}
 
@@ -262,7 +161,7 @@ void process_packet(struct msg_digest **mdp)
 		/* Some (old?) versions of the Cisco VPN client send an additional
 		 * 16 bytes of zero bytes - Complain but accept it
 		 */
-		if (DBGP(DBG_CONTROL)) {
+		if (DBGP(DBG_BASE)) {
 			DBG_log("size (%u) in received packet is larger than the size specified in ISAKMP HDR (%u) - ignoring extraneous bytes",
 				(unsigned) pbs_room(&md->packet_pbs),
 				md->hdr.isa_length);
@@ -280,10 +179,24 @@ void process_packet(struct msg_digest **mdp)
 		 * IKEv2 doesn't say what to do with low versions,
 		 * just drop them.
 		 */
-		plog_md(md, "ignoring packet with IKE major version '%d'", vmaj);
+		llog(RC_LOG, md->md_logger,
+		     "ignoring packet with IKE major version '%d'", vmaj);
 		return;
 
 	case ISAKMP_MAJOR_VERSION: /* IKEv1 */
+		if (pluto_ikev1_pol == GLOBAL_IKEv1_DROP) {
+			llog(RC_LOG, md->md_logger,
+			     "ignoring IKEv1 packet as policy is set to silently drop all IKEv1 packets");
+			return;
+		}
+#ifdef USE_IKEv1
+		if (pluto_ikev1_pol == GLOBAL_IKEv1_REJECT) {
+			llog(RC_LOG, md->md_logger,
+			     "rejecting IKEv1 packet as policy is set to reject all IKEv1 packets");
+			send_notification_from_md(md, INVALID_MAJOR_VERSION);
+			return;
+		}
+
 		if (vmin > ISAKMP_MINOR_VERSION) {
 			/* RFC2408 3.1 ISAKMP Header Format:
 			 *
@@ -299,7 +212,8 @@ void process_packet(struct msg_digest **mdp)
 			 * own, given the major version numbers are
 			 * identical.
 			 */
-			plog_md(md, "ignoring packet with IKEv1 minor version number %d greater than %d", vmin, ISAKMP_MINOR_VERSION);
+			llog(RC_LOG, md->md_logger,
+			     "ignoring packet with IKEv1 minor version number %d greater than %d", vmin, ISAKMP_MINOR_VERSION);
 			send_notification_from_md(md, INVALID_MINOR_VERSION);
 			return;
 		}
@@ -307,8 +221,9 @@ void process_packet(struct msg_digest **mdp)
 		    vmaj, vmin,
 		    enum_name(&exchange_names_ikev1orv2, md->hdr.isa_xchg),
 		    md->hdr.isa_xchg);
-		process_v1_packet(mdp);
+		process_v1_packet(md);
 		/* our caller will release_any_md(mdp) */
+#endif
 		break;
 
 	case IKEv2_MAJOR_VERSION: /* IKEv2 */
@@ -316,18 +231,19 @@ void process_packet(struct msg_digest **mdp)
 			/* Unlike IKEv1, for IKEv2 we are supposed to try to
 			 * continue on unknown minors
 			 */
-			plog_md(md, "Ignoring unknown IKEv2 minor version number %d", vmin);
+			llog(RC_LOG, md->md_logger,
+			     "Ignoring unknown IKEv2 minor version number %d", vmin);
 		}
 		dbg(" processing version=%u.%u packet with exchange type=%s (%d)",
 		    vmaj, vmin,
 		    enum_name(&exchange_names_ikev1orv2, md->hdr.isa_xchg),
 		    md->hdr.isa_xchg);
-		ikev2_process_packet(mdp);
-		/* our caller will release_any_md(mdp) */
+		ikev2_process_packet(md);
 		break;
 
 	default:
-		plog_md(md, "message contains unsupported IKE major version '%d'", vmaj);
+		llog(RC_LOG, md->md_logger,
+		     "message contains unsupported IKE major version '%d'", vmaj);
 		/*
 		 * According to 1.5.  Informational Messages outside
 		 * of an IKE SA, [...] the message is always sent
@@ -376,12 +292,7 @@ void process_packet(struct msg_digest **mdp)
  */
 static void process_md(struct msg_digest **mdp)
 {
-	ip_address old_from = push_cur_from((*mdp)->sender);
 	process_packet(mdp);
-	pop_cur_from(old_from);
-	release_any_md(mdp);
-	reset_cur_state();
-	reset_cur_connection();
 }
 
 /* wrapper for read_packet and process_packet
@@ -398,43 +309,29 @@ static void process_md(struct msg_digest **mdp)
  * enormous input packet buffer, an auto.
  */
 
-static bool impair_incoming(struct msg_digest **mdp);
+static bool impair_incoming(struct msg_digest *md);
 
-void comm_handle_cb(evutil_socket_t unused_fd UNUSED,
-		    const short unused_event UNUSED,
-		    void *arg)
+enum iface_status handle_packet_cb(const struct iface_endpoint *ifp,
+				   struct logger *logger)
 {
-	const struct iface_port *ifp = arg;
-	/*
-	 * Even though select(2) says that there is a message, it
-	 * might only be a MSG_ERRQUEUE message.  At least sometimes
-	 * that leads to a hanging recvfrom.  To avoid what appears to
-	 * be a kernel bug, check_msg_errqueue uses poll(2) and tells
-	 * us if there is anything for us to read.
-	 *
-	 * This is early enough that teardown isn't required:
-	 * just return on failure.
-	 */
-	threadtime_t errqueue_start = threadtime_start();
-	bool errqueue_ok = check_incoming_msg_errqueue(ifp, "read_packet");
-	threadtime_stop(&errqueue_start, SOS_NOBODY,
-			"%s() calling check_incoming_msg_errqueue()", __func__);
-	if (!errqueue_ok) {
-		return; /* no normal message to read */
-	}
-
 	threadtime_t md_start = threadtime_start();
-	struct msg_digest *md = read_packet(ifp);
-	if (md != NULL) {
+	struct msg_digest *md = NULL;
+	enum iface_status status = read_message(ifp, &md, logger);
+	if (status != IFACE_OK) {
+		pexpect(md == NULL);
+	} else if (pexpect(md != NULL)) {
 		md->md_inception = md_start;
-		if (!impair_incoming(&md)) {
+		if (!impair_incoming(md)) {
 			process_md(&md);
 		}
+		md_delref(&md, HERE);
 		pexpect(md == NULL);
+	} else {
+		status = IFACE_FATAL;
 	}
 	threadtime_stop(&md_start, SOS_NOBODY,
 			"%s() reading and processing packet", __func__);
-	pexpect_reset_globals();
+	return status;
 }
 
 /*
@@ -452,33 +349,32 @@ static void process_md_clone(struct msg_digest *orig, const char *fmt, ...) PRIN
 static void process_md_clone(struct msg_digest *orig, const char *fmt, ...)
 {
 	/* not whack FD yet is expected to be reset! */
-	pexpect_reset_globals();
-	struct msg_digest *md = clone_raw_md(orig, fmt /* good enough */);
+	struct msg_digest *md = clone_raw_md(orig, HERE);
 
-	LSWLOG(buf) {
-		lswlogs(buf, "IMPAIR: start processing ");
+	LLOG_JAMBUF(RC_LOG, md->md_logger, buf) {
+		jam_string(buf, "IMPAIR: start processing ");
 		va_list ap;
 		va_start(ap, fmt);
-		lswlogvf(buf, fmt, ap);
+		jam_va_list(buf, fmt, ap);
 		va_end(ap);
-		lswlogf(buf, " (%d bytes)", (int)pbs_room(&md->packet_pbs));
+		jam(buf, " (%d bytes)", (int)pbs_room(&md->packet_pbs));
 	}
-	if (DBGP(DBG_RAW)) {
+	if (DBGP(DBG_BASE)) {
 		DBG_dump(NULL, md->packet_pbs.start, pbs_room(&md->packet_pbs));
 	}
 
 	process_md(&md);
 
-	LSWLOG(buf) {
-		lswlogf(buf, "IMPAIR: stop processing ");
+	LLOG_JAMBUF(RC_LOG, md->md_logger, buf) {
+		jam(buf, "IMPAIR: stop processing ");
 		va_list ap;
 		va_start(ap, fmt);
-		lswlogvf(buf, fmt, ap);
+		jam_va_list(buf, fmt, ap);
 		va_end(ap);
 	}
 
+	md_delref(&md, HERE);
 	pexpect(md == NULL);
-	pexpect_reset_globals();
 }
 
 static unsigned long replay_count;
@@ -489,36 +385,38 @@ struct replay_entry {
 	unsigned long nr;
 };
 
-static void jam_replay_entry(struct lswlog *buf, const void *data)
+static void jam_replay_entry(struct jambuf *buf, const void *data)
 {
 	const struct replay_entry *r = data;
 	jam(buf, "replay packet %lu", r == NULL ? 0L : r->nr);
 }
 
-static struct list_head replay_packets;
-
 static const struct list_info replay_info = {
 	.name = "replay list",
 	.jam = jam_replay_entry,
 };
 
-static void save_any_md_for_replay(struct msg_digest **mdp)
+static struct list_head replay_packets = INIT_LIST_HEAD(&replay_packets, &replay_info);
+
+static void save_md_for_replay(bool already_impaired, struct msg_digest *md)
 {
-	if (mdp != NULL && *mdp != NULL) {
-		init_list(&replay_info, &replay_packets);
+	if (!already_impaired) {
 		struct replay_entry *e = alloc_thing(struct replay_entry, "replay");
-		e->md = clone_raw_md(*mdp, "copy of real message");
+		e->md = clone_raw_md(md, HERE);
 		e->nr = ++replay_count; /* yes; pre-increment */
 		e->entry = list_entry(&replay_info, e); /* back-link */
 		insert_list_entry(&replay_packets, &e->entry);
-		release_any_md(mdp);
 	}
 }
 
-static bool impair_incoming(struct msg_digest **mdp)
+static bool impair_incoming(struct msg_digest *md)
 {
-	if (IMPAIR(REPLAY_DUPLICATES)) {
-		save_any_md_for_replay(mdp);
+	if (impair_incoming_message(md)) {
+		return true;
+	}
+	bool impaired = false;
+	if (impair.replay_duplicates) {
+		save_md_for_replay(impaired, md);
 		/* MD is the most recent entry */
 		struct replay_entry *e = NULL;
 		FOR_EACH_LIST_ENTRY_NEW2OLD(&replay_packets, e) {
@@ -526,25 +424,37 @@ static bool impair_incoming(struct msg_digest **mdp)
 			process_md_clone(e->md, "duplicate packet");
 			break;
 		}
+		impaired = true;
 	}
-	if (IMPAIR(REPLAY_FORWARD)) {
-		save_any_md_for_replay(mdp);
+	if (impair.replay_forward) {
+		save_md_for_replay(impaired, md);
 		struct replay_entry *e = NULL;
 		FOR_EACH_LIST_ENTRY_OLD2NEW(&replay_packets, e) {
 			process_md_clone(e->md, "replay forward: packet %lu of %lu",
 					 e->nr, replay_count);
 		}
+		impaired = true;
 	}
-	if (IMPAIR(REPLAY_BACKWARD)) {
-		save_any_md_for_replay(mdp);
+	if (impair.replay_backward) {
+		save_md_for_replay(impaired, md);
 		struct replay_entry *e = NULL;
 		FOR_EACH_LIST_ENTRY_NEW2OLD(&replay_packets, e) {
 			process_md_clone(e->md, "start replay backward: packet %lu of %lu",
 					 e->nr, replay_count);
 		}
+		impaired = true;
+	}
+	return impaired;
+}
+
+void free_demux(void)
+{
+	struct replay_entry *e = NULL;
+	FOR_EACH_LIST_ENTRY_NEW2OLD(&replay_packets, e) {
+		md_delref(&e->md, HERE);
+		remove_list_entry(&e->entry);
+		pfreeany(e);
 	}
-	/* MDP NULL implies things were impaired */
-	return *mdp == NULL;
 }
 
 static callback_cb handle_md_event; /* type assertion */
@@ -553,8 +463,8 @@ static void handle_md_event(struct state *st, void *context)
 	pexpect(st == NULL);
 	struct msg_digest *md = context;
 	process_md(&md);
+	md_delref(&md, HERE);
 	pexpect(md == NULL);
-	pexpect_reset_globals();
 }
 
 void schedule_md_event(const char *name, struct msg_digest *md)
@@ -586,7 +496,7 @@ enum ike_version hdr_ike_version(const struct isakmp_hdr *hdr)
  *   either of the initiator and/or responder values
  *
  * - encourages the coding style where the two cases - REQUEST and
- *   RESPONSE - are clearly labled, that is:
+ *   RESPONSE - are clearly labeled, that is:
  *
  *       switch(role) {
  *       case MESSAGE_REQUEST: ...; break;
@@ -621,10 +531,11 @@ enum message_role v2_msg_role(const struct msg_digest *md)
  * Result is allocated on heap so caller must ensure it is freed.
  */
 
-char *cisco_stringify(pb_stream *input_pbs, const char *attr_name)
+char *cisco_stringify(pb_stream *input_pbs, const char *attr_name,
+		      bool ignore, struct logger *logger)
 {
 	char strbuf[500]; /* Cisco maximum unknown - arbitrary choice */
-	jambuf_t buf = ARRAY_AS_JAMBUF(strbuf); /* let jambuf deal with overflow */
+	struct jambuf buf = ARRAY_AS_JAMBUF(strbuf); /* let jambuf deal with overflow */
 	shunk_t str = pbs_in_left_as_shunk(input_pbs);
 
 	/*
@@ -649,14 +560,14 @@ char *cisco_stringify(pb_stream *input_pbs, const char *attr_name)
 			/*
 			 * preserve sanitize_string() behaviour:
 			 *
-			 * exception is that all veritical space just
+			 * exception is that all vertical space just
 			 * becomes white space
 			 */
 			jam(&buf, " ");
 			break;
 		default:
 			/*
-			 * preserve sanitize_string() behavour:
+			 * preserve sanitize_string() behaviour:
 			 *
 			 * XXX: isprint() is wrong as it is affected
 			 * by locale - need portable is printable
@@ -671,10 +582,14 @@ char *cisco_stringify(pb_stream *input_pbs, const char *attr_name)
 			break;
 		}
 	}
-	if (!jambuf_ok(&buf)) {
-		loglog(RC_INFORMATIONAL, "Received overlong %s: %s (truncated)", attr_name, strbuf);
-	} else {
-		loglog(RC_INFORMATIONAL, "Received %s: %s", attr_name, strbuf);
+	llog(RC_INFORMATIONAL, logger,
+		    "Received %s%s%s: %s%s",
+		    ignore ? "and ignored " : "",
+		    jambuf_ok(&buf) ? "" : "overlong ",
+		    attr_name, strbuf,
+		    jambuf_ok(&buf) ? "" : " (truncated)");
+	if (ignore) {
+		return NULL;
 	}
 	return clone_str(strbuf, attr_name);
 }
@@ -682,15 +597,15 @@ char *cisco_stringify(pb_stream *input_pbs, const char *attr_name)
 /*
  * list all the payload types
  */
-void lswlog_msg_digest(struct lswlog *buf, const struct msg_digest *md)
+void lswlog_msg_digest(struct jambuf *buf, const struct msg_digest *md)
 {
 	enum ike_version ike_version = hdr_ike_version(&md->hdr);
-	lswlog_enum_enum_short(buf, &exchange_type_names, ike_version,
-			       md->hdr.isa_xchg);
+	jam_enum_enum_short(buf, &exchange_type_names, ike_version,
+			    md->hdr.isa_xchg);
 	if (ike_version == IKEv2) {
 		switch (v2_msg_role(md)) {
-		case MESSAGE_REQUEST: lswlogs(buf, " request"); break;
-		case MESSAGE_RESPONSE: lswlogs(buf, " response"); break;
+		case MESSAGE_REQUEST: jam_string(buf, " request"); break;
+		case MESSAGE_RESPONSE: jam_string(buf, " response"); break;
 		case NO_MESSAGE: break;
 		}
 	}
@@ -698,7 +613,7 @@ void lswlog_msg_digest(struct lswlog *buf, const struct msg_digest *md)
 	const char *term = "";
 	for (unsigned d = 0; d < md->digest_roof; d++) {
 		const struct payload_digest *pd = &md->digest[d];
-		lswlogs(buf, sep);
+		jam_string(buf, sep);
 		if (ike_version == IKEv2 &&
 		    d+1 < md->digest_roof &&
 		    pd->payload_type == ISAKMP_NEXT_v2SK) {
@@ -711,9 +626,9 @@ void lswlog_msg_digest(struct lswlog *buf, const struct msg_digest *md)
 		} else {
 			sep = ",";
 		}
-		lswlog_enum_enum_short(buf, &payload_type_names,
-				       ike_version,
-				       pd->payload_type);
+		jam_enum_enum_short(buf, &payload_type_names,
+				    ike_version,
+				    pd->payload_type);
 	}
-	lswlogs(buf, term);
+	jam_string(buf, term);
 }
