@@ -15,11 +15,11 @@
 
 extern void list_crl_fetch_requests(struct fd *whackfd, bool utc);
 
-extern void start_crl_fetch_helper(void);
-extern void stop_crl_fetch_helper(void);
+extern void start_crl_fetch_helper(struct logger *logger);
+extern void stop_crl_fetch_helper(struct logger *logger);
 
 extern void free_crl_fetch(void);
-extern void check_crls(void);
+extern void check_crls(struct logger *logger);
 
 extern char *curl_iface;
 extern long curl_timeout;
