@@ -42,7 +42,8 @@ extern void main_outI1(struct fd *whack_sock,
 		       struct state *predecessor,
 		       lset_t policy,
 		       unsigned long try,
-		       const threadtime_t *inception);
+		       const threadtime_t *inception,
+		       chunk_t sec_label);
 
 /* extern initiator_function aggr_outI1; */
 extern void aggr_outI1(struct fd *whack_sock,
@@ -50,7 +51,8 @@ extern void aggr_outI1(struct fd *whack_sock,
 		       struct state *predecessor,
 		       lset_t policy,
 		       unsigned long try,
-		       const threadtime_t *inception);
+		       const threadtime_t *inception,
+		       chunk_t sec_label);
 
 extern void send_v1_delete(struct state *st);
 
