@@ -106,14 +106,15 @@ void main_outI1(struct fd *whack_sock,
 		struct state *predecessor,
 		lset_t policy,
 		unsigned long try,
-		const threadtime_t *inception)
+		const threadtime_t *inception,
+		chunk_t sec_label)
 {
-	struct ike_sa *ike = new_v1_istate(whack_sock);
+	struct ike_sa *ike = new_v1_istate(c, whack_sock);
 	struct state *st = &ike->sa;
 	statetime_t start = statetime_backdate(st, inception);
 
 	/* set up new state */
-	initialize_new_state(st, c, policy, try);
+	initialize_new_state(st, policy, try);
 
 	change_state(st, STATE_MAIN_I1);
 
@@ -121,7 +122,7 @@ void main_outI1(struct fd *whack_sock,
 		add_pending(whack_sock, ike, c, policy, 1,
 			    predecessor == NULL ?
 			    SOS_NOBODY : predecessor->st_serialno,
-			    true /* part of initiate */);
+			    sec_label, true /* part of initiate */);
 	}
 
 	if (predecessor == NULL) {
@@ -519,12 +520,11 @@ stf_status main_inI1_outR1(struct state *unused_st UNUSED,
 	}
 
 	/* random source ports are handled by find_host_connection */
-	c = find_host_connection(&md->iface->local_endpoint, &md->sender,
-				 POLICY_IKEV1_ALLOW, POLICY_AGGRESSIVE | POLICY_IKEV1_ALLOW);
+	c = find_host_connection(IKEv1, &md->iface->local_endpoint, &md->sender,
+				 LEMPTY, POLICY_AGGRESSIVE);
 
 	if (c == NULL) {
-		lset_t policy = preparse_isakmp_sa_body(sa_pd->pbs) |
-			POLICY_IKEV1_ALLOW;
+		lset_t policy = preparse_isakmp_sa_body(sa_pd->pbs);
 
 		/*
 		 * Other IKE clients, such as strongswan, send the XAUTH
@@ -545,8 +545,8 @@ stf_status main_inI1_outR1(struct state *unused_st UNUSED,
 		 * but Food Groups kind of assumes one.
 		 */
 		{
-			struct connection *d = find_host_connection(&md->iface->local_endpoint, NULL,
-								    policy, POLICY_XAUTH | POLICY_AGGRESSIVE | POLICY_IKEV1_ALLOW);
+			struct connection *d = find_host_connection(IKEv1, &md->iface->local_endpoint, NULL,
+								    policy, POLICY_XAUTH | POLICY_AGGRESSIVE);
 
 			while (d != NULL) {
 				if (d->kind == CK_GROUP) {
@@ -565,25 +565,22 @@ stf_status main_inI1_outR1(struct state *unused_st UNUSED,
 					 * Opportunistic or Shunt:
 					 * pick tightest match
 					 */
-					if (addrinsubnet(
-						&md->sender,
-						&d->spd.that.client) &&
-					    (c == NULL ||
-					     !subnetinsubnet(
-						&c->spd.that.client,
-						&d->spd.that.client))) {
+					if (endpoint_in_selector(&md->sender, &d->spd.that.client) &&
+					    (c == NULL || selector_in_selector(&c->spd.that.client,
+									       &d->spd.that.client))) {
 						c = d;
 					}
 				}
-				d = find_next_host_connection(d->hp_next,
-					policy, POLICY_XAUTH | POLICY_AGGRESSIVE | POLICY_IKEV1_ALLOW);
+				d = find_next_host_connection(IKEv1, d->hp_next,
+							      policy, POLICY_XAUTH | POLICY_AGGRESSIVE);
 			}
 		}
 
 		if (c == NULL) {
+			policy_buf pb;
 			llog(RC_LOG_SERIOUS, md->md_logger,
 			     "initial Main Mode message received but no connection has been authorized with policy %s",
-			     bitnamesof(sa_policy_bit_names, policy));
+			     str_policy(policy, &pb));
 			/* XXX notification is in order! */
 			return STF_IGNORE;
 		}
@@ -628,13 +625,11 @@ stf_status main_inI1_outR1(struct state *unused_st UNUSED,
 	}
 
 	/* Set up state */
-	struct ike_sa *ike = new_v1_rstate(md);
+	struct ike_sa *ike = new_v1_rstate(c, md);
 	struct state *st = md->st = &ike->sa;
 
 	passert(!st->st_oakley.doing_xauth);
 
-	update_state_connection(st, c);
-
 	st->st_try = 0; /* not our job to try again from start */
 	/* only as accurate as connection */
 	st->st_policy = c->policy & ~POLICY_IPSEC_MASK;
@@ -2165,7 +2160,7 @@ void send_v1_delete(struct state *st)
 			struct isakmp_delete isad = {
 				.isad_doi = ISAKMP_DOI_IPSEC,
 				.isad_spisize = sizeof(ipsec_spi_t),
-				.isad_protoid = ns->proto->ikev1,
+				.isad_protoid = ns->proto->ikev1_protocol_id,
 				.isad_nospi = 1,
 			};
 
