@@ -420,7 +420,7 @@ static bool decode_net_id(struct isakmp_ipsec_id *id,
 			/* XXX Could send notification back */
 			return false;
 		}
-		happy(endtosubnet(&temp_address, net, HERE));
+		*net = subnet_from_address(&temp_address);
 		subnet_buf b;
 		dbg("%s is %s", which, str_subnet(net, &b));
 		break;
@@ -594,18 +594,23 @@ void quick_outI1(struct fd *whack_sock,
 		 struct connection *c,
 		 lset_t policy,
 		 unsigned long try,
-		 so_serial_t replacing)
+		 so_serial_t replacing,
+		 chunk_t sec_label)
 {
-	struct state *st = ikev1_duplicate_state(isakmp_sa, whack_sock);
-	update_state_connection(st, c);
+	struct state *st = ikev1_duplicate_state(c, isakmp_sa, whack_sock);
 	passert(c != NULL);
 
 	st->st_policy = policy;
 	st->st_try = try;
 
 	if (c->spd.this.sec_label.ptr != NULL) {
-		dbg("pending phase 2 with security context \"%.*s\"",
+		dbg("pending phase 2 with base security context \"%.*s\"",
 		    (int)c->spd.this.sec_label.len, c->spd.this.sec_label.ptr);
+		if (sec_label.ptr != NULL) {
+			st->st_acquired_sec_label = clone_hunk(sec_label, "st_acquired_sec_label");
+			dbg("pending phase 2 with 'instance' security context \"%.*s\"",
+				(int)sec_label.len, sec_label.ptr);
+		}
 	}
 
 	st->st_myuserprotoid = c->spd.this.protocol;
@@ -642,7 +647,11 @@ void quick_outI1(struct fd *whack_sock,
 	}
 
 	LLOG_JAMBUF(RC_LOG, st->st_logger, buf) {
-		jam(buf, "initiating Quick Mode %s", prettypolicy(policy));
+		jam(buf, "initiating Quick Mode IKEv1");
+		if (policy != LEMPTY) {
+			jam(buf, "+");
+			jam_policy(buf, policy);
+		}
 		if (replacing != SOS_NOBODY) {
 			jam(buf, " to replace #%lu", replacing);
 		}
@@ -789,8 +798,9 @@ static stf_status quick_outI1_continue_tail(struct state *st,
 		lset_t pm = st->st_policy & (POLICY_ENCRYPT |
 					     POLICY_AUTHENTICATE |
 					     (can_do_IPcomp ? POLICY_COMPRESS : 0));
+		policy_buf pb;
 		dbg("emitting quick defaults using policy %s",
-		     bitnamesof(sa_policy_bit_names, pm));
+		    str_policy(pm, &pb));
 
 		if (!ikev1_out_sa(&rbody,
 				  &ipsec_sadb[pm >> POLICY_IPSEC_SHIFT],
@@ -830,7 +840,7 @@ static stf_status quick_outI1_continue_tail(struct state *st,
 	    !(st->st_policy & POLICY_TUNNEL) &&
 	    LHAS(st->hidden_variables.st_nat_traversal, NATED_HOST)) {
 		/** Send NAT-OA if our address is NATed */
-		if (!nat_traversal_add_natoa(&rbody, st, true /* initiator */)) {
+		if (!v1_nat_traversal_add_initiator_natoa(&rbody, st)) {
 			return STF_INTERNAL_ERROR;
 		}
 	}
@@ -951,9 +961,7 @@ stf_status quick_inI1_outR1(struct state *p1st, struct msg_digest *md)
 		if (id_pd->payload.ipsec_id.isaiid_idtype == ID_FQDN) {
 			log_state(RC_LOG_SERIOUS, p1st,
 				  "Applying workaround for MS-818043 NAT-T bug");
-			zero(&b.peers.net);
-			happy(endtosubnet(&c->spd.that.host_addr,
-					  &b.peers.net, HERE));
+			b.peers.net = subnet_from_address(&c->spd.that.host_addr);
 		}
 		/* End Hack for MS 818043 NAT-T Update */
 
@@ -1001,7 +1009,7 @@ stf_status quick_inI1_outR1(struct state *p1st, struct msg_digest *md)
 			nat_traversal_natoa_lookup(md, &hv, p1st->st_logger);
 
 			if (address_is_specified(&hv.st_nat_oa)) {
-				endtosubnet(&hv.st_nat_oa, &b.peers.net, HERE);
+				b.peers.net = subnet_from_address(&hv.st_nat_oa);
 				subnet_buf buf;
 				log_state(RC_LOG_SERIOUS, p1st,
 					  "IDci was FQDN: %s, using NAT_OA=%s %d as IDci",
@@ -1012,12 +1020,11 @@ stf_status quick_inI1_outR1(struct state *p1st, struct msg_digest *md)
 		}
 	} else {
 		/* implicit IDci and IDcr: peer and self */
-		if (endpoint_type(&c->spd.this.host_addr) !=
-		    endpoint_type(&c->spd.that.host_addr))
+		if (address_type(&c->spd.this.host_addr) != address_type(&c->spd.that.host_addr))
 			return STF_FAIL;
 
-		happy(endtosubnet(&c->spd.this.host_addr, &b.my.net, HERE));
-		happy(endtosubnet(&c->spd.that.host_addr, &b.peers.net, HERE));
+		b.my.net = subnet_from_address(&c->spd.this.host_addr);
+		b.peers.net = subnet_from_address(&c->spd.that.host_addr);
 		b.peers.proto = b.my.proto = 0;
 		b.peers.port = b.my.port = 0;
 	}
@@ -1154,7 +1161,6 @@ static stf_status quick_inI1_outR1_tail(struct verify_oppo_bundle *b)
 		/* fill in the client's true port */
 		if (c->spd.that.has_port_wildcard) {
 			int port = b->peers.port;
-			update_endpoint_port(&c->spd.that.host_addr, ip_hport(port));
 			update_selector_hport(&c->spd.that.client, port);
 
 			c->spd.that.port = b->peers.port;
@@ -1194,16 +1200,13 @@ static stf_status quick_inI1_outR1_tail(struct verify_oppo_bundle *b)
 
 	/* create our new state */
 	{
-		struct state *const st = ikev1_duplicate_state(p1st, null_fd);
+		struct state *const st = ikev1_duplicate_state(c, p1st, null_fd);
 
 		/* first: fill in missing bits of our new state object
 		 * note: we don't copy over st_peer_pubkey, the public key
 		 * that authenticated the ISAKMP SA.  We only need it in this
 		 * routine, so we can "reach back" to p1st to get it.
 		 */
-		if (st->st_connection != c) {
-			update_state_connection(st, c);
-		}
 
 		st->st_try = 0; /* not our job to try again from start */
 
@@ -1640,9 +1643,8 @@ stf_status quick_inR1_outI2_tail(struct state *st, struct msg_digest *md)
 				memcpy(idfqdn, IDcr->pbs.cur, idlen);
 				idfqdn[idlen] = '\0';
 
-				endtosubnet(&st->hidden_variables.st_nat_oa,
-					    &st->st_connection->spd.that.client,
-					    HERE);
+				st->st_connection->spd.that.client =
+					selector_from_address(&st->hidden_variables.st_nat_oa);
 				subnet_buf buf;
 				log_state(RC_LOG_SERIOUS, st,
 					  "IDcr was FQDN: %s, using NAT_OA=%s as IDcr",
