@@ -305,7 +305,7 @@ static void validate_address(struct p_dns_req *dnsr, unsigned char *addr)
 {
 	struct state *st = state_with_serialno(dnsr->so_serial);
 	ip_address ipaddr;
-	const struct ip_info *afi = address_type(&st->st_remote_endpoint);
+	const struct ip_info *afi = endpoint_type(&st->st_remote_endpoint);
 
 	if (dnsr->qtype != LDNS_RR_TYPE_A) {
 		return;
@@ -315,19 +315,20 @@ static void validate_address(struct p_dns_req *dnsr, unsigned char *addr)
 	if (data_to_address(addr, afi->ip_size, afi, &ipaddr) != NULL)
 		return;
 
-	if (!sameaddr(&ipaddr, &st->st_remote_endpoint)) {
-		address_buf ra, rb;
+	if (!endpoint_address_eq(&st->st_remote_endpoint, &ipaddr)) {
+		endpoint_buf ra;
+		address_buf rb;
 		dbg(" forward address of IDi %s do not match remote address %s != %s",
 		    dnsr->qname,
-		    str_address(&st->st_remote_endpoint, &ra),
+		    str_endpoint(&st->st_remote_endpoint, &ra),
 		    str_address(&ipaddr, &rb));
 		return;
 	}
 
 	dnsr->fwd_addr_valid = TRUE;
-	address_buf ra;
+	endpoint_buf ra;
 	dbg("address of IDi %s match remote address %s",
-	    dnsr->qname, str_address(&st->st_remote_endpoint, &ra));
+	    dnsr->qname, str_endpoint(&st->st_remote_endpoint, &ra));
 }
 
 static err_t parse_rr(struct p_dns_req *dnsr, ldns_pkt *ldnspkt)
