@@ -51,9 +51,9 @@ static struct redirect_dests active_dests = { NULL, NULL };
 
 const char *global_redirect_to(void)
 {
-	return global_dests.whole == NULL || *global_dests.whole == '\0' ?
-		"<unset>" :
-		global_dests.whole;
+	if (global_dests.whole == NULL)
+		return ""; /* allows caller to strlen() */
+	return global_dests.whole;
 }
 
 static void free_redirect_dests(struct redirect_dests *dests)
@@ -562,9 +562,9 @@ void send_active_redirect_in_informational(struct state *st)
 		v2_msgid_update_sent(ike, &ike->sa, NULL /* new exchange */, MESSAGE_REQUEST);
 	}
 
-	ipstr_buf b;
+	endpoint_buf b;
 	dbg("redirection %ssent to peer %s", e == STF_OK ? "" : "not ",
-	    sensitive_ipstr(&st->st_remote_endpoint, &b));
+	    str_endpoint(&st->st_remote_endpoint, &b));
 }
 
 stf_status process_IKE_SA_INIT_v2N_REDIRECT_response(struct ike_sa *ike,
