@@ -47,7 +47,7 @@ bool v2_process_ts_response(struct child_sa *child,
 bool v2_process_ts_request(struct child_sa *child,
 			   const struct msg_digest *md);
 
-struct traffic_selector ikev2_end_to_ts(const struct end *e);
+struct traffic_selector ikev2_end_to_ts(const struct end *e, chunk_t sec_label);
 
 stf_status v2_emit_ts_payloads(const struct child_sa *cst,
 			       pb_stream *outpbs,
