@@ -334,7 +334,8 @@ extern unsigned shunt_count(void);
 
 struct bare_shunt **bare_shunt_ptr(const ip_subnet *ours,
 				   const ip_subnet *peers,
-				   int transport_proto);
+				   int transport_proto,
+				   const char *why);
 
 /* A netlink header defines EM_MAXRELSPIS, the max number of SAs in a group.
  * Is there a PF_KEY equivalent?
@@ -349,10 +350,9 @@ struct bare_shunt **bare_shunt_ptr(const ip_subnet *ours,
  * Simple rule: use a string literal.
  */
 struct xfrm_user_sec_ctx_ike; /* forward declaration of tag */
-extern void record_and_initiate_opportunistic(const ip_selector *our_client,
-					      const ip_selector *peer_client,
-					      unsigned transport_proto,
-					      const chunk_t *sec_label,
+extern void record_and_initiate_opportunistic(const ip_endpoint *our_client,
+					      const ip_endpoint *peer_client,
+					      const chunk_t sec_label,
 					      const char *why);
 extern void init_kernel(struct logger *logger);
 
@@ -364,8 +364,8 @@ extern void migration_down(struct connection *c,  struct state *st);
 
 extern bool delete_bare_shunt(const ip_address *src, const ip_address *dst,
 			      int transport_proto, ipsec_spi_t shunt_spi,
-			      const char *why,
-			      struct logger *logger);
+			      bool skip_xfrm_raw_eroute_delete,
+			      const char *why, struct logger *logger);
 
 extern bool replace_bare_shunt(const ip_address *src, const ip_address *dst,
 			       policy_prio_t policy_prio,
