@@ -25,7 +25,6 @@
 
 #include <stdio.h>
 #include <stdlib.h>
-#include <ctype.h>
 #include <stdarg.h>
 #include <syslog.h>
 #include <errno.h>
@@ -37,17 +36,13 @@
 #include <sys/types.h>
 #include <fcntl.h>
 
-#include "libreswan/pfkeyv2.h"
-
 #include "sysdep.h"
 #include "constants.h"
 #include "lswconf.h"
 #include "lswfips.h"
-#include "lswlog.h"
 
 #include "defs.h"
 #include "log.h"
-#include "peerlog.h"
 #include "server.h"
 #include "state.h"
 #include "id.h"
@@ -62,12 +57,9 @@
 #include "ike_alg_integ.h"
 #include "plutoalg.h"
 /* for show_virtual_private: */
-#include "virtual.h"	/* needs connections.h */
 #include "crypto.h"
-
 #include "ip_address.h" /* for jam_address */
 
-#include "db_ops.h"
 
 #include "pluto_stats.h"
 
@@ -80,34 +72,23 @@ void linux_audit_conn(const struct state *st UNUSED, enum linux_audit_kind op UN
 
 #include <libaudit.h>
 
-#if __GNUC__ >= 7
-	/*
-	 * GCC 7+ warns about the following calls that truncate a string using
-	 * snprintf().  But here we are truncating the log message for a reason.
-	 */
-#pragma GCC diagnostic push
-#pragma GCC diagnostic ignored "-Wformat-truncation"
-#endif
-
-void linux_audit_init(int do_audit)
+void linux_audit_init(int do_audit, struct logger *logger)
 {
-	libreswan_log("Linux audit support [enabled]");
+	llog(RC_LOG, logger, "Linux audit support [enabled]");
 	/* test and log if audit is enabled on the system */
 	int audit_fd;
 	audit_fd = audit_open();
 	if (audit_fd < 0) {
 		if (errno == EINVAL || errno == EPROTONOSUPPORT ||
 			errno == EAFNOSUPPORT) {
-			loglog(RC_LOG_SERIOUS,
-				"Warning: kernel has no audit support");
+			llog(RC_LOG_SERIOUS, logger,
+				    "Warning: kernel has no audit support");
 			close(audit_fd);
 			log_to_audit = FALSE;
 			return;
 		} else {
-			loglog(RC_LOG_SERIOUS,
-				"FATAL: audit_open() failed : %s",
-				strerror(errno));
-			exit_pluto(PLUTO_EXIT_AUDIT_FAIL);
+			fatal_errno(PLUTO_EXIT_AUDIT_FAIL, logger, errno,
+				    "FATAL: audit_open() failed");
 		}
 	} else {
 		if (do_audit)
@@ -115,19 +96,18 @@ void linux_audit_init(int do_audit)
 	}
 	close(audit_fd);
 	if (do_audit)
-		libreswan_log("Linux audit activated");
+		llog(RC_LOG, logger, "Linux audit activated");
 }
 
-static void linux_audit(const int type, const char *message, const char *laddr, const int result)
+static void linux_audit(const int type, const char *message, const char *laddr, const int result,
+			struct logger *logger)
 {
 	int audit_fd, rc;
 
 	audit_fd = audit_open();
 	if (audit_fd < 0) {
-		loglog(RC_LOG_SERIOUS,
-		       "FATAL: audit_open() failed : %s",
-		       strerror(errno));
-		exit_pluto(PLUTO_EXIT_AUDIT_FAIL);
+		fatal_errno(PLUTO_EXIT_AUDIT_FAIL, logger, errno,
+			    "FATAL: audit_open() failed");
 	}
 
 	/*
@@ -147,10 +127,8 @@ static void linux_audit(const int type, const char *message, const char *laddr,
 	rc = audit_log_user_message(audit_fd, type, message, NULL, laddr, NULL, result);
 	close(audit_fd);
 	if (rc < 0) {
-		loglog(RC_LOG_SERIOUS,
-			"FATAL: audit log failed: %s",
-			strerror(errno));
-		exit_pluto(PLUTO_EXIT_AUDIT_FAIL);
+		fatal_errno(PLUTO_EXIT_AUDIT_FAIL, logger, errno,
+			    "FATAL: audit log failed");
 	}
 }
 
@@ -164,7 +142,7 @@ void linux_audit_conn(const struct state *st, enum linux_audit_kind op)
 	}
 
 	char audit_str[AUDIT_LOG_SIZE];
-	jambuf_t buf = ARRAY_AS_JAMBUF(audit_str);
+	struct jambuf buf = ARRAY_AS_JAMBUF(audit_str);
 	struct connection *const c = st->st_connection;
 	/* we need to free() this */
 	char *conn_encode = audit_encode_nv_string("conn-name", c->name, 0);
@@ -174,7 +152,11 @@ void linux_audit_conn(const struct state *st, enum linux_audit_kind op)
 	case LAK_PARENT_DESTROY:
 	case LAK_PARENT_FAIL:
 	{
-		bool initiator = (st->st_original_role == ORIGINAL_INITIATOR) || IS_PHASE1_INIT(st->st_state);
+		bool initiator = (st->st_ike_version == IKEv2 ? st->st_sa_role == SA_INITIATOR :
+#ifdef USE_IKEv1
+				  st->st_ike_version == IKEv1 ? IS_PHASE1_INIT(st->st_state) :
+#endif
+				  pexpect(false));
 		/* head */
 		jam(&buf, "op=%s direction=%s %s connstate=%lu ike-version=%s",
 		    op == LAK_PARENT_DESTROY ? "destroy" : "start", /* fail to start logged under op=start */
@@ -193,7 +175,7 @@ void linux_audit_conn(const struct state *st, enum linux_audit_kind op)
 				jam_string(&buf, (c->policy & POLICY_PSK) ? "PRESHARED_KEY" :
 					(c->policy & POLICY_RSASIG) ? "RSA_SIG" : "unknown");
 			else
-				lswlog_enum_short(&buf, &oakley_auth_names, st->st_oakley.auth);
+				jam_enum_short(&buf, &oakley_auth_names, st->st_oakley.auth);
 		}
 
 		jam(&buf, " cipher=%s ksize=%d",
@@ -238,7 +220,7 @@ void linux_audit_conn(const struct state *st, enum linux_audit_kind op)
 
 		jam(&buf, " prf=%s", prfname); /* could be "none" */
 		jam(&buf, " pfs=%s", (st->st_oakley.ta_dh == NULL ? "none"
-				      : st->st_oakley.ta_dh->common.name));
+				      : st->st_oakley.ta_dh->common.fqn));
 
 		/* XXX: empty SPI to keep tests happy */
 		jam(&buf, " ");
@@ -315,7 +297,8 @@ void linux_audit_conn(const struct state *st, enum linux_audit_kind op)
 	linux_audit((op == LAK_CHILD_START || op == LAK_CHILD_DESTROY || op == LAK_CHILD_FAIL) ?
 			AUDIT_CRYPTO_IPSEC_SA : AUDIT_CRYPTO_IKE_SA,
 			audit_str, laddr,
-			(op == LAK_PARENT_FAIL || op == LAK_CHILD_FAIL) ? AUDIT_RESULT_FAIL : AUDIT_RESULT_OK);
+		    (op == LAK_PARENT_FAIL || op == LAK_CHILD_FAIL) ? AUDIT_RESULT_FAIL : AUDIT_RESULT_OK,
+		    st->st_logger);
 }
 #if __GNUC__ >= 7
 #pragma GCC diagnostic pop
