@@ -62,6 +62,7 @@ void add_pending(struct fd *whack_sock,
 		 lset_t policy,
 		 unsigned long try,
 		 so_serial_t replacing,
+		 const chunk_t sec_label,
 		 bool part_of_initiate)
 {
 	/* look for duplicate pending IPsec SA's, skip add operation */
@@ -91,6 +92,7 @@ void add_pending(struct fd *whack_sock,
 	p->replacing = replacing;
 	p->pend_time = mononow();
 	p->part_of_initiate = part_of_initiate; /* useful */
+	p->sec_label = sec_label;
 
 	/*
 	 * If this is part of an initiate then there's already enough
@@ -207,6 +209,23 @@ static void delete_pending(struct pending **pp)
 	struct pending *p = *pp;
 	*pp = p->next;
 
+	if (DBGP(DBG_BASE)) {
+		if (p->connection == NULL) {
+			/*
+			 * ??? when does this happen?
+			 *
+			 * Look for p->connection=NULL below.  When
+			 * unpend()ing, the pending connection's
+			 * ownership is transfered to the state.
+			 */
+			DBG_log("removing pending policy {%p}; connection ownership transfered", p);
+		} else {
+			connection_buf cib;
+			DBG_log("removing pending policy {%p} with connection "PRI_CONNECTION"",
+				p, pri_connection(p->connection, &cib));
+		}
+	}
+
 	if (p->connection != NULL) {
 		/* above unlink means C is no longer pending */
 		pexpect(!connection_is_pending(p->connection));
@@ -216,17 +235,6 @@ static void delete_pending(struct pending **pp)
 	}
 	close_any(&p->whack_sock); /*on-heap*/
 
-	if (DBGP(DBG_BASE)) {
-		if (p->connection == NULL) {
-			/* ??? when does this happen? */
-			DBG_log("removing pending policy for no connection {%p}", p);
-		} else {
-			connection_buf cib;
-			DBG_log("removing pending policy for "PRI_CONNECTION" {%p}",
-				pri_connection(p->connection, &cib), p);
-		}
-	}
-
 	pfree(p);
 }
 
@@ -277,7 +285,7 @@ void unpend(struct ike_sa *ike, struct connection *cc)
 #ifdef USE_IKEv1
 				quick_outI1(p->whack_sock, &ike->sa, p->connection,
 					    p->policy,
-					    p->try, p->replacing);
+					    p->try, p->replacing, empty_chunk);
 #endif
 				break;
 			default:
