@@ -23,9 +23,8 @@
 
 #define SD_REPORT_NO_STATUS 0
 
-extern void pluto_sd_init(void);
+extern void pluto_sd_init(struct logger *logger);
 extern void pluto_sd(int action, int status);
-extern void sd_watchdog_event(void);
 
 #endif /* USE_SYSTEMD_WATCHDOG */
 
