@@ -1565,9 +1565,6 @@ int main(int argc, char **argv)
 		}
 	}
 
-	init_constants();
-	init_pluto_constants();
-
 	/*
 	 * Initialize logging then switch to the real logger.
 	 */
@@ -1581,6 +1578,9 @@ int main(int argc, char **argv)
 	free_logger(&logger, HERE);
 	logger = &global_logger;
 
+	init_constants();
+	init_pluto_constants();
+
 	pluto_init_nss(oco->nssdir, logger);
 	if (libreswan_fipsmode()) {
 		/*
@@ -1874,7 +1874,7 @@ void show_setup_plutomain(struct show *s)
 	show_comment(s,
 		"global-redirect=%s, global-redirect-to=%s",
 		enum_name(&allow_global_redirect_names, global_redirect),
-		global_redirect_to()
+		strlen(global_redirect_to()) > 0 ? global_redirect_to() : "<unset>"
 		);
 
 #ifdef HAVE_LABELED_IPSEC
