@@ -423,15 +423,15 @@ static void whack_process(const struct whack_message *const m, struct show *s)
 				set_debugging(cur_debugging | new_debugging);
 				LSWDBGP(DBG_BASE, buf) {
 					jam(buf, "old debugging ");
-					jam_enum_lset_short(buf, &debug_names,
-							    "+", old_debugging);
+					jam_lset_short(buf, &debug_names,
+						       "+", old_debugging);
 					jam(buf, " + ");
 					jam_lmod(buf, &debug_names, "+", m->debugging);
 				}
 				LSWDBGP(DBG_BASE, buf) {
 					jam(buf, "new debugging = ");
-					jam_enum_lset_short(buf, &debug_names,
-							    "+", new_debugging);
+					jam_lset_short(buf, &debug_names,
+						       "+", new_debugging);
 				}
 				set_debugging(new_debugging);
 				for (unsigned i = 0; i < m->nr_impairments; i++) {
@@ -580,6 +580,34 @@ static void whack_process(const struct whack_message *const m, struct show *s)
 		find_states_and_redirect(m->name, m->active_redirect_dests, whackfd);
 	}
 
+	if (m->global_redirect_to) {
+		dbg("whack: global_redirect_to %s", m->global_redirect_to);
+		if (streq(m->global_redirect_to, "<none>")) {
+			set_global_redirect_dests("");
+			global_redirect = GLOBAL_REDIRECT_NO;
+			llog(RC_LOG, logger,
+				"cleared global redirect targets and disabled global redirects");
+		} else {
+			set_global_redirect_dests(m->global_redirect_to);
+			llog(RC_LOG, logger,
+				"set global redirect target to %s", global_redirect_to());
+		}
+	}
+
+	if (m->global_redirect) {
+		if (m->global_redirect != GLOBAL_REDIRECT_NO && strlen(global_redirect_to()) == 0) {
+			llog(RC_LOG_SERIOUS, logger,
+			    "ipsec whack: --global-redirect set to no as there are no active redirect targets");
+			global_redirect = GLOBAL_REDIRECT_NO;
+		} else {
+			dbg("whack: global_redirect %d", m->global_redirect);
+			global_redirect = m->global_redirect;
+			llog(RC_LOG, logger,
+				"set global redirect to %s",
+				enum_name(&allow_global_redirect_names, global_redirect));
+		}
+	}
+
 	/* update any socket buffer size before calling listen */
 	if (m->ike_buf_size != 0) {
 		dbg("whack: ike_buf_size %lu", m->ike_buf_size);
@@ -756,11 +784,15 @@ static void whack_process(const struct whack_message *const m, struct show *s)
 			whack_log(RC_DEAF, whackfd,
 				  "need --listen before opportunistic initiation");
 		} else {
-			initiate_ondemand(&m->oppo_my_client,
-					  &m->oppo_peer_client, m->oppo_proto,
+			const ip_protocol *protocol = protocol_by_ipproto(m->oppo.ipproto);
+			ip_endpoint local = endpoint3(protocol, &m->oppo.local.address,
+						      m->oppo.local.port);
+			ip_endpoint remote = endpoint3(protocol, &m->oppo.remote.address,
+						       m->oppo.remote.port);
+			initiate_ondemand(&local, &remote,
 					  /*held*/false,
 					  /*background*/m->whack_async,
-					  NULL, "whack", logger);
+					  empty_chunk, "whack", logger);
 		}
 	}
 
