@@ -4,6 +4,7 @@
  * Copyright (C) 2012-2013 Paul Wouters <paul@libreswan.org>
  * Copyright (C) 2013 Florian Weimer <fweimer@redhat.com>
  * Copyright (C) 2019 Andrew Cagney
+ * Copyright (C) 2017 Mayank Totale <mtotale@gmail.com>
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the
@@ -19,16 +20,21 @@
 #ifndef _SERVER_H
 #define _SERVER_H
 
-#include <sys/queue.h>
-#include <event2/event.h>	/* from libevent devel */
+#include <event2/event.h>		/* from libevent devel */
 #include <event2/event_struct.h>
+#include <event2/listener.h>
+
 #include "timer.h"
 #include "err.h"
 #include "ip_address.h"
 #include "ip_endpoint.h"
+#include "diag.h"
 
 struct state;
 struct msg_digest;
+struct bufferevent;
+struct iface_endpoint;
+struct show;
 
 extern char *pluto_vendorid;
 
@@ -38,73 +44,53 @@ extern struct sockaddr_un ctl_addr;     /* address of control (whack) socket */
 extern int info_fd;                     /* file descriptor of control (info) socket */
 extern struct sockaddr_un info_addr;    /* address of control (info) socket */
 
-extern err_t init_ctl_socket(void);
+diag_t init_ctl_socket(struct logger *logger);
 extern void delete_ctl_socket(void);
 
+extern stf_status create_tcp_interface(struct state *st); /* TCP: terrible name? */
+
 extern bool listening;  /* should we pay attention to IKE messages? */
-extern enum ddos_mode pluto_ddos_mode; /* auto-detect or manual? */
-extern enum seccomp_mode pluto_seccomp_mode;
+extern bool pluto_listen_udp;
+extern bool pluto_listen_tcp;
+
+extern enum ddos_mode pluto_ddos_mode; /* auto-detect or manual */
+extern enum global_ikev1_policy pluto_ikev1_pol; /* accept, drop or reject */
 extern unsigned int pluto_max_halfopen; /* Max allowed half-open IKE SA's before refusing */
 extern unsigned int pluto_ddos_threshold; /* Max incoming IKE before activating DCOOKIES */
 extern deltatime_t pluto_shunt_lifetime; /* lifetime before we cleanup bare shunts (for OE) */
 extern unsigned int pluto_sock_bufsize; /* pluto IKE socket buffer */
 extern bool pluto_sock_errqueue; /* Enable MSG_ERRQUEUE on IKE socket */
 
-/* interface: a terminal point for IKE traffic, IPsec transport mode
- * and IPsec tunnels.
- * Essentially:
- * - an IP device (eg. eth1), and
- * - its partner, an ipsec device (eg. ipsec0), and
- * - their shared IP address (eg. 10.7.3.2)
- * Note: the port for IKE is always implicitly UDP/pluto_port.
- *
- * The iface is a unique IP address on a system. It may be used
- * by multiple port numbers. In general, two conns have the same
- * interface if they have the same iface_port->iface_alias.
- */
-struct iface_dev {
-	LIST_ENTRY(iface_dev) id_entry;
-	int id_count;
-	char *id_vname; /* virtual (ipsec) device name */
-	char *id_rname; /* real device name */
-	bool id_nic_offload;
-};
-
-struct iface_port {
-	struct iface_dev   *ip_dev;
-	ip_endpoint local_endpoint;	/* interface IP address:port */
-	int fd;                 /* file descriptor of socket for IKE UDP messages */
-	struct iface_port *next;
-	bool ike_float;
-	enum { IFN_ADD, IFN_KEEP, IFN_DELETE } change;
-	struct pluto_event *pev;
-};
-
-extern struct iface_port  *interfaces;   /* public interfaces */
 extern enum pluto_ddos_mode ddos_mode;
 extern bool pluto_drop_oppo_null;
 
-extern struct iface_port *find_iface_port_by_local_endpoint(ip_endpoint *local_endpoint);
-extern bool use_interface(const char *rifn);
-extern void find_ifaces(bool rm_dead);
-extern void show_ifaces_status(struct fd *whackfd);
-extern void free_ifaces(void);
-extern void show_debug_status(void);
-extern void show_fips_status(struct fd *whackfd);
-extern void call_server(char *conffile);
+extern void show_debug_status(struct show *s);
+extern void show_fips_status(struct show *s);
+extern void run_server(char *conffile, struct logger *logger) NEVER_RETURNS;
+
+/* XXX: grr, need pointer to function else NEVER_RETURNS is ignored */
+typedef void (*server_stopped_cb)(int r) NEVER_RETURNS;
+extern void stop_server(server_stopped_cb cb);
+
 typedef void event_callback_routine(evutil_socket_t, const short, void *);
-void fire_timer_photon_torpedo(struct event **evp, event_callback_fn cb, void *arg,
+
+void fire_timer_photon_torpedo(struct event **evp,
+			       event_callback_fn cb, void *arg,
 			       const deltatime_t delay);
+void attach_fd_read_sensor(struct event **ev, evutil_socket_t fd,
+			   event_callback_fn cb, void *arg);
+
 extern struct pluto_event *add_fd_read_event_handler(evutil_socket_t fd,
 						     event_callback_fn cb, void *arg,
 						     const char *name);
 extern void delete_pluto_event(struct pluto_event **evp);
+
 extern void link_pluto_event_list(struct pluto_event *e);
 bool ev_before(struct pluto_event *pev, deltatime_t delay);
 extern void set_pluto_busy(bool busy);
-extern void set_whack_pluto_ddos(enum ddos_mode mode);
+extern void set_whack_pluto_ddos(enum ddos_mode mode, struct logger *logger);
 
-extern void init_server(void);
+extern void init_server(struct logger *logger);
 extern void free_server(void);
 
 extern struct event_base *get_pluto_event_base(void);
@@ -112,7 +98,7 @@ extern struct event_base *get_pluto_event_base(void);
 /*
  * Schedule an event (with no timeout) to resume a suspended state.
  * SERIALNO (so_serial_t) is used to identify the state because the
- * state object may not be directly accessable (as happens with worker
+ * state object may not be directly accessible (as happens with worker
  * threads).
  *
  * For instance: a worker thread needing to resume processing of a
@@ -136,7 +122,7 @@ extern struct event_base *get_pluto_event_base(void);
  * XXX: resume_cb should return stf_status, but doing this is a mess.
  */
 
-typedef stf_status resume_cb(struct state *st, struct msg_digest **mdp,
+typedef stf_status resume_cb(struct state *st, struct msg_digest *md,
 			     void *context);
 void schedule_resume(const char *name, so_serial_t serialno,
 		     resume_cb *callback, void *context);
@@ -152,33 +138,4 @@ typedef void callback_cb(struct state *st, void *context);
 void schedule_callback(const char *name, so_serial_t serialno,
 		       callback_cb *callback, void *context);
 
-/*
- * Create a child process using fork()
- *
- * Typically used to perform a thread unfriendly operation, such as
- * calling PAM.
- *
- * On callback:
- *
- * ST either points at the state matching SERIALNO, or NULL (SERIALNO
- * is either SOS_NOBODY or the state doesn't exist).  A CB expecting a
- * state back MUST check ST before processing.  Caller sets CUR_STATE
- * so don't play with that.
- *
- * MDP either points at the unsuspended contents of .st_suspended_md,
- * or NULL.  On return, if *MDP is non-NULL, then it will be released.
- *
- * STATUS is the child processes exit code as returned by things like
- * waitpid().
- */
-
-typedef void pluto_fork_cb(struct state *st, struct msg_digest **mdp,
-			   int status, void *context);
-extern int pluto_fork(const char *name, so_serial_t serialno,
-		      int op(void *context),
-		      pluto_fork_cb *callback, void *context);
-
-bool check_incoming_msg_errqueue(const struct iface_port *ifp, const char *before);
-void check_outgoing_msg_errqueue(const struct iface_port *ifp, const char *before);
-
 #endif /* _SERVER_H */
