@@ -23,12 +23,10 @@
  *
  */
 
-
 #include "sysdep.h"
 #include "constants.h"
 #include "lswconf.h"
 #include "lswfips.h"
-#include "lswlog.h"
 
 #include "defs.h"
 #include "log.h"
@@ -37,59 +35,177 @@
 #include "pluto_stats.h"
 #include "connections.h"
 #include "kernel.h"
-#include "virtual.h"
+#include "virtual_ip.h"
 #include "plutoalg.h"
 #include "crypto.h"
-#include "db_ops.h"
-
-static void show_system_security(struct fd *whackfd)
-{
-	int selinux = libreswan_selinux();
-#ifdef FIPS_CHECK
-	bool fips = libreswan_fipsmode();
-#else
-	int fips = FALSE;
+#include "ikev1_db_ops.h"
+#include "kernel_xfrm_interface.h"
+#include "iface.h"
+#include "show.h"
+#ifdef HAVE_SECCOMP
+#include "pluto_seccomp.h"
 #endif
 
-	whack_comment(whackfd, " ");     /* spacer */
+struct show {
+	/*
+	 * where to send the output
+	 */
+	struct logger *logger;
+	/*
+	 * Should the next output be preceded by a blank line?
+	 */
+	enum separation { NO_SEPARATOR = 1, HAD_OUTPUT, SEPARATE_NEXT_OUTPUT, } separator;
+	/*
+	 * Where to build the messages.
+	 */
+	char scratch_array[LOG_WIDTH];
+	struct jambuf scratch_jambuf;
+};
 
-	whack_comment(whackfd, "fips mode=%s;", fips ? "enabled" : "disabled");
+struct show *alloc_show(struct logger *logger)
+{
+	struct show s = {
+		.separator = NO_SEPARATOR,
+		.logger = logger,
+	};
+	return clone_thing(s, "on show");
+}
 
-	whack_comment(whackfd, "SElinux=%s",
+static void blank_line(struct show *s)
+{
+	/* XXX: must not use s->jambuf */
+	char blank_buf[sizeof(" "/*\0*/) + 1/*canary*/ + 1/*why-not*/];
+	struct jambuf buf = ARRAY_AS_JAMBUF(blank_buf);
+	jam_string(&buf, " ");
+	jambuf_to_logger(&buf, s->logger, RC_COMMENT|WHACK_STREAM);
+}
+
+void free_show(struct show **sp)
+{
+	{
+		struct show *s = *sp;
+		switch (s->separator) {
+		case NO_SEPARATOR:
+		case HAD_OUTPUT:
+			break;
+		case SEPARATE_NEXT_OUTPUT:
+			blank_line(s);
+			break;
+		default:
+			bad_case(s->separator);
+		}
+	}
+	pfree(*sp);
+	*sp = NULL;
+}
+
+void show_separator(struct show *s)
+{
+	switch (s->separator) {
+	case NO_SEPARATOR:
+		break;
+	case HAD_OUTPUT:
+	case SEPARATE_NEXT_OUTPUT:
+		s->separator = SEPARATE_NEXT_OUTPUT;
+		break;
+	default:
+		bad_case(s->separator);
+		break;
+	}
+}
+
+struct jambuf *show_jambuf(struct show *s)
+{
+	s->scratch_jambuf = ARRAY_AS_JAMBUF(s->scratch_array);
+	return &s->scratch_jambuf;
+}
+
+struct logger *show_logger(struct show *s)
+{
+	return s->logger;
+}
+
+void jambuf_to_show(struct jambuf *buf, struct show *s, enum rc_type rc)
+{
+	switch (s->separator) {
+	case NO_SEPARATOR:
+	case HAD_OUTPUT:
+		break;
+	case SEPARATE_NEXT_OUTPUT:
+		blank_line(s);
+		break;
+	default:
+		bad_case(s->separator);
+	}
+	pexpect(rc == RC_RAW ||
+		rc == RC_COMMENT ||
+		rc == RC_INFORMATIONAL_TRAFFIC/*show_state_traffic()*/ ||
+		rc == RC_UNKNOWN_NAME/*show_traffic_status()*/);
+	jambuf_to_logger(buf, s->logger, rc|WHACK_STREAM);
+	s->separator = HAD_OUTPUT;
+}
+
+void show_comment(struct show *s, const char *message, ...)
+{
+	struct jambuf *buf = show_jambuf(s);
+	va_list args;
+	va_start(args, message);
+	jam_va_list(buf, message, args);
+	va_end(args);
+	jambuf_to_show(buf, s, RC_COMMENT);
+}
+
+void show_raw(struct show *s, const char *message, ...)
+{
+	struct jambuf *buf = show_jambuf(s);
+	va_list args;
+	va_start(args, message);
+	jam_va_list(buf, message, args);
+	va_end(args);
+	jambuf_to_show(buf, s, RC_RAW);
+}
+
+static void show_system_security(struct show *s)
+{
+	int selinux = libreswan_selinux(show_logger(s));
+	bool fips = libreswan_fipsmode();
+
+	show_separator(s);
+	show_comment(s, "fips mode=%s;", fips ? "enabled" : "disabled");
+	show_comment(s, "SElinux=%s",
 		selinux == 0 ? "disabled" : selinux == 1 ? "enabled" : "indeterminate");
 #ifdef HAVE_SECCOMP
-	whack_comment(whackfd, "seccomp=%s",
-		pluto_seccomp_mode == SECCOMP_ENABLED ? "enabled" :
-			pluto_seccomp_mode == SECCOMP_TOLERANT ? "tolerant" : "disabled");
+	show_comment(s, "seccomp=%s",
+		     pluto_seccomp_mode == SECCOMP_ENABLED ? "enabled" :
+		     pluto_seccomp_mode == SECCOMP_TOLERANT ? "tolerant" : "disabled");
 #else
-	whack_comment(whackfd, "seccomp=unsupported");
+	show_comment(s, "seccomp=unsupported");
 #endif
-	whack_comment(whackfd, " ");     /* spacer */
 }
 
-void show_global_status(struct fd *whackfd)
+void show_global_status(struct show *s)
 {
-	show_globalstate_status(whackfd);
-	show_pluto_stats(whackfd);
+	show_globalstate_status(s);
+	show_pluto_stats(s);
 }
 
-void show_status(struct fd *whackfd)
+void show_status(struct show *s)
 {
-	show_kernel_interface(whackfd);
-	show_ifaces_status(whackfd);
-	whack_comment(whackfd, " ");     /* spacer */
-	show_system_security(whackfd);
-	show_setup_plutomain(whackfd);
-	show_debug_status();
-	show_setup_natt(whackfd);
-	show_virtual_private(whackfd);
-	kernel_alg_show_status(whackfd);
-	ike_alg_show_status(whackfd);
-	db_ops_show_status(whackfd);
-	show_connections_status(whackfd);
-	show_states_status(whackfd, FALSE);
-#if defined(NETKEY_SUPPORT) || defined(KLIPS)
-	show_shunt_status(whackfd);
+	show_kernel_interface(s);
+	show_ifaces_status(s);
+	show_system_security(s);
+	show_setup_plutomain(s);
+	show_debug_status(s);
+	show_setup_natt(s);
+	show_virtual_private(s);
+	show_kernel_alg_status(s);
+	show_ike_alg_status(s);
+	show_db_ops_status(s);
+	show_connections_status(s);
+	show_brief_status(s);
+	show_states(s);
+#if defined(XFRM_SUPPORT)
+	show_shunt_status(s);
 #endif
 }
 
@@ -150,7 +266,7 @@ static void connection_state(struct state *st, void *data)
 		if (lc->conn->host_pair != st->st_connection->host_pair ||
 		    !same_peer_ids(lc->conn, st->st_connection, NULL))
 			return;
-		/* phase1 is shared with another connnection */
+		/* phase1 is shared with another connection */
 	}
 
 	/* ignore undefined states (i.e. just deleted) */
@@ -167,12 +283,15 @@ static void connection_state(struct state *st, void *data)
 		} else {
 			if (lc->phase1 < p1_init)
 				lc->phase1 = p1_init;
+#ifdef USE_IKEv1
+/* PW: This seems an unintensional flaw of using ikev1 only checks ??? */
 			if (IS_ISAKMP_ENCRYPTED(st->st_state->kind) &&
 			    lc->phase1 < p1_encrypt)
 				lc->phase1 = p1_encrypt;
 			if (IS_ISAKMP_AUTHENTICATED(st->st_state) &&
 			    lc->phase1 < p1_auth)
 				lc->phase1 = p1_auth;
+#endif
 		}
 	} else {
 		lc->phase1 = p1_down;
@@ -182,10 +301,12 @@ static void connection_state(struct state *st, void *data)
 	if (st->st_connection != lc->conn)
 		return;
 
+#ifdef USE_IKEv1
 	if (IS_PHASE15(st->st_state->kind)) {
 		if (lc->tunnel < tun_phase15)
 			lc->tunnel = tun_phase15;
 	}
+#endif
 
 	if (IS_CHILD_SA(st)) {
 		if (lc->tunnel < tun_phase2)
@@ -207,22 +328,18 @@ void binlog_state(struct state *st, enum state_kind new_state)
 		return;
 
 	if (st == NULL) {
-		DBG(DBG_CONTROLMORE, DBG_log(
-			    "log_state() called without state"));
+		dbg("log_state() called without state");
 		return;
 	}
 
 	struct connection *conn = st->st_connection;
 
 	if (conn == NULL || st->st_connection->name == NULL) {
-		DBG(DBG_CONTROLMORE,
-		    DBG_log("log_state() called without st->st_connection or without st->st_connection->name"));
+		dbg("log_state() called without st->st_connection or without st->st_connection->name");
 		return;
 	}
 
-	DBG(DBG_CONTROLMORE,
-	    DBG_log("log_state called for state update for connection %s ",
-		    conn->name));
+	dbg("log_state called for state update for connection %s ", conn->name);
 
 	struct log_conn_info lc = {
 		.conn = conn,
@@ -244,15 +361,13 @@ void binlog_state(struct state *st, enum state_kind new_state)
 		uint32_t sv = IPsecSAref2NFmark(st->st_ref) | LOG_CONN_STATSVAL(&lc);
 
 		if (conn->statsval == sv) {
-			DBG(DBG_CONTROLMORE,
-			    DBG_log("log_state for connection %s state change signature (%d) matches last one - skip logging",
-				    conn->name, sv));
+			dbg("log_state for connection %s state change signature (%d) matches last one - skip logging",
+			    conn->name, sv);
 			return;
 		}
 		conn->statsval = sv;
-		DBG(DBG_CONTROLMORE,
-			DBG_log("log_state set state change signature for connection %s to %d",
-				conn->name, sv));
+		dbg("log_state set state change signature for connection %s to %d",
+		    conn->name, sv);
 	}
 
 	const char *tun;
@@ -285,9 +400,8 @@ void binlog_state(struct state *st, enum state_kind new_state)
 	case p2_up:	p2 = "up";	break;
 	default:	p2 = "down";	break;
 	}
-	DBG(DBG_CONTROLMORE,
-	    DBG_log("log_state calling %s for connection %s with tunnel(%s) phase1(%s) phase2(%s)",
-		    pluto_stats_binary, conn->name, tun, p1, p2));
+	dbg("log_state calling %s for connection %s with tunnel(%s) phase1(%s) phase2(%s)",
+	    pluto_stats_binary, conn->name, tun, p1, p2);
 
 	/* ??? tun, p1, p2 cannot be NULL -- why handle that case? */
 
@@ -298,25 +412,24 @@ void binlog_state(struct state *st, enum state_kind new_state)
 		 "%s ipsec-tunnel-%s tunnel %s \\; "
 		 "%s ipsec-tunnel-%s phase1 %s \\; "
 		 "%s ipsec-tunnel-%s phase2 %s \\; "
-		 "%s ipsec-tunnel-%s nfmark-me/him 0x%x/0x%x",
+		 "%s ipsec-tunnel-%s nfmark-me/peer 0x%x/0x%x",
 
 		 pluto_stats_binary,
 		 conn->interface ? "push" : "drop", conn->name,
-		 conn->interface ? conn->interface->ip_dev->id_vname : "",
+		 (conn->xfrmi != NULL && conn->xfrmi->name != NULL) ? conn->xfrmi->name : "",
 		 tun ? "push" : "drop", conn->name, tun ? tun : "",
 		 p1  ? "push" : "drop", conn->name, p1  ? p1  : "",
 		 p2  ? "push" : "drop", conn->name, p2  ? p2  : "",
-		 (st->st_ref || st->st_refhim) ? "push" : "drop", conn->name,
+		 (st->st_ref || st->st_ref_peer) ? "push" : "drop", conn->name,
 		 st->st_ref == IPSEC_SAREF_NA ? IPSEC_SAREF_NA :
 		 st->st_ref == IPSEC_SAREF_NULL ? 0u :
 		 IPsecSAref2NFmark(st->st_ref) | IPSEC_NFMARK_IS_SAREF_BIT
 		 ,
-		 st->st_refhim == IPSEC_SAREF_NA ? IPSEC_SAREF_NA :
-		 st->st_refhim == IPSEC_SAREF_NULL ? 0u :
-		 IPsecSAref2NFmark(st->st_refhim) | IPSEC_NFMARK_IS_SAREF_BIT);
+		 st->st_ref_peer == IPSEC_SAREF_NA ? IPSEC_SAREF_NA :
+		 st->st_ref_peer == IPSEC_SAREF_NULL ? 0u :
+		 IPsecSAref2NFmark(st->st_ref_peer) | IPSEC_NFMARK_IS_SAREF_BIT);
 	if (system(buf) == -1) {
-		loglog(RC_LOG_SERIOUS, "statsbin= failed to send status update notification");
+		log_state(RC_LOG_SERIOUS, st, "statsbin= failed to send status update notification");
 	}
-	DBG(DBG_CONTROLMORE,
-	    DBG_log("log_state for connection %s completed", conn->name));
+	dbg("log_state for connection %s completed", conn->name);
 }
