@@ -391,7 +391,7 @@ union sas { struct child_sa child; struct ike_sa ike; struct state st; };
  * Caller must insert_state().
  */
 
-static struct state *new_state(enum ike_version ike_version,
+static struct state *new_state(struct connection *c,
 			       const ike_spi_t ike_initiator_spi,
 			       const ike_spi_t ike_responder_spi,
 			       enum sa_type sa_type, struct fd *whackfd)
@@ -405,8 +405,8 @@ static struct state *new_state(enum ike_version ike_version,
 		.st_state = &state_undefined,
 		.st_serialno = next_so++,
 		.st_inception = realnow(),
-		.st_ike_version = ike_version,
 		.st_establishing_sa = sa_type,
+		.st_connection = c,
 		.st_ike_spis = {
 			.initiator = ike_initiator_spi,
 			.responder = ike_responder_spi,
@@ -427,17 +427,17 @@ static struct state *new_state(enum ike_version ike_version,
 	return st;
 }
 
-struct ike_sa *new_v1_istate(struct fd *whackfd)
+struct ike_sa *new_v1_istate(struct connection *c, struct fd *whackfd)
 {
-	struct state *st = new_state(IKEv1, ike_initiator_spi(),
+	struct state *st = new_state(c, ike_initiator_spi(),
 				     zero_ike_spi, IKE_SA, whackfd);
 	struct ike_sa *ike = pexpect_ike_sa(st);
 	return ike;
 }
 
-struct ike_sa *new_v1_rstate(struct msg_digest *md)
+struct ike_sa *new_v1_rstate(struct connection *c, struct msg_digest *md)
 {
-	struct state *st = new_state(IKEv1, md->hdr.isa_ike_spis.initiator,
+	struct state *st = new_state(c, md->hdr.isa_ike_spis.initiator,
 				     ike_responder_spi(&md->sender, md->md_logger),
 				     IKE_SA, null_fd);
 	struct ike_sa *ike = pexpect_ike_sa(st);
@@ -445,14 +445,15 @@ struct ike_sa *new_v1_rstate(struct msg_digest *md)
 	return ike;
 }
 
-struct ike_sa *new_v2_ike_state(const struct state_v2_microcode *transition,
+struct ike_sa *new_v2_ike_state(struct connection *c,
+				const struct state_v2_microcode *transition,
 				enum sa_role sa_role,
 				const ike_spi_t ike_initiator_spi,
 				const ike_spi_t ike_responder_spi,
-				struct connection *c, lset_t policy,
+				lset_t policy,
 				int try, struct fd *whack_sock)
 {
-	struct state *st = new_state(IKEv2, ike_initiator_spi, ike_responder_spi,
+	struct state *st = new_state(c, ike_initiator_spi, ike_responder_spi,
 				     IKE_SA, whack_sock);
 	struct ike_sa *ike = pexpect_ike_sa(st);
 	ike->sa.st_sa_role = sa_role;
@@ -460,7 +461,7 @@ struct ike_sa *new_v2_ike_state(const struct state_v2_microcode *transition,
 	change_state(&ike->sa, fs->kind);
 	set_v2_transition(&ike->sa, transition, HERE);
 	v2_msgid_init_ike(ike);
-	initialize_new_state(&ike->sa, c, policy, try);
+	initialize_new_state(&ike->sa, policy, try);
 	return ike;
 }
 
@@ -1067,25 +1068,49 @@ void delete_state_tail(struct state *st)
 			free_any_iface_endpoint(p);
 		}
 	}
-	st->st_interface = NULL;
 
 	/*
-	 * Danger!
-	 *
+	 * Release stored IKE fragments. This is a union in st so only
+	 * call one!  XXX: should be a union???
+	 */
+	switch (st->st_connection->ike_version) {
+	case IKEv1:
+#ifdef USE_IKEv1
+		free_v1_message_queues(st);
+#endif
+		break;
+	case IKEv2:
+		free_v2_message_queues(st);
+		break;
+	default:
+		bad_case(st->st_connection->ike_version);
+	}
+
+	/*
+	 * This, effectively,  deletes any ISAKMP SA that this state
+	 * represents - lookups for this state no longer work.
+	 */
+	del_state_from_db(st);
+
+	/*
 	 * Break the STATE->CONNECTION link.  If CONNECTION is an
 	 * instance, then it too will be deleted.
 	 *
-	 * Needs to be done before the state is removed from it's DB
-	 * as it can otherwise leak the connection (or explode because
-	 * it was removed from the list). ???
+	 * - checks ST's POLICY_UP
+	 *
+	 *   is the established IKE SA being revived and, hence, the
+	 *   connection should not be deleted
+	 *
+	 * - checks for a another state still using the connection
+	 *
+	 *   since this state was removed from the CONNECTION -> STATE
+	 *   hash table this succeeding means that there must be a
+	 *   second state using the connection
 	 */
-	update_state_connection(st, NULL);
-
-	/*
-	 * Effectively, this deletes any ISAKMP SA that this state
-	 * represents
-	 */
-	del_state_from_db(st);
+	connection_delete_unused_instance(&st->st_connection, st,
+					  st->st_logger->global_whackfd);
+	pexpect(st->st_connection == NULL);
+	st->st_interface = NULL;
 
 	v2_msgid_free(st);
 
@@ -1100,23 +1125,6 @@ void delete_state_tail(struct state *st)
 #endif
 	pubkey_delref(&st->st_peer_pubkey, HERE);
 
-	/*
-	 * Release stored IKE fragments. This is a union in st so only
-	 * call one!  XXX: should be a union???
-	 */
-	switch (st->st_ike_version) {
-	case IKEv1:
-#ifdef USE_IKEv1
-		free_v1_message_queues(st);
-#endif
-		break;
-	case IKEv2:
-		free_v2_message_queues(st);
-		break;
-	default:
-		bad_case(st->st_ike_version);
-	}
-
 	/*
 	 * Free the accepted proposal first, it points into the
 	 * proposals.
@@ -1192,6 +1200,7 @@ void delete_state_tail(struct state *st)
 	pfreeany(st->st_seen_cfg_banner);
 
 	free_chunk_content(&st->st_seen_sec_label);
+	free_chunk_content(&st->st_acquired_sec_label);
 
 	free_chunk_content(&st->st_no_ppk_auth);
 
@@ -1415,7 +1424,7 @@ void delete_states_by_peer(const struct fd *whackfd, const ip_address *peer)
 			    str_endpoint(&this->st_remote_endpoint, &b),
 			    peerstr);
 
-			if (sameaddr(&this->st_remote_endpoint, peer)) {
+			if (endpoint_address_eq(&this->st_remote_endpoint, peer)) {
 				if (ph1 == 0 && IS_IKE_SA(this)) {
 					whack_log(RC_COMMENT, whackfd,
 						  "peer %s for connection %s crashed; replacing",
@@ -1439,7 +1448,8 @@ void delete_states_by_peer(const struct fd *whackfd, const ip_address *peer)
  * Caller must schedule an event for this object so that it doesn't leak.
  * Caller must insert_state().
  */
-static struct state *duplicate_state(struct state *st,
+static struct state *duplicate_state(struct connection *c,
+				     struct state *st,
 				     enum sa_type sa_type,
 				     struct fd *whackfd)
 {
@@ -1451,7 +1461,7 @@ static struct state *duplicate_state(struct state *st,
 		st->st_outbound_time = mononow();
 	}
 
-	nst = new_state(st->st_ike_version,
+	nst = new_state(c,
 			st->st_ike_spis.initiator,
 			st->st_ike_spis.responder,
 			sa_type, whackfd);
@@ -1461,8 +1471,6 @@ static struct state *duplicate_state(struct state *st,
 	    st->st_serialno, pri_connection(st->st_connection, &cib),
 	    nst->st_serialno, sa_type == IPSEC_SA ? "IPSEC SA" : "IKE SA");
 
-	update_state_connection(nst, st->st_connection);
-
 	if (sa_type == IPSEC_SA) {
 		nst->st_oakley = st->st_oakley;
 	}
@@ -1536,22 +1544,27 @@ static struct state *duplicate_state(struct state *st,
 	nst->st_seen_cfg_domains = clone_str(st->st_seen_cfg_domains, "child st_seen_cfg_domains");
 	nst->st_seen_cfg_banner = clone_str(st->st_seen_cfg_banner, "child st_seen_cfg_banner");
 
+	nst->st_acquired_sec_label = st->st_acquired_sec_label;
+	nst->st_seen_sec_label = st->st_seen_sec_label;
+
 	return nst;
 }
 
-struct state *ikev1_duplicate_state(struct state *st,
+struct state *ikev1_duplicate_state(struct connection *c,
+				    struct state *st,
 				    struct fd *whackfd)
 {
-	return duplicate_state(st, IPSEC_SA, whackfd);
+	return duplicate_state(c, st, IPSEC_SA, whackfd);
 }
 
-struct child_sa *new_v2_child_state(struct ike_sa *ike,
+struct child_sa *new_v2_child_state(struct connection *c,
+				    struct ike_sa *ike,
 				    enum sa_type sa_type,
 				    enum sa_role role,
 				    enum state_kind kind,
 				    struct fd *whackfd)
 {
-	struct state *cst = duplicate_state(&ike->sa, sa_type, whackfd);
+	struct state *cst = duplicate_state(c, &ike->sa, sa_type, whackfd);
 	cst->st_sa_role = role;
 	struct child_sa *child = pexpect_child_sa(cst);
 	v2_msgid_init_child(ike, child);
@@ -1852,16 +1865,15 @@ bool ikev2_viable_parent(const struct ike_sa *ike)
 struct state *find_phase1_state(const struct connection *c, lset_t ok_states)
 {
 	struct state *best = NULL;
-	bool is_ikev2 = (c->policy & POLICY_IKEV1_ALLOW) == LEMPTY;
 
 	dbg("FOR_EACH_STATE_... in %s", __func__);
 	struct state *st;
 	FOR_EACH_STATE_NEW2OLD(st) {
 		if (LHAS(ok_states, st->st_state->kind) &&
-		    (st->st_ike_version == IKEv2) == is_ikev2 &&
+		    c->ike_version == st->st_connection->ike_version &&
 		    c->host_pair == st->st_connection->host_pair &&
 		    same_peer_ids(c, st->st_connection, NULL) &&
-		    sameaddr(&st->st_remote_endpoint, &c->spd.that.host_addr) &&
+		    endpoint_address_eq(&st->st_remote_endpoint, &c->spd.that.host_addr) &&
 		    IS_IKE_SA(st) &&
 		    (best == NULL || best->st_serialno < st->st_serialno))
 		{
@@ -1980,7 +1992,6 @@ void fmt_state(struct state *st, const monotime_t now,
 {
 	/* what the heck is interesting about a state? */
 	const struct connection *c = st->st_connection;
-	char inst[CONN_INST_BUF];
 	char dpdbuf[128];
 	char traffic_buf[512], *mbcp;
 	const char *np1 = c->newest_isakmp_sa == st->st_serialno ?
@@ -1991,7 +2002,8 @@ void fmt_state(struct state *st, const monotime_t now,
 	const char *eo = c->spd.eroute_owner == st->st_serialno ?
 			 "; eroute owner" : "";
 
-	fmt_conn_instance(c, inst);
+	connection_buf cib;
+	const char *inst = str_connection_instance(c, &cib);
 
 	dpdbuf[0] = '\0';	/* default to empty string */
 	if (IS_IPSEC_SA_ESTABLISHED(st)) {
@@ -2626,7 +2638,7 @@ bool update_mobike_endpoints(struct ike_sa *ike, const struct msg_digest *md)
 		break;
 	case MESSAGE_REQUEST:
 		/* MOBIKE responder processing request */
-		c->spd.that.host_addr = md->sender;
+		c->spd.that.host_addr = endpoint_address(&md->sender);
 		dbg("%s() %s.host_port: %u->%u", __func__, c->spd.that.leftright,
 		    c->spd.that.host_port, endpoint_hport(&md->sender));
 		c->spd.that.host_port = endpoint_hport(&md->sender);
@@ -3144,11 +3156,8 @@ static void list_state_event(struct show *s, struct state *st,
 				    deltasecs(monotimediff(pe->ev_time, now)));
 			}
 			if (st->st_connection != NULL) {
-				/* fmt_connection(buf, st->st_connection); */
-				char cib[CONN_INST_BUF];
-				jam(buf, " \"%s\"%s",
-				    st->st_connection->name,
-				    fmt_conn_instance(st->st_connection, cib));
+				connection_buf cib;
+				jam(buf, " "PRI_CONNECTION, pri_connection(st->st_connection, &cib));
 			}
 			jam(buf, "  #%lu", st->st_serialno);
 		}
