@@ -318,7 +318,7 @@ struct state {
 	 * XXX: Can these attributes be made "const".  Probably,
 	 * new_state() could use clone_thing(const state on stack).
 	 */
-	/*const*/ enum ike_version st_ike_version;	/* IKEv1, IKEv2, ... */
+#define st_ike_version st_connection->ike_version /* XXX: DBD */
 	/*const*/ enum sa_type st_establishing_sa;	/* where is this state going? */
 
 	bool st_ikev2_anon;                     /* is this an anonymous IKEv2 state? */
@@ -414,7 +414,7 @@ struct state {
 	/* collected received fragments */
 	struct v2_ike_rfrags *st_v2_rfrags;
 	struct v2_outgoing_fragment *st_v2_outgoing[MESSAGE_ROLE_ROOF];
-	struct v2_incomming_fragments *st_v2_incomming[MESSAGE_ROLE_ROOF];
+	struct v2_incoming_fragments *st_v2_incoming[MESSAGE_ROLE_ROOF];
 
 	bool st_viable_parent;	/* can initiate new CERAET_CHILD_SA */
 	struct ikev2_proposal *st_accepted_ike_proposal;
@@ -737,6 +737,7 @@ struct state {
 	uint8_t st_reply_xchg;
 	bool st_peer_wants_null;		/* We received IDr payload of type ID_NULL (and we allow POLICY_AUTH_NULL */
 	chunk_t st_seen_sec_label;
+	chunk_t st_acquired_sec_label;
 };
 
 /*
@@ -777,18 +778,23 @@ extern bool states_use_connection(const struct connection *c);
 
 /* state functions */
 
-struct ike_sa *new_v1_istate(struct fd *whackfd);
-struct ike_sa *new_v1_rstate(struct msg_digest *md);
+struct ike_sa *new_v1_istate(struct connection *c, struct fd *whackfd);
+struct ike_sa *new_v1_rstate(struct connection *c, struct msg_digest *md);
+struct state *ikev1_duplicate_state(struct connection *c, struct state *st, struct fd *whackfd);
 
-struct ike_sa *new_v2_ike_state(const struct state_v2_microcode *transition,
+struct ike_sa *new_v2_ike_state(struct connection *c, 
+				const struct state_v2_microcode *transition,
 				enum sa_role sa_role,
 				const ike_spi_t ike_initiator_spi,
 				const ike_spi_t ike_responder_spi,
-				struct connection *c, lset_t policy,
+				lset_t policy,
 				int try, struct fd *whack_sock);
 /* could eventually be IKE or CHILD SA */
-struct child_sa *new_v2_child_state(struct ike_sa *st, enum sa_type sa_type,
-				    enum sa_role sa_role, enum state_kind kind,
+struct child_sa *new_v2_child_state(struct connection *c,
+				    struct ike_sa *ike,
+				    enum sa_type sa_type,
+				    enum sa_role sa_role,
+				    enum state_kind kind,
 				    struct fd *whackfd);
 
 void set_v1_transition(struct state *st, const struct state_v1_microcode *transition, where_t where);
@@ -811,8 +817,6 @@ enum send_delete { PROBABLY_SEND_DELETE, DONT_SEND_DELETE, };
 extern void delete_ike_family(struct ike_sa *ike, enum send_delete send_delete);
 extern void ikev2_schedule_next_child_delete(struct state *st, struct ike_sa *ike);
 
-struct state *ikev1_duplicate_state(struct state *st, struct fd *whackfd);
-
 extern struct state
 	*state_with_serialno(so_serial_t sn),
 	*find_phase2_state_to_delete(const struct state *p1st, uint8_t protoid,
@@ -840,7 +844,6 @@ extern struct state *find_v1_info_state(const ike_spis_t *ike_spis,
 					msgid_t msgid);
 
 extern void initialize_new_state(struct state *st,
-				 struct connection *c,
 				 lset_t policy,
 				 int try);
 
