@@ -20,29 +20,18 @@
  *
  */
 
-#include <stdlib.h>
-#include <string.h>
-#include <ctype.h>
+#include <stdlib.h>	/* for NULL et.al. */
 
-#include "sysdep.h"
-#include "constants.h"
-#include "defs.h"
-#include "log.h"
-#include "id.h"
-#include "x509.h"
-#include "certs.h"
-#include "connections.h"
-#include "packet.h"
-#include "demux.h"
-#include "server.h"
-#include "whack.h"
-#include "vendor.h"
-#include "quirks.h"
-#include "kernel.h"
+#include "libreswan.h"		/* for libreswan_vendorid */
 #include "crypt_hash.h"
 #include "ike_alg_hash.h"
 
-#include "nat_traversal.h"
+#include "defs.h"		/* for so_serial_t et.al. */
+#include "known_vendorid.h"
+#include "vendor.h"
+#include "log.h"
+#include "demux.h"
+#include "connections.h"
 
 /**
  * Listing of interesting but details unknown Vendor IDs:
@@ -583,8 +572,6 @@ static struct vid_struct vid_tab[] = {
 	{ VID_none, 0, NULL, NULL, NULL, 0 }
 };
 
-static const char hexdig[] = "0123456789abcdef";
-
 /*
  * Setup VendorID structs, and populate them
  * FIXME: This functions leaks a little bit, but these are one time leaks:
@@ -592,7 +579,7 @@ static const char hexdig[] = "0123456789abcdef";
  * leak: self-vendor ID, item size: 37
  * leak: 2 * vid->data, item size: 13
  */
-void init_vendorid(void)
+void init_vendorid(struct logger *logger)
 {
 	struct vid_struct *vid;
 
@@ -614,7 +601,8 @@ void init_vendorid(void)
 
 			/* TODO: This use must allowed even with USE_MD5=false */
 			struct crypt_hash *ctx = crypt_hash_init("vendor id",
-								 &ike_alg_hash_md5);
+								 &ike_alg_hash_md5,
+								 logger);
 			crypt_hash_digest_bytes(ctx, "data", d, strlen(vid->data));
 			crypt_hash_final_bytes(&ctx, vidm, MD5_DIGEST_SIZE);
 			vid->vid_len = MD5_DIGEST_SIZE;
@@ -627,7 +615,8 @@ void init_vendorid(void)
 			vid->vid = vidm;
 
 			struct crypt_hash *ctx = crypt_hash_init("vendor id",
-								 &ike_alg_hash_md5);
+								 &ike_alg_hash_md5,
+								 logger);
 			crypt_hash_digest_bytes(ctx, "data", vid->data, strlen(vid->data));
 			crypt_hash_final_bytes(&ctx, hash, MD5_DIGEST_SIZE);
 
@@ -662,38 +651,31 @@ void init_vendorid(void)
 	}
 }
 
-static void vidlog(const char *vidstr, size_t len, struct vid_struct *vid, bool vid_useful)
+static void dbg_vid(struct logger *logger, const char *vidstr, size_t len, struct vid_struct *vid, bool vid_useful)
 {
-	char vid_dump[128];
-
-	if (vid->flags & VID_SUBSTRING_DUMPHEXA) {
-		/* Dump description + Hexa */
-		size_t i, j;
-
-		snprintf(vid_dump, sizeof(vid_dump), "%s ",
-			 vid->descr ? vid->descr : "");
-		for (i = strlen(vid_dump), j = vid->vid_len
-		     ; (j < len) && (i < sizeof(vid_dump) - 2)
-		     ; i += 2, j++) {
-			vid_dump[i] = hexdig[(vidstr[j] >> 4) & 0xF];
-			vid_dump[i + 1] = hexdig[vidstr[j] & 0xF];
+	if (DBGP(DBG_BASE)) {
+		LLOG_JAMBUF(DEBUG_STREAM, logger, buf) {
+			jam(buf, "%s Vendor ID payload [",
+			    vid_useful ? "received" : "ignoring");
+			if (vid->flags & VID_SUBSTRING_DUMPHEXA) {
+				/* Dump description + Hexa */
+				if (vid->descr != NULL) {
+					jam(buf, "%s ", vid->descr);
+				}
+				jam_dump_bytes(buf, vidstr, len);
+			} else if (vid->flags & VID_SUBSTRING_DUMPASCII) {
+				/* Dump ASCII content */
+				/* no descr? */
+				jam_sanitized_bytes(buf, vidstr, len);
+			} else {
+				/* Dump description (descr) */
+				if (vid->descr != NULL) {
+					jam_string(buf, vid->descr);
+				}
+			}
+			jam(buf, "]");
 		}
-		vid_dump[i] = '\0';
-	} else if (vid->flags & VID_SUBSTRING_DUMPASCII) {
-		/* Dump ASCII content */
-		size_t i;
-
-		for (i = 0; i < len && i < sizeof(vid_dump) - 1; i++)
-			vid_dump[i] = isprint(vidstr[i]) ? vidstr[i] : '.';
-		vid_dump[i] = '\0';
-	} else {
-		/* Dump description (descr) */
-		snprintf(vid_dump, sizeof(vid_dump), "%s",
-			 vid->descr ? vid->descr : "");
 	}
-
-	DBG(DBG_CONTROL, DBG_log("%s Vendor ID payload [%s]",
-	       vid_useful ? "received" : "ignoring", vid_dump));
 }
 
 /*
@@ -701,7 +683,7 @@ static void vidlog(const char *vidstr, size_t len, struct vid_struct *vid, bool
  * We don't know about any real IKEv2 vendor id strings yet
  */
 
-static void handle_known_vendorid_v2(struct msg_digest *md UNUSED,
+static void handle_known_vendorid_v2(struct msg_digest *md,
 				  const char *vidstr,
 				  size_t len,
 				  struct vid_struct *vid)
@@ -721,7 +703,7 @@ static void handle_known_vendorid_v2(struct msg_digest *md UNUSED,
 		break;
 	}
 
-	vidlog(vidstr, len, vid, vid_useful);
+	dbg_vid(md->md_logger, vidstr, len, vid, vid_useful);
 }
 
 /*
@@ -738,9 +720,10 @@ static void handle_known_vendorid_v2(struct msg_digest *md UNUSED,
  * @return void
  */
 static void handle_known_vendorid_v1(struct msg_digest *md,
-				  const char *vidstr,
-				  size_t len,
-				  struct vid_struct *vid)
+				     const char *vidstr,
+				     size_t len,
+				     struct vid_struct *vid,
+				     struct logger *logger)
 {
 	bool vid_useful = TRUE; /* tentatively TRUE */
 
@@ -777,13 +760,10 @@ static void handle_known_vendorid_v1(struct msg_digest *md,
 		/* FALL THROUGH */
 	case VID_NATT_RFC:
 		if (md->quirks.qnat_traversal_vid < vid->id) {
-			DBG(DBG_NATT, DBG_log(" quirks.qnat_traversal_vid set to=%d [%s]",
-					      vid->id, vid->descr));
+			dbg(" quirks.qnat_traversal_vid set to=%d [%s]", vid->id, vid->descr);
 			md->quirks.qnat_traversal_vid = vid->id;
 		} else {
-			DBG(DBG_NATT,
-			    DBG_log("Ignoring older NAT-T Vendor ID payload [%s]",
-				    vid->descr));
+			dbg("Ignoring older NAT-T Vendor ID payload [%s]", vid->descr);
 			vid_useful = FALSE;
 		}
 		break;
@@ -803,8 +783,8 @@ static void handle_known_vendorid_v1(struct msg_digest *md,
 		break;
 
 	case VID_SSH_SENTINEL_1_4_1:
-		loglog(RC_LOG_SERIOUS,
-		       "SSH Sentinel 1.4.1 found, setting XAUTH_ACK quirk");
+		llog(RC_LOG_SERIOUS, logger,
+			    "SSH Sentinel 1.4.1 found, setting XAUTH_ACK quirk");
 		md->quirks.xauth_ack_msgid = TRUE;
 		break;
 
@@ -825,7 +805,7 @@ static void handle_known_vendorid_v1(struct msg_digest *md,
 		vid_useful = FALSE;
 		break;
 	}
-	vidlog(vidstr, len, vid, vid_useful);
+	dbg_vid(md->md_logger, vidstr, len, vid, vid_useful);
 }
 
 
@@ -833,12 +813,13 @@ static void handle_known_vendorid(struct msg_digest *md,
 				  const char *vidstr,
 				  size_t len,
 				  struct vid_struct *vid,
-				  bool ikev2)
+				  bool ikev2,
+				  struct logger *logger)
 {
 	if (ikev2)
 		handle_known_vendorid_v2(md, vidstr, len, vid);
 	else
-		handle_known_vendorid_v1(md, vidstr, len, vid);
+		handle_known_vendorid_v1(md, vidstr, len, vid, logger);
 }
 
 /*
@@ -855,7 +836,7 @@ static void handle_known_vendorid(struct msg_digest *md,
  * @return void
  */
 void handle_vendorid(struct msg_digest *md, const char *vid, size_t len,
-		     bool ikev2)
+		     bool ikev2, struct logger *logger)
 {
 	struct vid_struct *pvid;
 
@@ -876,14 +857,16 @@ void handle_vendorid(struct msg_digest *md, const char *vid, size_t len,
 			if (pvid->vid_len == len) {
 				if (memeq(pvid->vid, vid, len)) {
 					handle_known_vendorid(md, vid,
-							      len, pvid, ikev2);
+							      len, pvid, ikev2,
+							      logger);
 					return;
 				}
 			} else if (pvid->vid_len < len &&
 				   (pvid->flags & VID_SUBSTRING)) {
 				if (memeq(pvid->vid, vid, pvid->vid_len)) {
 					handle_known_vendorid(md, vid, len,
-							      pvid, ikev2);
+							      pvid, ikev2,
+							      logger);
 					return;
 				}
 			}
@@ -893,17 +876,14 @@ void handle_vendorid(struct msg_digest *md, const char *vid, size_t len,
 	/*
 	 * Unknown VendorID. Log the beginning.
 	 */
-	char log_vid[2 * MAX_LOG_VID_LEN + 1];
-	size_t i;
-
-	for (i = 0; i < len && i < MAX_LOG_VID_LEN; i++) {
-		log_vid[2 * i] = hexdig[(vid[i] >> 4) & 0xF];
-		log_vid[2 * i + 1] = hexdig[vid[i] & 0xF];
+	LLOG_JAMBUF(RC_LOG_SERIOUS, logger, buf) {
+		jam(buf, "ignoring unknown Vendor ID payload [");
+		jam_dump_bytes(buf, vid, PMIN(len, MAX_LOG_VID_LEN));
+		if (len > MAX_LOG_VID_LEN) {
+			jam(buf, "...");
+		}
+		jam(buf, "]");
 	}
-	log_vid[2 * i] = '\0';
-	loglog(RC_LOG_SERIOUS,
-	       "ignoring unknown Vendor ID payload [%s%s]",
-	       log_vid, (len > MAX_LOG_VID_LEN) ? "..." : "");
 }
 
 /**
@@ -914,7 +894,7 @@ void handle_vendorid(struct msg_digest *md, const char *vid, size_t len,
  * @param vid Int of VendorID to be sent (see vendor.h for the list)
  * @return bool True if successful
  */
-bool out_vid(uint8_t np, pb_stream *outs, unsigned int vid)
+bool out_vid(pb_stream *outs, unsigned int vid)
 {
 	struct vid_struct *pvid;
 
@@ -922,10 +902,9 @@ bool out_vid(uint8_t np, pb_stream *outs, unsigned int vid)
 	for (pvid = vid_tab; pvid->id != vid; pvid++)
 		passert(pvid->id != VID_none); /* we must find what we are trying to send */
 
-	DBG(DBG_EMITTING,
-	    DBG_log("out_vid(): sending [%s]", pvid->descr));
+	dbg("out_vid(): sending [%s]", pvid->descr);
 
-	return ikev1_out_generic_raw(np, &isakmp_vendor_id_desc, outs,
+	return ikev1_out_generic_raw(&isakmp_vendor_id_desc, outs,
 			       pvid->vid, pvid->vid_len, "V_ID");
 }
 
@@ -950,7 +929,7 @@ bool out_vid_set(pb_stream *outs, const struct connection *c)
 {
 	/* cusomizeable Vendor ID */
 	if (c->send_vendorid) {
-		if (!ikev1_out_generic_raw(ISAKMP_NEXT_VID, &isakmp_vendor_id_desc, outs,
+		if (!ikev1_out_generic_raw(&isakmp_vendor_id_desc, outs,
 					pluto_vendorid, strlen(pluto_vendorid), "Pluto Vendor ID")) {
 			return FALSE;
 		}
@@ -958,7 +937,7 @@ bool out_vid_set(pb_stream *outs, const struct connection *c)
 
 #define MAYBE_VID(q, vid) {  \
 	if (q) {  \
-		if (!out_vid(ISAKMP_NEXT_VID, outs, vid)) {  \
+		if (!out_vid(outs, vid)) {  \
 			return FALSE;  \
 		}  \
 	}  \
@@ -979,7 +958,7 @@ bool out_vid_set(pb_stream *outs, const struct connection *c)
 	 * we count on backpatching to fix our np.
 	 */
 
-	if (!out_vid(ISAKMP_NEXT_NONE, outs, VID_MISC_DPD)) {
+	if (!out_vid(outs, VID_MISC_DPD)) {
 		return FALSE;
 	}
 
@@ -998,7 +977,7 @@ bool vid_is_oppo(const char *vid, size_t len)
 		passert(pvid->id != VID_none); /* we must find VID_OPPORTUNISTIC */
 
 	if (pvid->vid_len == len && memeq(vid, pvid->vid, len)) {
-		DBG(DBG_CONTROL, DBG_log("VID_OPPORTUNISTIC received"));
+		dbg("VID_OPPORTUNISTIC received");
 		return TRUE;
 	} else {
 		return FALSE;
