@@ -11,6 +11,7 @@ REF_CONSOLE_FIXUPS="$REF_CONSOLE_FIXUPS cut-postfinal.sed"
 REF_CONSOLE_FIXUPS="$REF_CONSOLE_FIXUPS guest-prompt-sanitize.sed"
 
 REF_CONSOLE_FIXUPS="$REF_CONSOLE_FIXUPS cutout.sed"
+REF_CONSOLE_FIXUPS="$REF_CONSOLE_FIXUPS post-mortem.sed"
 REF_CONSOLE_FIXUPS="$REF_CONSOLE_FIXUPS wilog.sed"
 REF_CONSOLE_FIXUPS="$REF_CONSOLE_FIXUPS routes.sed"
 REF_CONSOLE_FIXUPS="$REF_CONSOLE_FIXUPS klips-debug-sanitize.sed"
@@ -20,6 +21,7 @@ REF_CONSOLE_FIXUPS="$REF_CONSOLE_FIXUPS host-ping-sanitize.sed"
 REF_CONSOLE_FIXUPS="$REF_CONSOLE_FIXUPS namespace.sed"
 REF_CONSOLE_FIXUPS="$REF_CONSOLE_FIXUPS xfrmi.sed"
 # note order, sed goes first
+REF_CONSOLE_FIXUPS="$REF_CONSOLE_FIXUPS sanitize-retransmits.sed"
 REF_CONSOLE_FIXUPS="$REF_CONSOLE_FIXUPS ipsec-look-sanitize.sed"
 REF_CONSOLE_FIXUPS="$REF_CONSOLE_FIXUPS ipsec-look-esp-sanitize.pl"
 REF_CONSOLE_FIXUPS="$REF_CONSOLE_FIXUPS ipsec-ver-remove.sed"
@@ -50,6 +52,8 @@ REF_CONSOLE_FIXUPS="$REF_CONSOLE_FIXUPS guest-ip-xfrm-policy.sed"
 REF_CONSOLE_FIXUPS="$REF_CONSOLE_FIXUPS guest-swan-prep.sed"
 REF_CONSOLE_FIXUPS="$REF_CONSOLE_FIXUPS guest-certutil.sed"
 REF_CONSOLE_FIXUPS="$REF_CONSOLE_FIXUPS guest-cp.sed"
-
+REF_CONSOLE_FIXUPS="$REF_CONSOLE_FIXUPS guest-tcpdump.sed"
+REF_CONSOLE_FIXUPS="$REF_CONSOLE_FIXUPS guest-taskset.sed"
+REF_CONSOLE_FIXUPS="$REF_CONSOLE_FIXUPS ipsec-auto-up.n.sed"
 # this is last
 REF_CONSOLE_FIXUPS="$REF_CONSOLE_FIXUPS guest-prompt-double.sed"
