@@ -1,8 +1,6 @@
-: ==== start ====
-TESTNAME=alias-01
-source /testing/pluto/bin/eastlocal.sh
-
+/testing/guestbin/swan-prep
 ipsec start
 /testing/pluto/bin/wait-until-pluto-started
-
+# connalias
 ipsec auto --add franklin
+echo "initdone"
