@@ -1,8 +1,7 @@
-ipsec look
+../../pluto/bin/ipsec-look.sh
 # ==== cut ====
 ipsec auto --status
 # ==== tuc ====
 ipsec whack --shutdown
 ../bin/check-for-core.sh
 if [ -f /sbin/ausearch ]; then ausearch -r -m avc -ts recent ; fi
-# ==== end ====
