@@ -1,7 +1,8 @@
 /testing/guestbin/swan-prep
 # block plaintext port 22
-#iptables -A INPUT -i eth1 -s 0.0.0.0/0 -p tcp --dport 22 -j LOGDROP
+#iptables -A INPUT -i eth1 -s 0.0.0.0/0 -p tcp --dport 22 -j DROP
 #iptables -I INPUT -m policy --dir in --pol ipsec -j ACCEPT
+(test -z $(ip netns identify) || PATH/sbin/sshd -p 22 > /dev/null 2>/dev/null &)
 ipsec start
 /testing/pluto/bin/wait-until-pluto-started
 ipsec auto --add westnet-eastnet
