@@ -1,11 +1,10 @@
 # A tunnel should have established with non-zero byte counters
-ipsec whack --trafficstatus 
+hostname | grep nic > /dev/null || ipsec whack --trafficstatus
 grep "negotiated connection" /tmp/pluto.log
 # you should see both RSA and NULL
-grep IKEv2_AUTH_ /tmp/pluto.log 
+grep -e 'auth method: ' -e 'hash algorithm identifier' -e ': authenticated using ' /tmp/pluto.log 
 : ==== cut ====
 ipsec auto --status
 : ==== tuc ====
 ../bin/check-for-core.sh
 if [ -f /sbin/ausearch ]; then ausearch -r -m avc -ts recent ; fi
-: ==== end ====
