@@ -1,10 +1,9 @@
 # A tunnel should have established with non-zero byte counters
 hostname | grep nic > /dev/null || ipsec whack --trafficstatus
 # you should see both RSA and NULL
-grep IKEv2_AUTH_ /tmp/pluto.log 
+hostname | grep nic > /dev/null || grep -e 'auth method: ' -e 'hash algorithm identifier' -e ': authenticated using ' /tmp/pluto.log 
 : ==== cut ====
 ipsec auto --status
 : ==== tuc ====
 ../bin/check-for-core.sh
 if [ -f /sbin/ausearch ]; then ausearch -r -m avc -ts recent ; fi
-: ==== end ====
