@@ -5,7 +5,7 @@ cp policies/* /etc/ipsec.d/policies/
 echo "192.1.3.209/32"  >> /etc/ipsec.d/policies/private-or-clear
 ipsec start
 /testing/pluto/bin/wait-until-pluto-started
-ipsec whack --debug-all --impair retransmits
+ipsec whack --impair suppress-retransmits
 # give OE policies time to load
 sleep 5
 echo "initdone"
