@@ -7,7 +7,7 @@ echo "192.1.3.0/24"  >> /etc/ipsec.d/policies/clear-or-private
 ip addr show scope global
 ipsec start
 /testing/pluto/bin/wait-until-pluto-started
-ipsec whack --debug-all --impair retransmits
+ipsec whack --impair suppress-retransmits
 # give OE policies time to load
 sleep 5
 echo "initdone"
