@@ -2,7 +2,7 @@
 hostname | grep nic > /dev/null || ipsec trafficstatus
 ../../pluto/bin/ipsec-look.sh
 # you should see both RSA and NULL
-grep IKEv2_AUTH_ /tmp/pluto.log 
+grep -e 'auth method: ' -e 'hash algorithm identifier' -e ': authenticated using ' /tmp/pluto.log 
 : ==== cut ====
 ipsec auto --status
 ipsec stop
@@ -11,4 +11,3 @@ ip addr show scope global
 : ==== tuc ====
 ../bin/check-for-core.sh
 if [ -f /sbin/ausearch ]; then ausearch -r -m avc -ts recent ; fi
-: ==== end ====
