@@ -2,10 +2,9 @@
 ipsec whack --trafficstatus
 grep "negotiated connection" /tmp/pluto.log
 # you should one RSA and one NULL, asymetric OE
-grep IKEv2_AUTH_ /tmp/pluto.log
+grep -e 'auth method: ' -e 'hash algorithm identifier' -e ': authenticated using ' /tmp/pluto.log
 : ==== cut ====
 ipsec auto --status
 : ==== tuc ====
 ../bin/check-for-core.sh
 if [ -f /sbin/ausearch ]; then ausearch -r -m avc -ts recent ; fi
-: ==== end ====
