@@ -1,5 +1,5 @@
 # you should see one RSA and on NULL only
-grep IKEv2_AUTH_ /tmp/pluto.log
+grep -e 'auth method: ' -e 'hash algorithm identifier' -e ': authenticated using ' /tmp/pluto.log
 # NO ipsec tunnel should be up
 ipsec whack --trafficstatus
 : ==== cut ====
@@ -7,4 +7,3 @@ ipsec auto --status
 : ==== tuc ====
 ../bin/check-for-core.sh
 if [ -f /sbin/ausearch ]; then ausearch -r -m avc -ts recent ; fi
-: ==== end ====
