@@ -2,7 +2,7 @@
 # confirm that the network is alive
 ../../pluto/bin/wait-until-alive -I 192.0.1.254 192.0.2.254
 # ensure that clear text does not get through
-iptables -A INPUT -i eth1 -s 192.0.2.0/24 -j LOGDROP
+iptables -A INPUT -i eth1 -s 192.0.2.0/24 -j DROP
 iptables -I INPUT -m policy --dir in --pol ipsec -j ACCEPT
 # confirm clear text does not get through
 ../../pluto/bin/ping-once.sh --down -I 192.0.1.254 192.0.2.254
@@ -10,4 +10,5 @@ ipsec start
 /testing/pluto/bin/wait-until-pluto-started
 ipsec auto --add westnet-eastnet-ipv4-psk-ikev2
 ipsec auto --status |grep westnet-eastnet-ipv4-psk-ikev2
+ipsec whack --impair suppress-retransmits
 echo "initdone"
