@@ -6,7 +6,7 @@ ipsec status
 sleep 60
 sleep 60
 ping -q -n -c 4 -I 192.0.2.100 192.1.2.23
-grep -E  'EVENT_SA_EXPIRE|EVENT_SA_REPLACE' OUTPUT/road.pluto.log | grep -v pe@ | head -9
-echo "re-authenticateded. The state number should 3 and 2"
+grep -E  'EVENT_SA_EXPIRE|EVENT_SA_REPLACE' OUTPUT/road.pluto.log | grep '#' | sed -e 's/ timeout in [1-9][^ ]* / timeout in N.N /'
+: "re-authenticateded. The state number should 3 and 2"
 ipsec status
 echo done
