@@ -11,12 +11,13 @@ ip route add 192.0.210.0/24 via 192.1.2.23  dev eth1
 ip route add 192.0.211.0/24 via 192.1.2.23  dev eth1
 
 # ensure that clear text does not get through
-iptables -A INPUT -i eth1 -s 192.0.2.0/24 -j LOGDROP
-iptables -A INPUT -i eth1 -s 192.0.200.0/24 -j LOGDROP
+iptables -A INPUT -i eth1 -s 192.0.2.0/24 -j DROP
+iptables -A INPUT -i eth1 -s 192.0.200.0/24 -j DROP
 iptables -I INPUT -m policy --dir in --pol ipsec -j ACCEPT
 ipsec start
 /testing/pluto/bin/wait-until-pluto-started
-ipsec whack --impair suppress-retransmits,delete-on-retransmit
+ipsec whack --impair suppress-retransmits
+ipsec whack --impair delete-on-retransmit
 
 ipsec auto --add westnet-eastnet-ikev2
 
