@@ -1,5 +1,5 @@
 /testing/guestbin/swan-prep
 ipsec start
 /testing/pluto/bin/wait-until-pluto-started
-ipsec auto --add west-east-auto
+#ipsec auto --add west-east-auto
 echo "initdone"
