@@ -1,7 +1,7 @@
 /testing/guestbin/swan-prep
 domainname testing.libreswan.org
 ipsec setup start
-ipsec whack --impair retransmits
+ipsec whack --impair suppress-retransmits
 /testing/pluto/bin/wait-until-pluto-started
 ipsec auto --add gssapi
 # see if KDC is up by getting a ticket
