@@ -6,7 +6,5 @@ rm -f ipsec-test-module.mod ipsec-test-module.pp
 ipsec start
 /testing/pluto/bin/wait-until-pluto-started
 ipsec auto --add labeled
-gcc -Wall -o OUTPUT/getpeercon_server -lselinux /source/contrib/labeled-ipsec/getpeercon_server.c
-chmod 755 OUTPUT/getpeercon_server
-OUTPUT/getpeercon_server 4300 &
+ipsec getpeercon_server 4300 &
 echo "initdone"
