@@ -1,9 +1,8 @@
 grep NO_PROPOSAL_CHOSEN /tmp/pluto.log
 ipsec whack --shutdown
-grep leak /tmp/pluto.log
+grep -e leak /tmp/pluto.log | grep -v -e '|'
 : ==== cut ====
 ipsec auto --status
 : ==== tuc ====
 ../bin/check-for-core.sh
 if [ -f /sbin/ausearch ]; then ausearch -r -m avc -ts recent ; fi
-: ==== end ====
