@@ -1,8 +1,8 @@
-ip xfrm state
 ipsec whack --trafficstatus
+ip xfrm state
+ip xfrm policy
 : ==== cut ====
 ipsec auto --status
 : ==== tuc ====
 ../bin/check-for-core.sh
 if [ -f /sbin/ausearch ]; then ausearch -r -m avc -ts recent ; fi
-: ==== end ====
