@@ -1,6 +1,6 @@
 /testing/guestbin/swan-prep
 ipsec start
 /testing/pluto/bin/wait-until-pluto-started
-ipsec whack --impair add-unknown-payload-to-auth
+ipsec whack --impair add-unknown-v2-payload-to:IKE_AUTH
 ipsec auto --add westnet-eastnet-ipv4-psk-ikev2
 echo "initdone"
