@@ -1,9 +1,8 @@
 # we should see a connection switch on east
 ipsec whack --trafficstatus
-hostname | grep east && grep switched /tmp/pluto.log
+hostname | grep east && grep '^[^|].* switched ' /tmp/pluto.log
 : ==== cut ====
 ipsec auto --status
 : ==== tuc ====
 ../bin/check-for-core.sh
 if [ -f /sbin/ausearch ]; then ausearch -r -m avc -ts recent ; fi
-: ==== end ====
