@@ -1,5 +1,5 @@
-ipsec auto --up  westnet-eastnet-ikev2
+ipsec auto --up westnet-eastnet-ikev2
 ping -n -c4 -I 192.0.1.254 192.0.2.254
-ipsec look
+../../pluto/bin/ipsec-look.sh
 grep "fragment number" /tmp/pluto.log
 echo done
