@@ -1,3 +1,3 @@
-strongswan up client
+strongswan up client | grep -v resolv
 ping -n -c 4 -I 192.0.1.254 192.0.2.254
 echo done
