@@ -3,7 +3,7 @@ ping -W 1 -q -n -c 2 192.0.2.254
 ipsec whack --trafficstatus
 # note this end should be 192.1.3.209
 ip xfrm state
-ip xfrm pol
+ip xfrm policy
 sleep 15
 # remove this end ip next one will take over
 ip route show scope global | grep 192.1.3.254 && ip route del default via 192.1.3.254
