@@ -4,10 +4,11 @@ ping6 -n -c 4 2001:db8:1:2::23
 # make sure that clear text echo does not get through
 ip6tables -A INPUT -i eth0 -s 2001:db8:1:2::23 -p ipv6-icmp --icmpv6-type echo-reply  -j DROP
 ip6tables -I INPUT -m policy --dir in --pol ipsec -j ACCEPT
-# confirm with a ping
+# confirm clear text does not get through
 ping6 -n -c 4 2001:db8:1:2::23
 ipsec start
 /testing/pluto/bin/wait-until-pluto-started
 ipsec auto --add v6-tunnel-east-road
 ipsec auto --status | grep v6-tunnel-east-road
+ipsec whack --impair suppress-retransmits
 echo "initdone"
