@@ -1,4 +1,4 @@
-/testing/guestbin/swan-prep --x509
+/testing/guestbin/swan-prep --x509 --x509name semiroad
 certutil -D -n east -d sql:/etc/ipsec.d
 cp road-ikev2-oe.conf /etc/ipsec.d/ikev2-oe.conf
 cp policies/* /etc/ipsec.d/policies/
@@ -6,8 +6,6 @@ echo "192.1.2.23/32"  >> /etc/ipsec.d/policies/private-or-clear
 ipsec start
 /testing/pluto/bin/wait-until-pluto-started
 ipsec whack --impair suppress-retransmits
-# ensure for tests acquires expire before our failureshunt=2m
-echo 30 > /proc/sys/net/core/xfrm_acq_expires
 # give OE policies time to load
 sleep 5
 ipsec status | grep "our auth" | grep private
