@@ -1,8 +1,7 @@
 # Authentication should be AUTH_NULL
-hostname | grep nic > /dev/null || grep Authenticated /tmp/pluto.log
+hostname | grep nic > /dev/null || grep authenticated /tmp/pluto.log
 : ==== cut ====
 ipsec auto --status
 : ==== tuc ====
 ../bin/check-for-core.sh
 if [ -f /sbin/ausearch ]; then ausearch -r -m avc -ts recent ; fi
-: ==== end ====
