@@ -1,6 +1,6 @@
 /testing/guestbin/swan-prep
 cp policies/* /etc/ipsec.d/policies/
-ping -n -c 4 -I 192.1.3.209 7.7.7.7
+../../pluto/bin/ping-once.sh --up -I 192.1.3.209 7.7.7.7
 ipsec start
 # ensure for tests acquires expire before our failureshunt=2m
 echo 30 > /proc/sys/net/core/xfrm_acq_expires
