@@ -1,7 +1,6 @@
 # first ping will be lost in triggering OE
-ping -n -c 4 -I 192.1.3.209 192.1.2.23
-sleep 1
-ipsec status | grep "STATE_" 
+../../pluto/bin/ping-once.sh --down -I 192.1.3.209 192.1.2.23
+ ../../pluto/bin/wait-for.sh --match private-or-clear -- ipsec whack --trafficstatus
 # issue a manual rekey of parent
 ipsec whack --rekey-ike --name 1
 # wait two seconds to ensure old parent has expired
