@@ -1,4 +1,7 @@
 /testing/guestbin/swan-prep
+# prevent stray DNS packets hitting OE - DNS not used on east in this test
+rm /etc/resolv.conf
+touch /etc/resolv.conf
 cp policies/* /etc/ipsec.d/policies/
 echo "192.1.2.0/24"  >> /etc/ipsec.d/policies/private-or-clear
 echo "192.1.3.0/24"  >> /etc/ipsec.d/policies/private-or-clear
