@@ -4,6 +4,8 @@ cp ikev2-oe.conf /etc/ipsec.d/ikev2-oe.conf
 echo "192.1.2.0/24"  >> /etc/ipsec.d/policies/private-or-clear
 ipsec start
 /testing/pluto/bin/wait-until-pluto-started
-ipsec whack --debug-all --impair retransmits
+ipsec whack --impair suppress-retransmits
+# give OE policies time to load
+sleep 5
 ipsec auto --status
 echo "initdone"
