@@ -4,7 +4,7 @@ cp policies/* /etc/ipsec.d/policies/
 echo "192.1.2.23/32"  >> /etc/ipsec.d/policies/private-or-clear
 ipsec start
 /testing/pluto/bin/wait-until-pluto-started
-ipsec whack --debug-all --impair retransmits
+ipsec whack --impair suppress-retransmits
 # ensure for tests acquires expire before our failureshunt=2m
 echo 30 > /proc/sys/net/core/xfrm_acq_expires
 # give OE policies time to load
