@@ -1,5 +1,4 @@
 # only expected to show failure on west
-grep "Certificate rejected" /tmp/pluto.log
+grep "certificate payload rejected" /tmp/pluto.log
 ../bin/check-for-core.sh
 if [ -f /sbin/ausearch ]; then ausearch -r -m avc -ts recent ; fi
-: ==== end ====
