@@ -1,7 +1,6 @@
-hostname | grep east > /dev/null && grep switched /tmp/pluto.log
+hostname | grep east > /dev/null && grep "switched from" /tmp/pluto.log
 : ==== cut ====
 ipsec auto --status
 : ==== tuc ====
 ../bin/check-for-core.sh
 if [ -f /sbin/ausearch ]; then ausearch -r -m avc -ts recent ; fi
-: ==== end ====
