@@ -1,11 +1,6 @@
-: ==== start ====
-TESTNAME=whackrecord-01
-source /testing/pluto/bin/eastlocal.sh
-
+/testing/guestbin/swan-prep
 ipsec start
 /testing/pluto/bin/wait-until-pluto-started
-
 ipsec whack --whackrecord /var/tmp/east.record
 ipsec auto --add westnet-eastnet
 ipsec whack --debug none --debug control --debug controlmore --debug crypt
-
