@@ -1,3 +1,3 @@
-ipsec whack --debug-all --impair drop-xauth-r0
+ipsec whack --impair drop-xauth-r0
 ipsec whack --xauthname 'use3' --xauthpass 'use1pass' --name road-east --initiate
 echo done
