@@ -1,7 +1,6 @@
-egrep '(inserting|handling) event (EVENT_v1_SEND_XAUTH|EVENT_RETRANSMIT)' OUTPUT/east.pluto.log
+grep -E '(inserting|handling) event (EVENT_v1_SEND_XAUTH|EVENT_RETRANSMIT)' OUTPUT/east.pluto.log
 : ==== cut ====
 ipsec auto --status
 : ==== tuc ====
 ../bin/check-for-core.sh
 if [ -f /sbin/ausearch ]; then ausearch -r -m avc -ts recent ; fi
-: ==== end ====
