@@ -4,6 +4,6 @@ ipsec start
 ../bin/block-non-ipsec.sh
 ipsec auto --add north-east
 ipsec whack --xauthname 'use1' --xauthpass 'use1pass' --name north-east --initiate
-ping -n -c 4 -I 192.0.2.101 192.0.2.254
+../../pluto/bin/ping-once.sh --up -I 192.0.2.101 192.0.2.254
 ipsec whack --trafficstatus
 echo initdone
