@@ -17,7 +17,7 @@ import re
 import shutil
 import functools
 import copy
-import bz2
+import gzip
 
 from collections import defaultdict
 from datetime import datetime
@@ -52,17 +52,17 @@ def log_arguments(logger, args):
     logger.info("  publish-hash: '%s'", args.publish_hash)
 
 
-results_to_print = printer.Print(printer.Print.test_name,
-                                 printer.Print.test_kind,
-                                 printer.Print.test_status,
-                                 printer.Print.test_host_names,
-                                 printer.Print.start_time,
-                                 printer.Print.end_time,
-                                 printer.Print.result,
-                                 printer.Print.issues,
-                                 printer.Print.runtime,
-                                 printer.Print.boot_time,
-                                 printer.Print.script_time)
+results_to_print = printer.Print(printer.Print.TEST_NAME,
+                                 printer.Print.TEST_KIND,
+                                 printer.Print.TEST_STATUS,
+                                 printer.Print.TEST_HOST_NAMES,
+                                 printer.Print.START_TIME,
+                                 printer.Print.STOP_TIME,
+                                 printer.Print.RESULT,
+                                 printer.Print.ISSUES,
+                                 printer.Print.RUNTIME,
+                                 printer.Print.BOOT_TIME,
+                                 printer.Print.SCRIPT_TIME)
 
 JSON_RESULTS = []
 JSON_SUMMARY = { }
@@ -152,8 +152,9 @@ def test_output_files(logger, args, result):
             logger.info("copying '%s' to '%s'", src, dst)
             shutil.copyfile(src, dst)
             continue
-        # copy compressed files
-        dst = dst + ".bz2"
+        # copy compressed files; gzip is used as that works with the
+        # web's deflate?!?
+        dst = dst + ".gz"
         if os.path.isfile(dst) \
         and os.path.getmtime(src) < os.path.getmtime(dst):
             continue
@@ -161,7 +162,7 @@ def test_output_files(logger, args, result):
             logger.info("compressing '%s' to '%s'", src, dst)
             with open(src, "rb") as f:
                 data = f.read()
-            with bz2.open(dst, "wb") as f:
+            with gzip.open(dst, "wb") as f:
                 f.write(data)
             continue
 
@@ -199,7 +200,7 @@ def json_result(logger, args, result):
             _add(JSON_SUMMARY, "errors", issue)
     # extend the times
     _update_time(min, "start_time", json_result)
-    _update_time(max, "end_time", json_result)
+    _update_time(max, "stop_time", json_result)
 
 
 def json_results(logger, args):
@@ -217,10 +218,11 @@ def json_summary(logger, args):
         return
     # times
     start_time = JSON_SUMMARY.get("start_time")
-    end_time = JSON_SUMMARY.get("end_time")
-    if start_time and end_time:
-        runtime = (end_time - start_time).total_seconds()
-        JSON_SUMMARY["runtime"] = datetime.utcfromtimestamp(runtime).strftime("%H:%M")
+    stop_time = JSON_SUMMARY.get("stop_time")
+    if start_time and stop_time:
+        # in minutes
+        runtime = round((stop_time - start_time).total_seconds() / 60.0)
+        JSON_SUMMARY["runtime"] = "%d:%02d" % (runtime / 60, runtime % 60)
     # other stuff
     JSON_SUMMARY["total"] = len(JSON_RESULTS)
     if args.publish_hash:
